local data = {rangesort = {harm = {}, help = {}}, usedSpell = {harm = {}, help = {}}, minspell = {}, minspellrange = {}, maxspellrange = {}, melee = nil }
local distance = {28, 10, 8}
local IsSpellInRange = IsSpellInRange
local CheckInteractDistance = CheckInteractDistance

local function CheckMinRange(self, unitdead)
	local dist2, dist3 = CheckInteractDistance(self.unit, 2), CheckInteractDistance(self.unit, 3)
	if dist2 and not dist3 then
		return distance[3], distance[2]
	elseif dist3 then
		local targetType = self.targetType
		if unitdead then
			return 0, distance[3]
		end
		local dist
		if targetType and targetType == "harm" and data.melee then
			if IsSpellInRange(data.melee, self.unit) == 1 then
				return 0, 5
			else
				dist = 5
			end
		end
		if targetType and data.minspellrange[targetType] then
			local spell = data.minspell[targetType]
			if spell then
				local ok = true
				if T_RangeSpellCompare[spell] then
					ok = T_RangeSpellCompare[spell](self)
				end
				if ok then
					if IsSpellInRange(spell, self.unit) == 0 then
						return 0, data.minspellrange[targetType]
					else
						return data.minspellrange[targetType],  distance[3]
					end
				end
			end
		end
		return dist or 0, distance[3]
	end
end

local function CheckMaxRange(self, unitdead)
	local distmin, distmax, spellmin, spellmax, lastIndex
	local targetType = self.targetType
	local dist1 = CheckInteractDistance(self.unit, 1)
	if not targetType or not next(data.usedSpell[targetType]) then
		if dist1 then
			return distance[2], distance[1]
		else
			return nil, distance[1]
		end
	end

	local prevRange = 0
	if dist1 then
		distmax = distance[1]
		distmin = distance[CheckInteractDistance(self.unit, 3) and 3 or 2]
	end
	if unitdead or not targetType then
		if distmax then
			return distmin, distmax
		else
			return nil, distance[1]
		end
	else
		for i = 1, #data.rangesort[targetType] do
			local range = data.rangesort[targetType][i]
			local spell = data.usedSpell[targetType][range]
			local ok = true
			if T_RangeSpellCompare[spell] then
				ok = T_RangeSpellCompare[spell](self)
			end
			if ok then
				if IsSpellInRange(spell, self.unit) == 1 then
					if distmax then
						return max(prevRange, distmin), min(distmax, range)
					else
						return max(prevRange, distance[1]), range
					end
				else
					lastIndex = i
					prevRange = range
				end
			end
		end
	end

	if distmin then return max(distmin, prevRange), distmax end

	local r = data.maxspellrange[targetType]
	local spell = r and data.usedSpell[targetType][r]
	if spell and T_RangeSpellCompare[spell] then
		if T_RangeSpellCompare[spell](self) then
			return nil, data.maxspellrange[targetType]
		else
			return nil, max(data.rangesort[targetType][lastIndex] or 0, distance[1])
		end
	end
	return nil, max(r or 0, distance[1])
end
local function deltable(tbl)
	for unittype, v in pairs(tbl) do
		for name in pairs(v) do
			tbl[unittype][name] = nil
		end
	end
end
local function CopySpellRange(self)
	local s = 0
	deltable(self.rangesort)
	deltable(self.usedSpell)
	for _,spell in pairs(T_RangeSpell) do
		local minrange, maxrange = select(8, GetSpellInfo(spell))
		minrange = minrange and floor(minrange+0.5)
		maxrange = maxrange and floor(maxrange+0.5)
		for ut in pairs({harm = IsHarmfulSpell(spell), help = IsHelpfulSpell(spell)}) do
			self.maxspellrange[ut] = self.maxspellrange[ut] or 0
			if minrange and minrange > 0 and minrange < distance[3] then
				if not self.minspellrange[ut] or self.minspellrange[ut] > minrange then
					self.minspellrange[ut] = minrange
					self.minspell[ut] = spell
				end
			end
			if maxrange and maxrange > 0 then
				self.maxspellrange[ut] = math.max(self.maxspellrange[ut], maxrange)
				if not self.usedSpell[ut][maxrange] then
					table.insert(self.rangesort[ut], maxrange)
					self.usedSpell[ut][maxrange] = spell
				end
			end
		end
	end
	table.sort(self.rangesort.harm)
	table.sort(self.rangesort.help)
	self.melee = T_RangeMelee and GetSpellInfo(T_RangeMelee)
end
local function GetUnitType(unit)
	return UnitCanAssist("player", unit) and "help" or UnitCanAttack("player", unit) and "harm"
end

local function OnEvent(self, event, ...)
	if UnitExists(self.unit) and not UnitIsUnit(self.unit, "player") then
		self.targetType = GetUnitType(self.unit)
		self.nextUpdate = 1
		self:Show()
		if self.UnitCheck then
			self:UnitCheck(self.unit)
		end
	else
		self:Hide()
	end
end
local function OnUpdate(self, elasped)
	self.nextUpdate = self.nextUpdate + elasped
	if self.nextUpdate <= 0.2 then
		return
	end
	self.nextUpdate = 0

	if not UnitExists(self.unit) then
		self:Hide()
		return
	end

	local maxrange, minrange
	local unitdead = UnitIsDead(self.unit)

	if self.UnitUpdate then
		local guid = UnitGUID(self.unit)
		if self.guid ~= guid then
			self.guid = guid
			self:UnitUpdate()
		end
	end

	minrange, maxrange = CheckMinRange(self, unitdead)
	if minrange then
		if self.minrange ~= minrange or self.maxrange ~= maxrange then
			self.minrange = minrange
			self.maxrange = maxrange
			if self.MinRange then self:MinRange() end
		end
		return
	end

	minrange, maxrange = CheckMaxRange(self, unitdead)
	if self.minrange ~= minrange or self.maxrange ~= maxrange then
		self.minrange = minrange
		self.maxrange = maxrange
		if self.MaxRange then self:MaxRange() end
	end
end

local unitcount = {}

local function new(self, unit, minfunc, maxfunc, unitupdate, unitcheck)
	unit = unit:lower()
	unitcount[unit] = (unitcount[unit] or 0) + 1
	local name = ("T_Range%sFrame%d"):format(unit:gsub("^.", function(x) return x:upper() end), unitcount[unit])
	local frame = CreateFrame("FRAME", name, UIParent)
	frame:Hide()
	frame.unit = unit
	if unit == "target" then
		frame:RegisterEvent("PLAYER_TARGET_CHANGED")
	elseif unit == "focus" then
		frame:RegisterEvent("PLAYER_FOCUS_CHANGED")
	end
	frame:SetScript("OnEvent", OnEvent)
	frame:SetScript("OnUpdate", OnUpdate)
	frame.MinRange = minfunc
	frame.MaxRange = maxfunc
	frame.UnitUpdate = unitupdate
	frame.UnitCheck = unitcheck
	return frame
end
T_Range = setmetatable(data, {__call = new})

local f = CreateFrame("frame", nil, UIParent)
f:RegisterEvent("LEARNED_SPELL_IN_TAB")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:SetScript("OnEvent", function(self, event)
	if event == "PLAYER_ENTERING_WORLD" then
		self.entering = 0
		self:UnregisterEvent(event)
	end
	CopySpellRange(T_Range)
end)
f:SetScript("OnUpdate", function(self, elapsed)
	if self.entering then
		self.entering = self.entering + elapsed
		if self.entering > 1 then
			self.entering = nil
			CopySpellRange(T_Range)
			self:SetScript("OnUpdate", nil)
		end
	end
end)

--T_Cmd("TRange", "trload", "ㅅㄱ", function() CopySpellRange(T_Range) end)

SLASH_TRANGE1, SlashCmdList.TRANGE = "/trload", function() CopySpellRange(T_Range) end


-- CheckInteractDistance 함수로 알수 있는 거리 1 = 30, 2 = 11.11, 3 = 9.9, 4 = 30
-- T_Range.usedSpell 자신이 배운주문의 거리를 저장할 테이블
--    -- T_Range.usedSpell[self.targetType][거리] = 주문명
-- T_Range.minspell[self.targetType] = 최소거리 확인용 주문
-- T_Range.maxspellrange[self.targetType] = 최대거리
-- T_Range.minspellrange[self.targetType] = 최소거리

-- self.minrange 최소거리, self.maxrange 최대거리
-- self.minrange 값이 없으면 최대거리 초과
-- self.targetType 대상이 공격이 가능하면 "harm" 도움을 줄수 있다면 "help" 알 수 없다면 nil
-- T_Range("unit", 최소거리에 사용 할 함수 0~9사이, 최소거리가 아니면 사용 할 함수, 대상이 바뀌었을때 사용 할 함수(1회실행), 대상을 검사할 함수)
--     self.MinRange = 최소거리에 사용 할 함수 0~9사이
--     self.MaxRange = 최소거리가 아니면 사용 할 함수
--     self.UnitUpdate = 대상이 바뀌었을때 사용 할 함수(1회실행)
--     self.UnitCheck = 대상을 검사할 함수 arg값 self.unit
-- unit = "target", "focus" 두개... 그 외에는 필요없다고 생각함