local _G = getfenv(0)
local pairs = pairs
local SetItemButtonDesaturated = SetItemButtonDesaturated

local GetBagPosition = function(size, position)
	if(not position or position == 0) then return nil end
	if(position > 0) then
		return position
	else
		return size+position+1
	end
end

function cargBags:CreateSlots()
	local slotid = 0
	for _, bag in pairs(self.Bags) do
		bag.bagType = select(2, self:Handler("GetContainerNumFreeSlots", bag.id))
		if(not bag.Hidden and bag.size > 0) then
			for k = (GetBagPosition(bag.size, bag.start) or 1), (GetBagPosition(bag.size, bag.finish) or bag.size) do
				slotid = slotid+1
				local button
				if(self.Container[slotid]) then
					button = self.Container[slotid]
				else
					if(bag.id == BANK_CONTAINER) then
						button = CreateFrame("Button", self.Name.."Slot"..slotid, bag, "BankItemButtonGenericTemplate")
					else
						button = CreateFrame("Button", self.Name.."Slot"..slotid, bag, "ContainerFrameItemButtonTemplate")
					end
					button.name = self.Name.."Slot"..slotid
					self.Container[slotid] = button
				end
				button:SetWidth(self.SlotWidth)
				button:SetHeight(self.SlotHeight)
				button:SetParent(bag)
				button:SetID(k)
				button.id = k
				if(button.bag) then button.bag[button] = nil end
				button.bag = bag
				button.bag[button] = true
				button:Show()

				if(self.PostCreateButton) then self:PostCreateButton(button, bag) end
			end
		end
	end
	if(self.Container.num > slotid) then
		for i=slotid+1, self.Container.num do self.Container[i]:Hide() end
	end
	self.Container.num = slotid

	local col, row = 0, 0
	local cols = math.floor(self.Container:GetWidth() / (self.SlotWidth + self.SlotSpacing))-1
	local xgrowth = (string.find(self.SlotAnchor,"RIGHT") and -1) or 1
	local ygrowth = (string.find(self.SlotAnchor,"TOP") and -1) or 1
	for i=1, slotid do
		button = self.Container[i]
		button:ClearAllPoints()
		button:SetPoint(self.SlotAnchor, self.Container, self.SlotAnchor, xgrowth * col * (self.SlotWidth + self.SlotSpacing), ygrowth * row * (self.SlotHeight + self.SlotSpacing))
		if(col >= cols) then
			col = 0
			row = row + 1
		else
			col = col + 1
		end
	end
	self.Container:SetHeight((row + (col>0 and 1 or 0)) * (self.SlotHeight + self.SlotSpacing))
	if(self.UpdateDimensions) then self:UpdateDimensions() end
end

function cargBags:CheckBags()
	local needUpdate
	for _, bag in pairs(self.Bags) do
		if(bag.id == KEYRING_CONTAINER) then
			size = self:Handler("GetKeyRingSize")
		else
			size = self:Handler("GetContainerNumSlots", bag.id)
		end
		if(not bag.size or bag.size ~= size) then
			bag.size = size
			needUpdate = true
		end
	end
	return needUpdate
end

function cargBags:UpdateSlots(event, bagID, updateNeeded)
	local texture, count, locked, link, button, cooldown, start, finish
	if(self:CheckBags() or updateNeeded) then self:CreateSlots() end
	self.Container.used = 0
	if(not bagID or not self.bagIDs[bagID]) then
		for i=1, self.Container.num do
			if(event == "ITEM_LOCK_CHANGED") then
				self:UpdateItem(self.Container[i])
			elseif(event == "BAG_UPDATE_COOLDOWN") then
				self:UpdateCooldown(self.Container[i])
			else
				self:UpdateItem(self.Container[i])
				self:UpdateCooldown(self.Container[i])
			end
		end
	else
		for button, _ in pairs(self.bagIDs[bagID]) do
			if(type(button) == "table") then
				if(event == "ITEM_LOCK_CHANGED") then
					self:UpdateItem(button)
				elseif(event == "BAG_UPDATE_COOLDOWN") then
					self:UpdateCooldown(button)
				else
					self:UpdateItem(button)
					self:UpdateCooldown(button)
				end
			end
		end
	end
end

function cargBags:UpdateItem(button)
	local texture, count, locked = self:Handler("GetContainerItemInfo", button.bag.id, button.id)
	_G[button.name.."IconTexture"]:SetTexture(texture)
	SetItemButtonCount(button, count)
	local link = self:Handler("GetContainerItemLink", button.bag.id, button.id)

	if(oGlow and not self.NoGlow and link) then
		oGlow(button, select(3, GetItemInfo(link)))
	elseif(button.bc) then
		button.bc:Hide()
	end

	SetItemButtonDesaturated(button, locked, 0.5, 0.5, 0.5)
end

function cargBags:UpdateCooldown(button)
	local start, finish, enable = self:Handler("GetContainerItemCooldown", button.bag.id, button.id)
	local cooldown = _G[button.name.."Cooldown"]
	if(cooldown) then CooldownFrame_SetTimer(cooldown, start, finish, enable) end
end

cargBags.bagUpdaters["Container"] = cargBags.UpdateSlots
cargBags.ITEM_LOCK_CHANGED = cargBags.UpdateSlots
cargBags.BAG_UPDATE_COOLDOWN = cargBags.UpdateSlots