﻿-- The function which inserts the free/used space of the object
local cb_Scale = 0.9

local UpdateSpace = function(self, free, max)
	self.Space:SetText("여유공간 "..free.."/"..max)
end

-- Function is called after a button was created
-- We color the borders of the button to see if it is an ammo bag or else
local PostCreateButton = function(self, button, bag)
	if(bag.id == KEYRING_CONTAINER) then
		_G[button.name.."NormalTexture"]:SetVertexColor(1, 0.7, 0)
	elseif(bag.bagType and bag.bagType > 0 and bag.bagType < 8) then
		_G[button.name.."NormalTexture"]:SetVertexColor(1, 1, 0)
	elseif(bag.bagType and bag.bagType > 4) then
		_G[button.name.."NormalTexture"]:SetVertexColor(0, 1, 0)
	else
		_G[button.name.."NormalTexture"]:SetVertexColor(1, 1, 1)
	end
end

-- More slot buttons -> more space!
local UpdateDimensions = function(self)
	local height
	if(self.BagButtons and self.BagButtons:IsShown()) then
		height = 53				-- Bag button space
	else
		height = 10				-- Normal margin space
	end
	if(self.Space or self.Money) then
		height = height + 20	-- additional info display space
	end
	self:SetHeight(self.Container:GetHeight() + height)
end

-- Style of the bag and its contents
local func = function(settings, self, type)
	self:EnableMouse(true)

	self.SlotSpacing = 1	-- The spacing between the item buttons

	self.UpdateDimensions = UpdateDimensions	-- Called when the size of the bag changes
	
	self:SetFrameStrata("HIGH")
	tinsert(UISpecialFrames, self:GetName()) -- Close on "Esc"
	
	-- Make main frames movable
	if(self.Name == "cb_main" or self.Name == "cb_bank") then
		self:SetMovable(true)
		self:RegisterForClicks("LeftButton", "RightButton");
	    --[[self:SetScript("OnMouseDown", function() 
	        if(IsAltKeyDown()) then 
	            self:ClearAllPoints() 
	            self:StartMoving() 
	        end 
	    end)]]--
		self:SetScript("OnMouseDown", function() self:StartMoving() end)
		self:SetScript("OnMouseUp", function() self:StopMovingOrSizing() end)
	end
	
	self:SetBackdrop{
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	}
	self:SetBackdropColor(0, 0, 0, 1)
	self:SetBackdropBorderColor(1, 1, 1, 0.5)

	-- This is the container which contains the item slots
	local container = CreateFrame("Frame", self.Name.."Cont", self)
	container:SetPoint("TOPLEFT", 6, -6)
	self.Container = container
	self.PostCreateButton = PostCreateButton	-- Called when a button is created

	-- The number of columns is calculated by the width of the container-frame,
	-- so a new column starts if the next slot added would be outside the frame
	if(self.Name == "cb_keyring") then
		container:SetWidth((self.SlotWidth+self.SlotSpacing)*2) -- This is a fix which calculates the width of the slots and multiplies them with 2 --->  2 columns for keyring
		self:SetScale(0.72)	-- Make key ring a bit smaller
	elseif(self.Name == "cb_bank") then
		container:SetWidth((self.SlotWidth+self.SlotSpacing)*12) -- 12 columns for the bank as you can see
		self:SetScale(cb_Scale)
	else
		self:SetScale(cb_Scale)
		container:SetWidth((self.SlotWidth+self.SlotSpacing)*8)  -- and here are 8 for the main bags
	end
	self:SetWidth(container:GetWidth() + 10)	-- 10 px margin

	if(self.Name == "cb_main" or self.Name == "cb_bank") then
		local money = CreateFrame("Frame", self.Name.."Money", self, "SmallMoneyFrameTemplate")	-- The frame for money display
		money:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 7)
		self.Money = money

		local space = self:CreateFontString(nil, "OVERLAY")	-- The font string for bag space display
		space:SetPoint("BOTTOMLEFT", self, 6, 7)
		space:SetJustifyH"LEFT"
		space:SetFontObject(GameFontNormal)
		space:SetTextColor(1, 1, 1)
		space.countAll = true -- Count the number of space in all included bags, not only the visible slots
		self.Space = space
		self.UpdateSpace = UpdateSpace -- link update function
		
		local bagButtons = CreateFrame("Frame", self.Name.."BagButtons", self) -- A nice bag bar for changing/toggling bags
		bagButtons:SetPoint("TOPLEFT", self.Container, "BOTTOMLEFT", 0, -5)
		bagButtons:SetWidth(38*4)
		bagButtons:SetHeight(1)
		bagButtons:Hide()
		self.BagButtons = bagButtons
		
		-- A little fix that positions the bagToggle between space and money
		local spacer = CreateFrame("Frame", nil, self)
		spacer:SetPoint("BOTTOMLEFT", space, "BOTTOMRIGHT", 0, 0)
		spacer:SetPoint("RIGHT", money, "LEFT", 0, 0)
		self.Spacer = spacer
		
		local bagToggle = CreateFrame("CheckButton", nil, self) -- We don't need the bag bar every time, so let's create a toggle button for them to show
		bagToggle:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
		bagToggle:SetWidth(40)
		bagToggle:SetHeight(12)
		bagToggle:SetPoint("CENTER", spacer)
		bagToggle:RegisterForClicks("LeftButtonUp")
		bagToggle:SetScript("OnClick", function()
			if(self.BagButtons:IsShown()) then
				self.BagButtons:Hide()
			else
				self.BagButtons:Show()
			end
			self:UpdateDimensions()	-- The bag buttons take space, so let's update the height of the frame
		end)
		local bagToggleText = bagToggle:CreateFontString(nil, "OVERLAY")
		bagToggleText:SetPoint("CENTER", bagToggle)
		bagToggleText:SetFontObject(GameFontNormalSmall)
		bagToggleText:SetText("가방 보기")
	end

	if(self.Name == "cb_main") then
		local keytoggle = CreateFrame("CheckButton", nil, self.BagButtons) -- main window gets a fake bag button for toggling key ring
		keytoggle:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight", "ADD")
		keytoggle:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
		keytoggle:SetWidth(18)
		keytoggle:SetHeight(37)
		keytoggle:SetPoint("TOPLEFT", self.BagButtons, "TOPRIGHT", 0, 0)
		self.keyToggle = keytoggle
		
		local keyTexture = keytoggle:CreateTexture()
		keyTexture:SetTexture('Interface/Buttons/UI-Button-KeyRing')
		keyTexture:SetAllPoints(keytoggle)
		keyTexture:SetTexCoord(0, 0.5625, 0, 0.609375)
	
		keytoggle:RegisterForClicks("LeftButtonUp")
		keytoggle:SetScript("OnClick", function()
			if(cb_keyring:IsShown()) then
				cb_keyring:Hide()
				keytoggle:SetChecked(0)
			else
				cb_keyring:Show()
				keytoggle:SetChecked(1)
			end
		end)
	end
	
	if(self.Name == "cb_bank") then -- For purchasing bank slots
		local purchase = CreateFrame("Button", nil, self.BagButtons, "UIPanelButtonTemplate")
		purchase:SetText(BANKSLOTPURCHASE)
		purchase:SetWidth(121)
		purchase:SetHeight(21)
		purchase:SetPoint("TOPRIGHT", self.Container, "BOTTOMRIGHT", 0, -30)
		purchase:SetScript("OnClick", function() StaticPopup_Show("CONFIRM_BUY_BANK_SLOT") end)
		self.PurchaseButton = purchase
		
		local cost = CreateFrame("Button", self.Name.."PurchaseCost", self.BagButtons, "SmallMoneyFrameTemplate")
		cost:SetPoint("TOPRIGHT", self.Container, "BOTTOMRIGHT", 0, -10)
		self.PurchaseCost = cost
	end

	return self
end

cargBags:RegisterStyle("Pernobilis", setmetatable({}, {__call = func}))

-- cargBags:Spawn(	name,		Table of Bags	)
local main 	= cargBags:Spawn("cb_main", {"0-4"})		-- Bagpack (0) and bags (1 to 4)
local key = cargBags:Spawn("cb_keyring", {-2})			-- Keyring (-2)
local bank = cargBags:Spawn("cb_bank", {-1, "5-11"})	-- Bank frame (-1) and bank bags (5 to 11)

--[[
	Closer look at the bag-arguments:
	-------------------------------------------
	Splitting bags:
		A bag can be defined by this scheme:	[BagID]:[StartSlot]:[EndSlot]
		When the Slots are not defined or 0, the standard is used (Standard StartSlot: First slot in the bag	-	Standard EndSlot: Last slot in the bag)
		A positive number is the slot counted from the first slot on - So 1 matches the first slot
		A negative number starts at the end - So -1 matches the last slot

		The negative numbers are especially important when you want to split a bag on multiple objects.
		Without them there would not been the accurate number of slots in the bag shown if the bag-sizes varies (switching bags / characters and so on)
	
	Merging bags:
		You can use a minus for adding multiple bags at once	-	for example:   "5-11" for all bank bags
	
	Examples:
	-------------
	cargBags:Spawn("cb_main", {"0:9", "1-4"})	-- Adding slots 9 to end of backpack and rest of inventory bags
	cargBags:Spawn("cb_second", {"0:0:8"})	-- Adding slots from start to 8 of backpack
]]

-- Positioning
main:SetPoint("RIGHT", -5, 0)
key:SetPoint("TOPRIGHT", main, "TOPLEFT", -5, 0)
bank:SetPoint("LEFT", 5, 0)

-- Opening / Closing Functions
function OpenCargBags()
	main:Show()
end

function CloseCargBags()
	main:Hide()
	key:Hide()
	main.keyToggle:SetChecked(0)
	bank:Hide()
end

function ToggleCargBags(forceopen)
	if(main:IsShown() and not forceopen) then CloseCargBags() else OpenCargBags() end
end

-- Blizzard Replacement Functions
ToggleBackpack = ToggleCargBags
OpenAllBags = ToggleCargBags
CloseAllBags = CloseCargBags
OpenBackpack = OpenCargBags
CloseBackpack = CloseCargBags