-----------------------------
-- 한글화: 드로아크        --
-- 버그픽스/기능보완: 화수 --
-----------------------------
--[[------------------------------------------
MinnaStats

Author:
Robert John Shepherd (robertj@torturedsouls.com)
aka Vampira/Maila/Nurmi/Iamabush of Head Hunters
Give us a /wave on Outland!

Type /mstats help for help

Site: www.hhguild.co.uk

Version 1.92

    Frame:SetMaxResize(maxWidth, maxHeight) - Set the maximum dimensions this frame can be resized to.
    Frame:SetMinResize(minWidth, minHeight) - Set the minimum dimensions this frame can be resized to.
    Frame:SetMovable(isMovable) - Set whether the frame can be moved.
    Frame:SetResizable(isResizable) - Set whether the frame can be resized.

Type this once per account (works across all characters on that account) to extend your combatlog range:
/console SET CombatLogRangePartyPet "150"
/console SET CombatLogRangeFriendlyPlayers "150"
/console SET CombatLogRangeFriendlyPlayersPets "150"
/console SET CombatLogRangeHostilePlayers "150"
/console SET CombatLogRangeHostilePlayersPets "150"
/console SET CombatLogRangeCreature "150"

From this bit onwards, expect rough notes to myself about stuff I`d like to do, or remind
me how to do something. Oh and very little commenting in the code.

-- TODO --

*1. Save entire combat records for reloading at a later date
*2. Add command line option for enabling realtime non-cached stats processing
*3. Cache a list of friendly targets when you go into combat to speed up lookups
*4. Profile functions
*5. Resize window
*6. Export combined stats
*7. Convert stats lines into an external array to make it easier to add new functions
*8. Added Resisted (now is a separate stat from absorbed)
*9. Tracks multiple mobs as different targets (based on GUI)
*10. Lookup table for GUIDs with friendly/mob, wiped at the start of each new fight


-- CHANGES FROM 1.91 TO 1.92 --

1. Added lock/unlock option to MinnaButton
2. Remembered to include MinnaButton


-- CHANGES FROM 1.90 TO 1.91 --

1. Bumped TOC number to latest patch
2. Fixed menu issues


-- CHANGES FROM 1.80 TO 1.90 --

1. Changed the combatlog processing to work with patch 2.4
2. Minnastats goes realtime
3. Only damage and healing works at present


-- CHANGES FROM 1.71 TO 1.80 --

1. Added /mstats inpartyonly and /mstats notpartyonly to enable/disable stats tracking when not grouped
2. Changed class colours for Shamans/Paladins to the Blizzard defaults
3. Added "BOSS:" text before boss names
4. Added Mount Hyjal and Zul'aman boss names
5. Removed self damage as a combat timer trigger due to problems with detection of gaps in Hyjal trash
	(basically warlock life tapping reset the combat timer so Bosses often got merged with the last wave)
6. Now whispers people running out dated versions during a /mstats check
7. Added an In Combat indicator (C) to the title bar
8. Removed damage done to self from damage totals
9. Added combat time to report output
10. Added colour to fight record lines to help differentiate boss fights, from normal fights, from totals
11. No longer counts pet deaths as their owners
12. Removed various debug lines which should result in a marginal speed increase


-- CHANGES FROM 1.70 TO 1.71 --

1. Updated TOC for latest 2.1 patch
2. Added command line option for disabling overhealing tracking (in case it lags you out)


-- CHANGES FROM 1.60 TO 1.70 --

1. Moved button code out to an external mod called MinnaButton
2. Added submenus for filter options (was getting way too big for one menu)
3. Added overhealing and split healing into net and gross
NB: Due to this change this version WILL not sync with earlier versions!
NB: You WILL lose all current data due to the new stat added, sorry :/
NB: All health events are now processed in real time, so CPU load is increased but still should be way lower than any other damage meters mod
NB: To save CPU load, a player's position in a party/raid is added during their first interaction in a particular fight, this position is used to get current and maximum health values, so if you move players around in a raid (or if players leave or join a non-raid party) during a fight, this will cause innaccuracies with overhealing, but this is such a rare event I feel the trade off for saving CPU usage is well worth it
4. Added a list of all 5 man and known raid bosses to the boss fight naming table
5. Fixed a bug whereby some combatlog lines such as Dragon's breath may not be recorded properly

-- CHANGES FROM 1.51 TO 1.60 --

1. Added a few quick config menu options when you right click the Combat Record button on the main stats window
2. Added Average hit and Average Heal reports
3. Fixed error caused when someone takes a heal but with no usual source
4. Added damage/s, hits/s and heals/s (note this should be reasonably accurate for each client but is not a synced stat)

-- CHANGES FROM 1.50 TO 1.51 --

1. Slightly changed how blocked and absorbed messages are tracked to reduce chances of an error
2. Now trackes (%d resisted) values
3. Fixed a bug where pet damage wasn't being combined with yours if you were in a party (worked fine solo or in raid)

-- CHANGES FROM 1.40 TO 1.50 --

1. Fixed error caused when someone suffers damage from a cause but with no source
2. Added in broadcasting of individual player's stats
3. Right clicking F toggles between Running Total and Fight by Fight report filtering
4. Right clicking the minimize button toggles whether processing the combatlog is enabled or not (note it will always be enabled by default)

-- CHANGES FROM 1.30 TO 1.40 --

1. Clicking on a name in stats window switches to individual player view
2. Clicking on entries in new player view switches to relevant report for fight
3. Fixed a few minor scrolling issues
4. /mstats check now only spams your own replies, or replies to your own checks, with versions for users of 1.40 or later
5. Fixed error that appears when someone is tracked as drowning

-- CHANGES FROM 1.20 TO 1.30 --

1. Shamans and Paladins are now once again coloured Pink, however Horde players will see Paladins
as a Bluey Grey, whilst Alliance will see Shamans this way
2. Renews on yourself are now tracked properly as originating from you
3. Odd bug where some people upgrading ended up with a tall window now has a work around (no idea why, nothing in the code would do that)
4. Now tracks resists (seemed to have forgotten to add it before) like it tracks misses/immunes/etc

-- CHANGES FROM 1.10 TO 1.20 --

1. Fixed bug where "gains" style healing was recording source and target the wrong way around,
meaning Healing Stream totems gave Healing Done to everyone they benefitted (thanks to BlueGlyph for report)
2. Added tracking of pet damage/healing to player data, however pets may give all their damage
to the first player in the raid that shares the same pet/player name, eg. Mage elementals
(thanks Oz for letting me hang about your hunter for hours in Winterspring whilst I debugged)
3. Added /mstats pets to toggle combining pet data with player data or keeping them as separate entities
(note, if you track pets as separate entities you will NOT send or receive sync data to other players,
this is to prevent corruption of the two data sets)

-- CHANGES FROM 1.01 TO 1.10 --

1. Fixed lots of errors caused by foreach's depreciation
(very annoying, I knew it was depreciated, but thought it still worked,
it doesn't throw errors, but no longer works as expected all the time)
2. UI window should now only be visible if you had it open when last logged off
3. New background options /mstats dog and /mstats nodog
4. Added current memory usage to /mstats status
5. Added new set option FSCALE so you can scale the UI (closest I can give to a resize atm)

-- CHANGES FROM 1.0 TO 1.01 --

1. Fixed for patch 2.01

-- CHANGES FROM 0.9 TO 1.0 --

1. Replaced own qsort implimentation with inbuilt lua table sort (didn`t fix the problem I thought it would, and may have added cpu load)
2. Fixed deleting of fight history and all fights to protect Totals
3. Added an Alpha setting (/mstats set falpha=0-100)
4. Added Raid Worth to (essentially this is just Damage + Healing + Decursing * 500) a slightly tidied up Reports menu
5. Added Quick Reset menu when you right click the C button
6. Added Edit menu popups on right clicking of combat history entries
7. Export Combat Record to XML either normally or cumulatively (graph it in Excel!)

-- KNOWN BUGS --

1. Sometimes it misnames a boss fight, must not be matching a bossname for some reason
2. Sometimes thinks a party member is a mob

-- TODO --

1. Set up Quick Settings options (PVP, Instance, Questing)
2. Fix Onyxia healing error
3. Export class in sep column of XML
-- RELEASE
1. View Combatlog option (filterable snapshot of last 5000 combat messages)
2. Add resurrection counter (if possible)
3. Save combat record for later retrieval
4. When combat record exceeds X size, auto dump first one
5. Click on a player to view all details
6. Some sort of graph over time
7. Resizing of window

--------------------------------------------]]


--[[-------------------------------------------
****************** CONSTANTS ******************
---------------------------------------------]]

--String constants
local MSTATS_DASHES =						"---------------------------------------";
local MSTATS_SPACES =						"                   ";
local MSTATS_MINNAEARSANDTAIL =	"MinnaStats               /\\_/\\     \\||/_";
local MSTATS_VERSION = "1.92";
local MSTATS_NAMEANDVER = "|c00F48CBAMinna|c00FFF468Stats |c00FFFFFFv" .. MSTATS_VERSION;

--Display and Sort Types
local MSTATS_DT_DAMAGEDONE = 1;
local MSTATS_DT_DAMAGETAKEN = 2;
local MSTATS_DT_DAMAGEABSORBED = 3;
local MSTATS_DT_HEALINGDONE = 4;
local MSTATS_DT_HEALINGTAKEN = 5;
local MSTATS_DT_CRITS = 6;
local MSTATS_DT_DEATHS = 7;
local MSTATS_DT_DAMAGETOSELF = 8;
local MSTATS_DT_MISSES = 9;
local MSTATS_DT_DECURSES = 10;
local MSTATS_DT_VSMISSES = 11;
local MSTATS_DT_HIGHESTHIT = 12;
local MSTATS_DT_HIGHESTHEAL = 13;
local MSTATS_DT_CRITPERC = 14;
local MSTATS_DT_MISSPERC = 15;
local MSTATS_DT_EFFICIENCY = 16;
local MSTATS_DT_LIABILITY = 17;
local MSTATS_DT_RAIDWORTH = 18;
local MSTATS_DT_AVERAGEHIT = 19;
local MSTATS_DT_AVERAGEHEAL = 20;
local MSTATS_DT_DAMAGEPS = 21;
local MSTATS_DT_HITSPS = 22;
local MSTATS_DT_HEALINGPS = 23;
local MSTATS_DT_OVERHEALING = 24;
local MSTATS_DT_HEALINGNET = 25;
local MSTATS_DT_DAMAGELOST = 26;

local MSTATS_MT_ALL = 0;
local MSTATS_MT_FIRE = 1;
local MSTATS_MT_FROST = 2;
local MSTATS_MT_NATURE = 3;
local MSTATS_MT_SHADOW = 4;
local MSTATS_MT_ARCANE = 5;
local MSTATS_MT_HOLY = 6;
local MSTATS_MT_PHYSICAL = 7;

local MSTATS_FT_ALL = 0;
local MSTATS_FT_WARRIOR = 1;
local MSTATS_FT_ROGUE = 2;
local MSTATS_FT_PALADIN = 3;
local MSTATS_FT_PRIEST = 4;
local MSTATS_FT_SHAMAN = 5;
local MSTATS_FT_DRUID = 6;
local MSTATS_FT_WARLOCK = 7;
local MSTATS_FT_MAGE = 8;
local MSTATS_FT_HUNTER = 12;
local MSTATS_FT_PET = 13;
local MSTATS_FT_YOURSELF = 9;
local MSTATS_FT_MOBS = 10;
local MSTATS_FT_EXCLUDEMOBS = 11;

local MSTATS_AN_PARTY = 0;
local MSTATS_AN_RAID = 1;
local MSTATS_AN_GUILD = 2;
local MSTATS_AN_OFFICER = 3;
local MSTATS_AN_CHANNEL = 4;
local MSTATS_AN_WHISPER = 5;

local MSTATS_EW_FIGHTNAME = 0;
local MSTATS_EW_CHANNELNAME = 1;
local MSTATS_EW_WHISPERNAME = 2;

--Special CombatLog Strings
local MSTATS_CL_PREFIX = "MS1";
local MSTATS_CL_SYNCPREFIX = "V3";
local MSTATS_CL_SYNCDATA_TYPEA = MSTATS_CL_PREFIX .. MSTATS_CL_SYNCPREFIX .. "SA";
local MSTATS_CL_SYNCDATA_TYPEB = MSTATS_CL_PREFIX .. MSTATS_CL_SYNCPREFIX .. "SB";
local MSTATS_CL_TIMETICKER = MSTATS_CL_PREFIX .. "TIME";
local MSTATS_CL_LOCALTIMETICKER = MSTATS_CL_PREFIX .. "LOCALTIME";
local MSTATS_CL_INCOMBAT = MSTATS_CL_PREFIX .. "INCOMBAT";
local MSTATS_CL_STILLINCOMBAT = MSTATS_CL_PREFIX .. "STILLINCOMBAT";
local MSTATS_CL_SYNCCHECK = MSTATS_CL_PREFIX .. "SYNCCHECK";
local MSTATS_CL_SYNCREPLY = MSTATS_CL_PREFIX .. "SYNCREPLY";

--Update, interval and thread lock vars
local intMStatsProcessing = 0;
local MStats_UpdateInterval = 0.3;
local MStats_TimeSinceLastUpdate = 0;
local MStats_FightTimeSinceLastUpdate = 0;
local MStats_MiniMapUpdateInterval = 0.3;
local MStats_MiniMapTimeSinceLastUpdate = 0;
local MStats_InCombatTimeSinceLastUpdate = 0;

--Various local vars
local objMStatsTotalStatistics = {};
local strMStatsSyncOutLog = {};
local intMStatsSyncDirty = 0;
local intMStatsProcessingDisabled = 0;
local intUpdateWalker = 0;
local intTimeOutOfCombat = 0;
local intMaxTimeOutOfCombat = 10;
local intLogInCombat = 0;
local intLogLastHour = 0;
local intLogLastMinute = 0;
local intLogLastSeconds = 0;
local intInCombat = 0;
local intCurrentFight = 0;
local intCurrentFightOffset = 0;
local intCurrentStatOffset = 0;
local intCurrentStatMaximumOffset = 0;
local strStatFighter = nil;
local strStatFighterClass = nil;
local intEditWindow = MSTATS_EW_FIGHTNAME;
local funcSortDataFunc = nil;

--Fight naming
local MSTATS_NAMED_NOT = 0;
local MSTATS_NAMED_TRASH = 1;
local MSTATS_NAMED_BOSS = 2;

--Decurse names : korean Local By DroArc
local strDecurseNames = {
	"질병 치료",
	"질병 해제",
	"해독",
	"독 흡수",
	"하급 저주 해제",
	"저주 해제",
	"마법 무효화",
	"순화",
	"정화" };

--Resurrection Buff names : korean Local By DroArc
local strResurrectionNames = {
	"부활",
	"구원",
	"환생",
	"고대의 영혼",
	"윤회" };

--Boss names : Korean Local add by DroArc (original document by LibBabble-Boss-3.0)
local strBossNames = {
	--안퀴라즈
	"아누비사스 문지기",
	"전투감시병 살투라",
	"쑨",
	"제왕 베클로어",
	"제왕 베크닐라쉬",
	"쑨의 눈",
	"불굴의 판크리스",
	"군주 크리",
	"아우로",
	"공주 후후란",
	"공주 야우즈",
	"벌레 무리",
	"예언자 스케람",
	"쌍둥이 제왕",
	"벰",
	"비시디우스",
	--아킨둔
	--아케나이 납골당
	"총독 말라다르",
	"죽음의 감시인 쉴라크",
	--마나 무덤
	"연합왕자 샤파르",
	"팬더모니우스",
	"타바로크",
	--어둠의 미궁
	"사자 지옥아귀",
	"선동자 검은심장",
	"단장 보르필",
	"울림",
	--세데크 전당
	"안주", --소환 보스
	"흑마술사 시스",
	"갈퀴대왕 이키스",

	--Blackfathom Deeps
	"아쿠마이",
	"남작 아쿠아니스",
	"겔리하스트",
	"가무라 ",
	"여왕 사레베스",
	"늙은 세라키스",
	"황혼의 군주 켈리스",

	--검은바위 나락
	"사자 화염채찍",
	"격노의 문지기",
	"아눕쉬아",
	"벨가르",
	"Chest of The Seven",
	"운명의 문지기",
	"최면의 문지기",
	"제왕 다그란 타우릿산",
	"적출자",
	"파이너스 다크바이어",
	"사령관 앵거포지",
	"그늘의 문지기",
	"골렘군주 아젤마크",
	"광신자 고로쉬", --check
	"그리즐",
	"증오의 문지기",
	"왕거미 헤드룸",
	"대심문관 게르스탄",
	"타우릿산의 대여사제",
	"사냥개조련사 그렙마르",
	"헐레이 블랙브레스",
	"군주 인센디우스",
	"불의군주 록코르",
	"마그무스",
	"파괴자 오크토르",
	"멸망의 파이론",
	"무적의 판저",
	"팔란스",
	"플러거스파즈링",
	"공주 모이라 브론즈비어드",
	"화염술사 로어그레인",
	"리블리 스크류스피곳",
	"불안의 문지기",
	"The Seven Dwarves",
	"베레크",
	"타락의 문지기",
	"문지기 스틸기스",

	--검은바위 첨탑
	--하층
	"반노크 그림액스",
	"불타는 지옥수호병",
	"수정 맹독 거미",
	"고크 배시구드",
	"흉포한 기즈룰",
	"할리콘",
	"대군주 오모크",
	"모르 그레이후프",
	"여왕 불그물거미",
	"대군주 윔타라크",
	"병참장교 지그리스",
	"어둠사냥꾼 보쉬가진",
	"뾰족바위일족 전투대장",
	"뾰족바위일족 학살자",
	"뾰족바위일족 마법사장",
	"우르크 둠하울",
	"대장군 부네",
	--상층
	"사령관 드라키사스",
	"고랄루크 앤빌크랙",
	"기스",
	"제드 룬와처",
	"군주 발타라크",
	"불의 수호자 엠버시어",
	"화염고리 솔라카르",
	"괴수",
	"대족장 렌드 블랙핸드",

	--검은날개 둥지
	"용기대장 래쉬레이어",
	"크로마구스",
	"에본로크",
	"화염아귀",
	"플레임고르",
	"감시자 그레토크",
	"군주 빅터 네파리우스",
	"네파리안",
	"폭군 서슬송곳니",
	"타락한 밸라스트라즈",

	--검은 사원
	"격노의 정수",
	"욕망의 정수",
	"고뇌의 정수",
	"파괴자 가디오스",
	"구르토그 블러드보일", -- check
	"고위 황천술사 제레보르",
	"대장군 나젠투스",
	"일리단 스톰레이지",
	"일리다리 의회",
	"여군주 말란데",
	"대모 샤라즈",
	"영혼의 성물함",
	"아카마의 망령",
	"궁극의 심연",
	"테론 고어핀드",
	"일리다리 의회", -- check
	"베라스 다크섀도",

	--Borean Tundra
	--The Eye of Eternity
	"말리고스",
	--The Nexus
	"아노말루스",
	"대학자 텔레스트라",
	"케리스트라자",
	"정원사 오르모로크",
	--The Oculus
	"심문관 드라코스",
	"지맥 수호자 에레고스",
	"마법사 군주 우롬",
	"바로스 클라우드스트라이더",

	--시간의 동굴
	--옛 힐스브레드 구릉지
	"경비대장 스칼록",
	"시대의 사냥꾼",
	"부관 드레이크",
	--옛 스트라솔룸
	"살덩이고리",
	"시간의 군주 에포크",
	"말가니스",
	"살덩이창조자 살람",
	--검은 늪
	"아에누스",
	"시간의 군주 데자",
	"메디브",
	"템퍼루스",

	--Chamber of Aspects
	--The Obsidian Sanctum
	--["Sartharion"] = true,
	--["Shadron"] = true,
	--["Tenebron"] = true,
	--["Vesperon"] = true,

	--Coilfang Reservoir
	--Serpentshrine Cavern
	"갈퀴송곳니 정예병",
	"갈퀴송곳니 포자손",
	"심연의 군주 카라드레스",
	"불안정한 히드로스",
	"여군주 바쉬",
	"눈먼 레오테라스",
	"겅둥파도 모로그림",
	"순수한 히드로스의 피조물",
	"레오테라스의 그림자",
	"오염된 히드로스의 피조물",
	"심연의 잠복꾼",
	"겅둥파도 잠복꾼",
	--The Slave Pens
	"배반자 멘누",
	"쿠아그미란",
	"딱딱이 로크마르",
	"아훈",
	--The Steamvault
	"풍수사 세스피아",
	"기계공학자 스팀리거",
	"장군 칼리스레쉬",
	--The Underbog
	"클로",
	"가즈안",
	"헝가르펜",
	"우두머리 성난파도",
	"늪군주 뮤즐레크",
	"검은 추적자",

	--Dire Maul
	--Arena
	"머쉬고그",
	"무적의 스카르",
	"라자",
	--East
	"칼날바람 알진",
	"히드로스폰",
	"이살리엔",
	"레스텐드리스",
	"핌기브",
	"푸실린",
	"제브림 쏜후프",
	--North
	"대장 크롬크러쉬",
	"정찰병 초루쉬",
	"경비병 펜구스",
	"경비병 몰다르",
	"경기병 슬립킥",
	"왕 고르독",
	"노트 팀블젝의 은닉품", -- check
	"천둥발 크리그",
	--West
	"일샨나 레이븐호크",
	"이몰타르",
	"군주 헬누라스",
	"마법사 칼렌드리스",
	"왕자 토르텔드린",
	"굽이나무 텐드리스",
	"츄지",

	--Dragonblight
	-- Ahn'kahet: The Old Kingdom
	"장로 나독스",
	[""]= "사자 볼라즈",
	"어둠추적자 제도가",
	"공작 탈다람",
	--Azjol-Nerub
	"아눕아락",
	"하드로녹스",
	"문지기 크릭시르",

	--Gnomeregan
	"고철 압축기 9-60",
	"검은무쇠단 사절",
	"기계화 문지기 6000",
	"그루비스 ",
	"멕기니어 텔마플러그",
	"첨단로봇",
	"방사성 폐기물",

	--Grizzly Hills
	--Draktharon Keep
	"랩터왕 서슬발톱",
	"소환사 노보스",
	"예언자 타론자",
	"트롤고어",

	--Gruul's Lair
	"현자 블라인드아이",
	"용 학살자 그룰",
	"왕중왕 마울가르",
	"광기의 키글러",
	"크로쉬 파이어핸드",
	"소환사 올름",

	--Hellfire Citadel
	--Hellfire Ramparts
	"나잔",
	"무적의 오모르",
	"사자 바즈루덴",
	"바즈루덴",
	"감시자 가르골마르",
	--Magtheridon's Lair
	"지옥불 역술사",
	"마그테리돈",
	--The Blood Furnace
	"브로고크",
	"파괴자 켈리단",
	"재앙의 창조자",
	--The Shattered Halls
	"혈투사 포룽",
	"대흑마법사 네더쿠르스",
	"돌격대장 오므로그",
	"대족장 카르가스 블레이드피스트",

	--Howling Fjord
	--Utgarde Keep
	"건축가와 감시자", --these are one encounter, so we do this as an encounter name
	"감시자 달론",
	"약탈자 앙그바르",
	"공작 켈레세스",
	"건축가 스카발드",
	--Utgarde Pinnacle
	"학살자 스카디",
	"왕 이미론",
	"소발라 소로우그레이브",
	"고르톡 페일후프",

	--Hyjal Summit
	"아네테론",
	"아키몬드",
	"아즈갈로",
	"카즈로갈",
	"격노한 윈터칠",

	--Karazhan
	"비전 보초",
	"사냥꾼 어튜멘",
	"Chess Event",
	"도로시",
	"Dust Covered Chest",
	"할머니",
	"잠복꾼 히아키스",
	"줄리엔",
	"킬렉",
	"국왕 레인",
	"고결의 여신",
	"천둥이",
	"모로스",
	"황천의 원령",
	"파멸의 어둠",
	"공작 말체자르",
	"잠 못 드는 해골",
	"어흥이",
	"파괴자 로카드",
	"로밀로 & 줄리엔",
	"로밀로",
	"아란의 망령",
	"활강의 샤디키스",
	"허수아비",
	"테레스티안 일후프",
	"커다란 나쁜 늑대",
	"마녀",
	"전시 관리인",
	"양철나무꾼",
	"티토",
	"대족장 블랙핸드",

	-- Magisters' Terrace
	"캘타스 선스트라이더",
	"여사제 델리사",
	"셀린 파이어하트",
	"벡살루스",

	--Maraudon
	"저주받은 셀레브라스",
	"겔크",
	"콜크",
	"산사태",
	"군주 바일텅",
	"마그라",
	"마라우도스",
	"정원사 메슬로크",
	"녹시온",
	"공주 테라드라스",
	"칼날채찍",
	"썩은 아귀",
	"땜장이 기즐록",
	"벵",

	--Molten Core
	"남작 게돈",
	"Cache of the Firelord",
	"가르",
	"게헨나스",
	"초열의 골레마그",
	"루시프론",
	"마그마다르",
	"청지기 이그젝큐투스",
	"라그나로스",
	"샤즈라",
	"설퍼론 사자",

	--Naxxramas
	"아눕레칸",
	"죽음의 기사 수습생",
	"퓨진",
	"Four Horsemen Chest",
	"글루스",
	"영혼 착취자 고딕",
	"귀부인 팰리나",
	"그라불루스",
	"부정의 헤이건",
	"대영주 모그레인",
	"훈련교관 라주비어스",
	"켈투자드",
	"여군주 블라미우스",
	"로데브",
	"맥스나",
	"역병술사 노스",
	"패치워크",
	"사피론",
	"젤리에크 경",
	"스탈라그",
	"타디우스",
	"영주 코스아즈",
	"4인의 기병대",

	--Onyxia's Lair
	"오닉시아",

	--Ragefire Chasm
	"바잘란",
	"기원사 제로쉬",
	"마우르 그림토템",
	"욕망의 타라가만",

	--Razorfen Downs
	"혹한의 암네나르",
	"게걸먹보",
	"불꽃눈 모드레쉬",
	"썩어가는 역병아귀",
	"너덜주둥이",
	"투텐카쉬",

	--Razorfen Kraul
	"흉포한 아가테로스",
	"장님 사냥꾼",
	"서슬깃 차를가",
	"죽음의 예언자 잘그바",
	"대지술사 함가르",
	"대군주 램터스크",

	--Ruins of Ahn'Qiraj
	"아누비사스 감시자",
	"사냥꾼 아야미스",
	"먹보 부루",
	"장군 라작스",
	"쿠린낙스",
	"사령관 안도로브",
	"모암",
	"무적의 오시리안",

	--Scarlet Monastery
	--Armory
	"헤로드",
	--Cathedral
	"종교재판관 페어뱅크스",
	"종교재판관 화이트메인",
	"붉은십자군 사령관 모그레인",
	--Graveyard
	"잠들지 않는 아즈시르",
	"혈법사 탈노스",
	"타락한 용사",
	"심문관 비샤스",
	"무쇠해골",
	"저주받은 기사",
	--Library
	"신비술사 도안",
	"사냥개 조련사 록시",

	--Scholomance
	"키르토노스의 혈지기",
	"암흑스승 간틀링",
	"죽음의 기사 다크리버",
	"학자 테올린 크라스티노브",
	"조교 말리시아",
	"잔다이스 바로브",
	"사자 키르토노스",
	"코르모크",
	"여군주 일루시아 바로브",
	"군주 알렉세이 바로브",
	"현자 폴켈트",
	"마르두크 블랙풀",
	"라스 프로스트위스퍼",
	"들창어금니",
	"라베니안",
	"벡투스",

	--Shadowfang Keep
	"대마법사 아루갈",
	"아루갈의 보이드워커",
	"남작 실버레인",
	"사령관 스프링베일",
	"죽음의 경비대장", -- check
	"파멸의 펜루스",
	"눈먼감시자 오도",
	"도살자 칼날발톱",
	"늑대왕 난도스",

	--Stratholme
	"기록관 갈포드",
	"발나자르",
	"남작 리븐데어",
	"남작부인 아나스타리",
	"검은호위대 검제작자",
	"포병대장 윌리",
	"진홍십자군 대장장이",
	"프라스 샤비",
	"하스싱어 포레스턴",
	"집정관 바실라스",
	"냉혈한 말레키",
	"네룹엔칸",
	"우체국장 말로운",
	"먹보 람스타인",
	"스컬",
	"뾰족바위",
	"용서받지 못한 자",
	"잔혹한 티미",

	--Sunwell Plateau
	"칼렉고스",
	"타락의 사스로바르",
	"브루탈루스",
	"지옥안개",
	"킬제덴",
	"므우루",
	"엔트로피우스",
	"에레다르 쌍둥이",
	"여군주 사크로래쉬",
	"대흑마법사 알리테스",

	--Tempest Keep
	--The Arcatraz
	"파멸의 예언자 달리아",
	"선구자 스키리스",
	"교도관 멜리챠르",
	"격노의 점술사 소코드라테스",
	"속박이 풀린 제레케스",
	--The Botanica
	"지휘관 새래니스",
	"고위 식물학자 프레이윈",
	"라즈",
	"감시인 쏜그린",
	"차원의 분리자",
	--The Eye
	"알라르",
	"붕괴의 지팡이",
	"황폐의 도끼",
	"대점성술사 카퍼니안",
	"고위 점성술사 솔라리안",
	"무한의 비수",
	"캘타스 선스트라이더",
	"군주 생귀나르",
	"수석기술자 텔로니쿠스",
	"황천매듭 장궁",
	"위상 변화의 보루방패",
	"태양의 전당 요원",
	"태양의 전당 사제",
	"우주 에너지 주입기",
	"암흑의 인도자 탈라드레드",
	"공허의 절단기",
	"차원의 절단기",
	--The Mechanar
	"문지기 회전톱날",
	"문지기 무쇠주먹",
	"기계군주 캐퍼시투스",
	"황천술사 세페스레아",
	"철두철미한 파탈리온",

	--The Deadmines
	"세뇌당한 귀족",
	"선장 그린스킨",
	"쿠키",
	"에드윈 밴클리프",
	"현장감독 시슬네틀",
	"길니드",
	"마리사 두페이지",
	"광부 존슨",
	"미스터 스마이트",
	"라크조르",
	"스니드",
	"스니드의 벌목기",

	--The Stockade
	"바질 스레드",
	"무쇠주먹 브루갈",
	"덱스트렌 워드",
	"햄혹",
	"캄 딥퓨리",
	"흉악범 타고르",

	--The Temple of Atal'Hakkar
	"아탈알라리온",
	"학카르의 화신",
	"드림사이드",
	"게이셔",
	"하자스",
	"후쿠",
	"제이드",
	"예언자 잠말란",
	"타락한 카즈카즈",
	"로로",
	"마이잔",
	"몰파즈",
	"비운의 오그옴",
	"에라니쿠스의 사령",
	"식인트롤 베이쟉",
	"위버",
	"젝키스",
	"졸로",
	"줄로",

	--Uldaman
	"고대 바위 문지기",
	"아카에다스",
	"밸로그",
	"발굴단장 쇼벨플랜지",
	"갈간 파이어해머",
	"그림로크",
	"아이로나야",
	"흑요석 파수꾼",
	"레벨로쉬",

	-- Ulduar
	-- Halls of Lightning
	"장군 비야른그림",
	"아이오나",
	"로켄",
	"볼칸",
	-- Halls of Stone
	"크리스탈루스",
	"고뇌의 여신",
	"무쇠구체자 쇼니르",
	--["The Tribunal of Ages"] = true,

	-- The Violet Hold
	--["Cyanigosa"] = true,
	--["Erekem"] = true,
	--["Ichoron"] = true,
	--["Lavanthor"] = true,
	--["Moragg"] = true,
	--["Xevozz"] = true,
	--["Zuramat the Obliterator"] = true,

	--Wailing Caverns
	"보안",
	"돌연변이 요정용",
	"크레쉬",
	"여군주 아나콘드라",
	"군주 코브란",
	"군주 피타스",
	"군주 서펜디스",
	"광기의 매글리시",
	"걸신들린 무타누스",
	"스컴",
	"채찍꼬리 트리고어",
	"영생의 베르단",

	--World Bosses
	"아발란치온",
	"아주어고스",
	"남작 차르",
	"남작 카줌",
	"파멸의 군주 카자크",
	"파멸의 절단기",
	"에메리스",
	"대장군 휠락시스", -- check
	"레손",
	"군주 카자크",
	"군주 스퀄",
	"왕자 스칼레녹스",
	"공주 템페스트리아",
	"타에라",
	"칼날바람",
	"이손드레",

	--Zul'Aman
	"아킬존",
	"할라지",
	"잔알라이",
	"말라크라스", -- check
	"날로라크",
	"줄진",
	["s"] = "주술 군주 말라크라스",

	--Zul'Farrak
	"안투술",
	"족장 우코르즈 샌드스칼프",
	"더스트레이스",
	"가즈릴라",
	"유체술사 벨라타",
	"무르타 그림구트",
	"네크룸 거트츄어",
	"오로 아이가우지",
	"루즐루",
	"Sandarr Dunereaver",
	"성난모래부족 사형집행인",
	"하사관 블라이",
	"어둠의사제 세즈지즈",
	"순교자 데카",
	"의술사 줌라",
	"제릴리스",
	"줄파락 죽음의 영웅",

	-- Zul'Drak
	-- Gundrak
	"혈석 파괴자",	-- check
	"드라카리 거대골렘",
	"갈다라",
	"무라비",
	"슬라드란",

	--Zul'Gurub
	"혈군주 만도키르",
	"가즈란카",
	"그리렉",
	"학카르",
	"하자라",
	"대사제 데칼",
	"대사제 베녹시스",
	"대여사제 알로크",
	"대여사제 제클릭",
	"대여사제 말리",
	"주술사 진도",
	"레나타키",
	"우슬레이",

	--Ring of Blood (where? an instance? should be in other file?)
	"망치발굽",
	"모고르",
	"수렁피일족 쌍둥이",
	"수렁피일족 쌍둥이",
	"파괴의 군주 로크다르",
	"스크라가스",
	"푸른 형제들",
	"전쟁망치일족 용사"};

--[[-------------------------------------------
******************** MAIN *********************
---------------------------------------------]]

function MStats_OnLoad()
	SlashCmdList["MINNASTATS"] = function(msg) MStats_SlashHandler(msg) end;
	SLASH_MINNASTATS1 = "/mstats";
	SLASH_MINNASTATS1 = "/데미지";

	--In and Out of combat
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");

	--Combat msgs
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("PLAYER_CONTROL_GAINED");
	this:RegisterEvent("PLAYER_CONTROL_LOST");
	this:RegisterEvent("CHAT_MSG_ADDON");
	this:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

	objMStats = { strDataVersion = MSTATS_VERSION, intFightStartTime = 0, intFightEndTime = 0, intFightLocalStartTime = 0, intFightLocalEndTime = 0, intFightNamed = 0, strFightName = "", intHourOffset = 0, intTrackMobs = 0, intTrackPets = 0, intDebugMode = 0, intDebugLevel = 1, intChatWindow = 1, intDisplayType = MSTATS_DT_DAMAGEDONE, intDisplayMagicType = MSTATS_MT_ALL, intFilterType = MSTATS_FT_ALL, intLiveTotal = 0, intDeleteProtectTotals = 0, intAnnounceType = MSTATS_AN_PARTY, strAnnounceChannel = "channelname", strAnnounceWhisper = "Gandalf", intGuiLocked = 0, intGuiHidden = 0, intAlpha = 30, intScale = 82, strBackground = "background.tga", intOverHeal = 1, intPartyTrackOnly = 0};

	objMStatsStatistics = {};
	objMStatsStatHistory = {};

	MStats_Print("'/mstats' 또는 '/데미지'를 입력하면 UI창이 열립니다.");
end


function MStats_GetWords(str)
   local ret = {};
   local pos=0;
   while(true) do
     local word;
     _,pos,word=string.find(str, "^ *([^%s]+) *", pos+1);
     if(not word) then
       return ret;
     end
     table.insert(ret, word);
   end
 end

function MStats_SlashHandler(msg)

	msg = string.lower(msg);
	local argList = MStats_GetWords(msg);

	if ((msg == "help") or (msg == "도움말") or (msg == "")) then
		MStats_Print("명령어:");
		MStats_Print("/데미지 도움말 - 명령어 목록에 대한 도움말을 표시합니다.");
		MStats_Print("/데미지 표시 - 데미지 미터기창을 표시/숨김 상태를 전환합니다.");
		MStats_Print("/데미지 체크 - Mstats 애드온을 사용하는 파티/공격대원을 체크합니다.");
		MStats_Print("설정 명령어:");
		MStats_Print("/데미지 펫 - 펫 데이터를 별도로 표시할 것인지 선택합니다.");
		MStats_Print("/데미지 기록 - 플레이어 외에 몹 데이터도 포함시킬 것인지 선택합니다.");
		MStats_Print("/데미지 잠금 - 미니맵 버튼과 데미지 미터기창을 잠급니다.");
		MStats_Print("/데미지 풀림 - 미니맵 버튼과 데미지 미터기창을 풉니다.");
		MStats_Print("/데미지 노오버힐 - 오버힐량을 체크하지 않습니다. (CPU 점유율을 낮춥니다.)");
		MStats_Print("/데미지 오버힐 - 오버힐량을 체크합니다.");
		MStats_Print("/데미지 파티 - 파티나 공격대에 속해있을 때만 데이터를 수집합니다.");
		MStats_Print("/데미지 솔로 - 항상 데이터를 수집합니다.");
		MStats_Print("/데미지 배경 - 뒷배경 텍스쳐를 사용합니다.");
		MStats_Print("/데미지 배경없음 - 뒷배경 테스쳐를 사용하지 않습니다.");
		MStats_Print("/데미지 상태 - 현재 중요 설정상태를 보여줍니다.");
		MStats_Print("/데미지 설정 (크기, 투명도, 시간차감) 숫자 - 선택한 항목을 설정한 숫자만큼 변경합니다.");
		MStats_Print("디버깅 명령어:");
		MStats_Print("/데미지 디버그 - 디버그 정보를 사용 전환합니다.");
		MStats_Print("/데미지 디버그 1 - 디버그 출력 레벨을 1 로 설정합니다.(0-9 가능, 9 는 매우 평범)");
		MStats_Print("/데미지 대화창 1 - 디버그를 출력할 채팅창 번호를 1 번으로 지정합니다.");
		MStats_Print("/데미지 초기화 - 데미지 미터기의 수집 정보를 초기화 합니다.");
		return;
	elseif ((msg == "debug") or (msg == "디버그")) then
		if (objMStats.intDebugMode == 1) then
			objMStats.intDebugMode = 0;
			MStats_Print("디버그 모드 비활성");
		else
			objMStats.intDebugMode = 1;
			MStats_Print("디버그 모드 활성화");
		end
	elseif ((msg == "show") or (msg == "표시")) then
		--MStatsFrame:Show();
		--objMStats.intGuiHidden = 0;
		if (MStatsFrame:IsShown()) then
			MStatsFrame:Hide();
			MStatsFightsFrame:Hide();
			objMStats.intGuiHidden = 1;
		else
			MStatsFrame:Show();
			-- Set frame width and height (to fix odd bug)
			MStatsFrame:SetWidth(230);
			MStatsFrame:SetHeight(200);
			objMStats.intGuiHidden = 0;
		end
	elseif ((argList[1] == "debug") or (argList[1] == "디버그")) then
		objMStats.intDebugLevel = tonumber(argList[2]);
	elseif ((argList[1] == "chat") or (argList[1] == "대화창")) then
		objMStats.intChatWindow = tonumber(argList[2]);
	elseif ((msg == "reset") or (msg == "초기화")) then
		objMStatsStatistics = {};
		objMStatsStatHistory = {};
		intMStatsProcessing = 0;
		strMStatsSyncOutLog = {};
		intMStatsSyncDirty = 0;
		MStats_UpdateTmpFightTotalObject();
		MStats_Print("초기화!");
	elseif ((msg == "check") or (msg == "체크")) then
		SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_SYNCCHECK .. UnitName("player"), "RAID");
		MStats_Print("동기 체크를 보냅니다.");
	elseif ((msg == "track") or (msg == "기록")) then
		if (objMStats.intTrackMobs == 0) then
			objMStats.intTrackMobs = 1;
			MStats_Print("이제 '플레이어 + 몬스터'를 기록합니다.");
		else
			objMStats.intTrackMobs = 0;
			MStats_Print("이제 플레이어만 기록합니다.");
		end
	elseif ((msg == "pets") or (msg == "펫")) then
		if (objMStats.intTrackPets == 0) then
			objMStats.intTrackPets = 1;
			MStats_Print("이제 펫는 따로 기록합니다.");
		else
			objMStats.intTrackPets = 0;
			MStats_Print("이제 펫의 데이터를 플레이어 데이터와 병합해서 기록합니다.");
		end
	elseif ((msg == "lock") or (msg == "잠금")) then
		objMStats.intGuiLocked = 1;
		MStats_Print("이제 프레임과 미니맵 버튼이 잠깁니다..");
	elseif ((msg == "unlock") or (msg == "풀림")) then
		objMStats.intGuiLocked = 0;
		MStats_Print("이제 프레임과 미니맵 버튼의 이동이 가능합니다.");
	elseif ((msg == "overheal") or (msg == "오버힐")) then
		objMStats.intOverHeal = 1;
		MStats_Print("이제 초과치유량이 기록됩니다.");
	elseif ((msg == "nooverheal") or (msg == "노오버힐")) then
		objMStats.intOverHeal = 0;
		MStats_Print("이제 더이상 초과치유량이 기록되지 않습니다.");
	elseif ((msg == "inpartyonly") or (msg == "파티")) then
		objMStats.intPartyTrackOnly = 1;
		MStats_Print("이제 파티나 공격대에 참가했을 때만 기록합니다.");
	elseif ((msg == "notpartyonly") or (msg == "솔로")) then
		objMStats.intPartyTrackOnly = 0;
		MStats_Print("이제 모든 전투에서 기록합니다.");
	elseif ((msg == "status") or (msg == "상태")) then
		MStats_Print("현재 설정 상태는 다음과 같습니다...");
		MStats_Print("시간차감 = " .. objMStats.intHourOffset .. " (로컬시간에 대한 차감을 게임시간으로 부터 가져옵니다.)");
		MStats_Print("프레임투명도 = " .. objMStats.intAlpha .. " (프레임 배경에 대한 투명도는 0부터 100까지 설정됩니다.)");
		MStats_Print("프레임크기 = " .. objMStats.intScale .. " (프레임 UI 크기 비율은 1에서 200까지 퍼센트로 설정됩니다.)");
		MStats_Print("현재 " .. (floor((gcinfo() / 1024) * 100) / 100) .. "MB의 UI 메모리를 사용중입니다.");
	elseif ((msg == "dog") or (msg == "배경")) then
		--objMStats.strBackground = "background.tga";
		objMStats.strBackground = "background";
		MStatsFrameMainBackdrop:SetTexture("Interface\\AddOns\\MinnaStats\\Textures\\" .. objMStats.strBackground);
	elseif ((msg == "nodog") or (msg == "배경없음")) then
		--objMStats.strBackground = "background2.tga";
		objMStats.strBackground = "background2";
		MStatsFrameMainBackdrop:SetTexture("Interface\\AddOns\\MinnaStats\\Textures\\" .. objMStats.strBackground);
	elseif ((argList[1] == "set") or (argList[1] == "설정")) then
		local strVar, strValue = argList[2], argList[3]
		--_,_,strVar,strValue = string.find(string.sub(msg, 5), "(%a+)=([+-]*%d+)");

		if (strVar and strValue) then
			if ((string.lower(strVar) == "hoffset") or (string.lower(strVar) == "시간차감")) then
				objMStats.intHourOffset = tonumber(strValue);
				MStats_Print(objMStats.intHourOffset .. "|1로;으로; 시간차감을 설정합니다.");
			elseif ((string.lower(strVar) == "falpha") or (string.lower(strVar) == "투명도")) then
				objMStats.intAlpha = tonumber(strValue);
				MStats_Print(objMStats.intAlpha .. "|1로;으로; 프레임 투명도를 설정합니다.");
  			MStatsFrameMainBackdrop:SetAlpha(objMStats.intAlpha / 100);
  			MStatsFightsFrameMainBackdrop:SetAlpha(objMStats.intAlpha / 100);
  		elseif ((string.lower(strVar) == "fscale") or (string.lower(strVar) == "크기")) then
				objMStats.intScale = tonumber(strValue);
				MStats_Print(objMStats.intScale .. "|1로;으로; 프레임 크기를 설정합니다.");
  			MStatsFrame:SetScale(objMStats.intScale / 100);
  			MStatsFightsFrame:SetScale(objMStats.intScale / 100);
			end
		end
	else
		MStats_Print("'/mstats' 또는 '/데미지' 를 입력하면 명령어 도움말을 보실수 있습니다.");
	end

end

function MStats_OnEvent(event)
	if (event == "ADDON_LOADED") then
		if (arg1 == "MinnaStats") then
			-- If upgraded set any new vars
			if (objMStats.intDeleteProtectTotals == nil) then
				objMStats.intDeleteProtectTotals = 0;
			end
			if (objMStats.intLiveTotal == nil) then
				objMStats.intLiveTotal = 0;
			end
			if (objMStats.intAnnounceType == nil) then
				objMStats.intAnnounceType = MSTATS_AN_PARTY;
			end
			if (objMStats.strAnnounceChannel == nil) then
				objMStats.strAnnounceChannel = "channelname";
			end
			if (objMStats.strAnnounceWhisper == nil) then
				objMStats.strAnnounceWhisper = "Gandalf";
			end
			if (objMStats.intAlpha == nil) then
				objMStats.intAlpha = 90;
			end
			if (objMStats.intGuiHidden == nil) then
				objMStats.intGuiHidden = 0;
			end
			if (objMStats.strBackground == nil) then
				--objMStats.strBackground = "background.tga";
				objMStats.strBackground = "background";
			end
			if (objMStats.intScale == nil) then
				objMStats.intScale = 100;
			end
			if (objMStats.intTrackPets == nil) then
				objMStats.intTrackPets = 0;
			end
			if (objMStats.intOverHeal == nil) then
				objMStats.intOverHeal = 1;
			end
			if (objMStats.intPartyTrackOnly == nil) then
				objMStats.intPartyTrackOnly = 0;
			end
			if (tonumber(objMStats.strDataVersion) < 1.70) then
				objMStats.intFightLocalStartTime = 0;
				objMStats.intFightLocalEndTime = 0;
				objMStatsStatistics = {};
				objMStatsStatHistory = {};
				intMStatsProcessing = 0;
				strMStatsSyncOutLog = {};
				intMStatsSyncDirty = 0;
				MStats_UpdateTmpFightTotalObject();

				MStats_Print("***************************************");
				MStats_Print("Combat Log wiped due to Upgrade, Sorry!");
				MStats_Print("***************************************");
			end
			objMStats.strDataVersion = MSTATS_VERSION;

			-- Set frame alpha
		  MStatsFrameMainBackdrop:SetAlpha(objMStats.intAlpha / 100);
		  MStatsFightsFrameMainBackdrop:SetAlpha(objMStats.intAlpha / 100);

		  -- Set frame background
		  MStatsFrameMainBackdrop:SetTexture("Interface\\AddOns\\MinnaStats\\Textures\\" .. objMStats.strBackground);

			-- Set frame scale
			MStatsFrame:SetScale(objMStats.intScale / 100);
			MStatsFightsFrame:SetScale(objMStats.intScale / 100);

			if (objMStats.intGuiHidden == 1) then
				MStatsFrame:Hide();
				MStatsFightsFrame:Hide();
			else
				MStatsFrame:Show();
				MStatsFightsFrame:Hide();
			end

			MStats_UpdateTmpFightTotalObject();

			-- Register with MinnaButton
			MinnaButton_RegisterMod("MinnaStats");
			MinnaButton_RegisterMenuItem("초기화", nil, MStats_MiniMapButtonResetClick);
			MinnaButton_RegisterMenuItem("표시/숨김", nil, MStats_MiniMapButtonShowHideClick);
			MinnaButton_RegisterMenuItem("잠금/풀림", nil, MStats_MiniMapButtonLockUnlockClick);
			MinnaButton_RegisterMenuItem("현재 MStats 설정상태", nil, MStats_MiniMapButtonStatusClick);
			MinnaButton_RegisterMenuItem("현재 MStats 사용자 체크", nil, MStats_MiniMapButtonCheckClick);
			MinnaButton_RegisterMenuItem("초과치유량 체크 전환", nil, MStats_MiniMapButtonOverHealClick);
			MinnaButton_RegisterMenuItem("펫 데이터 포함/분리", nil, MStats_MiniMapButtonPetsClick);
			MinnaButton_RegisterMenuItem("몬스터 데이터 표시/숨김", nil, MStats_MiniMapButtonTrackClick);
			MinnaButton_RegisterMenuItem("솔로잉시에 데이터 수집 전환", nil, MStats_MiniMapButtonGroupClick);
			MinnaButton_RegisterMenuItem("배경 텍스쳐 표시/숨김", nil, MStats_MiniMapButtonBackClick);
			MinnaButton_RegisterMenuItem("프레임 크기 조절", nil, MStats_MiniMapButtonScaleClick);
			MinnaButton_RegisterMenuItem("배경 투명도 조절", nil, MStats_MiniMapButtonAlphaClick);
			MinnaButton_RegisterUpdate(MStats_MiniMapButtonUpdate);

			-- 화면 밖으로 나가지 않게하기 위한 보호 코드
			MStatsFrame:SetClampedToScreen(true)
		end

	elseif (event == "PLAYER_REGEN_DISABLED") then
		intInCombat = 1;
		SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_STILLINCOMBAT ..  UnitName("player"), "RAID");
	elseif (event == "PLAYER_REGEN_ENABLED") then
		local hours, minutes;

		intInCombat = 0;
	elseif (event == "CHAT_MSG_ADDON") then
		if (arg4 ~= UnitName("player")) then
			if (arg1 == MSTATS_CL_PREFIX) then
				MStats_ProcessSyncEvents(arg2);
			end
		end
	elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		if (intMStatsProcessingDisabled == 1) then
			return;
		end

		if (objMStats.intPartyTrackOnly == 1) then
			if (GetNumRaidMembers() == 0) then
				if (GetNumPartyMembers() == 0) then
					return;
				end
			end
		end


		--MStats_RecordDamage(source, srcflags, target, targflags, damage, type, crit, resisted, absorbed)
		-- 수정, argN 틀렸다.
		if (arg2 == "SPELL_DAMAGE") then
			--SPELL_DAMAGE,0x0000000000021759,"Vampira",0x511,0xF73000481D02E0F1,"Dampscale Basilisk",0x10a48,38700,"Arcane Missiles",0x40,1141,64,0,0,0,1,nil,nil
			--MStats_RecordDamage(arg4, arg5, arg7, arg8, arg12, arg13, arg17, arg14, arg16);
			MStats_RecordDamage(arg4, arg5, arg7, arg8, arg12, arg14, arg18, arg15, arg17);
		elseif (arg2 == "SPELL_PERIODIC_DAMAGE") then
			--MStats_RecordDamage(arg4, arg5, arg7, arg8, arg12, arg13, arg17, arg14, arg16);
			MStats_RecordDamage(arg4, arg5, arg7, arg8, arg12, arg14, arg18, arg15, arg17);
		elseif (arg2 == "SWING_DAMAGE") then
			--SWING_DAMAGE,0x0000000000021759,"Vampira",0x511,0xF73000481D02E0F1,"Dampscale Basilisk",0x10a48,73,1,0,0,0,nil,nil,nil
			--MStats_RecordDamage(arg4, arg5, arg7, arg8, arg9, arg10, arg14, arg11, arg13);
			MStats_RecordDamage(arg4, arg5, arg7, arg8, arg9, arg11, arg15, arg12, arg14);
		elseif (arg2 == "RANGE_DAMAGE") then
			--RANGE_DAMAGE,0x0000000000021759,"Vampira",0x511,0xF73000481D02E0F1,"Dampscale Basilisk",0x10a48,5019,"Shoot",0x1,371,32,0,0,0,1,nil,nil
			--MStats_RecordDamage(arg4, arg5, arg7, arg8, arg12, arg13, arg17, arg14, arg16);
			MStats_RecordDamage(arg4, arg5, arg7, arg8, arg12, arg14, arg18, arg15, arg17);
		--elseif (arg2 == "ENVIRONMENTAL_DAMAGE") then
		elseif (string.sub(arg2, -7) == "_MISSED") then
			--MStats_ProcessMissedEvents();
		elseif (arg2 == "SPELL_HEAL") then
			--MStats_RecordHealing(arg4, arg5, arg7, arg8, arg12, arg13);
			MStats_RecordHealing(arg4, arg5, arg7, arg8, arg12, arg14);
		elseif (arg2 == "SPELL_PERIODIC_HEAL") then
			--MStats_RecordHealing(arg4, arg5, arg7, arg8, arg12, arg13);
			MStats_RecordHealing(arg4, arg5, arg7, arg8, arg12, arg14);
		elseif (string.sub(arg2, -9) == "_ENERGIZE") then
			--MStats_ProcessEnergizeEvents();
		elseif (string.sub(arg2, -6) == "_LEECH") then
			--MStats_ProcessLeechEvents();
		elseif (string.sub(arg2, -6) == "_DRAIN") then
			--MStats_ProcessDrainEvents();
		elseif (arg2 == "PARTY_KILL") then
			--MStats_ProcessKillEvents();
		--elseif (arg2 == "UNIT_DIED") then
			--MStats_RecordDeath(arg7, arg8);
		--elseif (arg2 == "UNIT_DESTROYED") then
			--MStats_RecordDeath(arg7, arg8);
		--else
			--MStats_ProcessUnprocessedEvents();
		end

		--_AURA_DISPELLED
		--_AURA_STOLEN
		--_AURA_APPLIED
		--_AURA_REMOVED
		--_INTERRUPT
		--_DISPEL_FAILED

		--MStats_ProcessCastEvents();
	end
end

function MStats_MiniMapButtonUpdate(elapsed)
	local hours, minutes, seconds, k, v;

	MStats_MiniMapTimeSinceLastUpdate = MStats_MiniMapTimeSinceLastUpdate + elapsed;

	--Add time elapsed since last OOC reset, and set OOC if exceeds threshold
	if (intLogInCombat == 1) then
		intTimeOutOfCombat = intTimeOutOfCombat + elapsed;
		if (intTimeOutOfCombat > intMaxTimeOutOfCombat) then
	  	intLogInCombat = 0;
	  end
	end

	if (intInCombat == 1) then
		MStats_InCombatTimeSinceLastUpdate = MStats_InCombatTimeSinceLastUpdate + elapsed;
		if (MStats_InCombatTimeSinceLastUpdate > 1) then
			SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_STILLINCOMBAT .. UnitName("player"), "RAID");
			MStats_InCombatTimeSinceLastUpdate = 0;
		end
	end

	if (MStats_MiniMapTimeSinceLastUpdate < MStats_MiniMapUpdateInterval) then
  	return;
  end

  --Add time ticker to combat log and broadcast if in combat
	hours,minutes = GetGameTime();
	MStats_ProcessSyncEvents(MSTATS_CL_TIMETICKER .. hours .. ":" .. minutes);
	seconds = GetTime();
	MStats_ProcessSyncEvents(MSTATS_CL_LOCALTIMETICKER .. seconds);

  -- Make sure this isn't re-cursive
	if (intMStatsProcessing == 1) then
		return;
	end

	intMStatsProcessing = 1;

	-- Process events
	if not (intInCombat == 1) then
		-- Send a sync data
		if (#strMStatsSyncOutLog > 0) then
			SendAddonMessage(MSTATS_CL_PREFIX, strMStatsSyncOutLog[1], "RAID");
			table.remove(strMStatsSyncOutLog, 0);
		elseif (intMStatsSyncDirty == 1) then
			local count;

			-- Build sync data
			strMStatsSyncOutLog = {};
			count = 1;
			-- Only build sync data if pet damage is combined with player damage
			if (objMStats.intTrackPets == 0) then
				for k,v in ipairs(objMStatsStatistics) do
					strMStatsSyncOutLog[count] = MSTATS_CL_SYNCDATA_TYPEA .. objMStats.intFightStartTime .. "||" .. objMStatsStatistics[k].name .. "||" .. objMStatsStatistics[k].dmgdone .. "||" .. objMStatsStatistics[k].dmgtaken .. "||" .. objMStatsStatistics[k].dmgabsorbed .. "||" .. objMStatsStatistics[k].dmgtoself .. "||" .. objMStatsStatistics[k].noncrits .. "||" .. objMStatsStatistics[k].crits .. "||" .. objMStatsStatistics[k].hits .. "||" .. objMStatsStatistics[k].misses .. "||" .. objMStatsStatistics[k].vshits .. "||" .. objMStatsStatistics[k].vsmisses .. "||" .. objMStatsStatistics[k].healingdone .. "||" .. objMStatsStatistics[k].overhealingdone .. "||" .. objMStatsStatistics[k].healingtaken .. "||" .. objMStatsStatistics[k].deaths .. "||" .. objMStatsStatistics[k].decurses .. "||" .. objMStatsStatistics[k].highesthit .. "||" .. objMStatsStatistics[k].highestheal;
					count = count + 1;
					strMStatsSyncOutLog[count] = MSTATS_CL_SYNCDATA_TYPEB .. objMStats.intFightStartTime .. "||" .. objMStatsStatistics[k].name .. "||" .. objMStatsStatistics[k].firedone .. "||" .. objMStatsStatistics[k].firetaken .. "||" .. objMStatsStatistics[k].fireabsorbed .. "||" .. objMStatsStatistics[k].frostdone .. "||" .. objMStatsStatistics[k].frosttaken .. "||" .. objMStatsStatistics[k].frostabsorbed .. "||" .. objMStatsStatistics[k].naturedone .. "||" .. objMStatsStatistics[k].naturetaken .. "||" .. objMStatsStatistics[k].natureabsorbed .. "||" .. objMStatsStatistics[k].shadowdone .. "||" .. objMStatsStatistics[k].shadowtaken .. "||" .. objMStatsStatistics[k].shadowabsorbed .. "||" .. objMStatsStatistics[k].arcanedone .. "||" .. objMStatsStatistics[k].arcanetaken .. "||" .. objMStatsStatistics[k].arcaneabsorbed .. "||" .. objMStatsStatistics[k].holydone .. "||" .. objMStatsStatistics[k].holytaken .. "||" .. objMStatsStatistics[k].holyabsorbed;
					count = count + 1;
				end
			end
			intMStatsSyncDirty = 0;
		end
	end

	intMStatsProcessing = 0;
	MStats_MiniMapTimeSinceLastUpdate = 0;
end

function MStats_Update(elapsed)
	local intPercent = 0;
	local funcDataFunc, i, top, high, objTmpStats, intLineCounter, intCFIndex, k, v;

	MStats_TimeSinceLastUpdate = MStats_TimeSinceLastUpdate + elapsed;

  if (MStats_TimeSinceLastUpdate < MStats_UpdateInterval) then
  	return;
  end

	-- Update UI
	if (intMStatsProcessingDisabled == 0) then
		if (intInCombat == 1) then
			MStatsFrameTitle:SetText(MSTATS_NAMEANDVER .. " |c00FFF468C");
		else
			MStatsFrameTitle:SetText(MSTATS_NAMEANDVER);
		end
	else
		if (intUpdateWalker < 2) then
			MStatsFrameTitle:SetText(MSTATS_NAMEANDVER);
		else
			MStatsFrameTitle:SetText("*비활성됨!*");
		end
	end

	-- Update animation walker var
	intUpdateWalker = intUpdateWalker + 1;
	if (intUpdateWalker > 3) then
		intUpdateWalker = 0;
	end

	if (intCurrentFight > 0) then
		objTmpStats = objMStatsStatHistory[intCurrentFight].objMStatsStatistics;
	else
		if (objMStats.intLiveTotal == 1) then
			objTmpStats = objMStatsTotalStatistics;
		else
			objTmpStats = objMStatsStatistics;
		end
	end

	MStats_SortData();

	if (strStatFighter == nil) then
		MStatsFrameDataType:SetText(MStats_GetDataTypeString());
		funcDataFunc = MStats_GetDataValueFunction();
	else
		-- Find index of this player
		for k,v in ipairs(objTmpStats) do
			if (objTmpStats[k].name == strStatFighter) then
				intCFIndex = k;
				break;
			end
		end

		MStatsFrameDataType:SetText(strStatFighter .. " (" .. strStatFighterClass .. ")");
	end

	-- Debug title
	--MStatsFrameTitle:SetText("StatMax: " .. intCurrentStatMaximumOffset .. " / " .. #objTmpStats);

	intLineCounter = 1;
	top = 0;
	intCurrentStatMaximumOffset = 0;
	if (strStatFighter == nil) then
		for k,v in ipairs(objTmpStats) do
			if (funcDataFunc(objTmpStats[k]) == 0) then
				break;
			end

			if (MStats_FilteredClass(objTmpStats[k].name, objTmpStats[k].class)) then
				if ((intLineCounter > intCurrentStatOffset) and (intLineCounter < (11 + intCurrentStatOffset))) then
					local strTemp;

					strTemp = intLineCounter .. ".";
					if (string.len(strTemp) < 3) then
						strTemp = " " .. strTemp;
					end
					getglobal("MStatsFrameBar" .. (intLineCounter - intCurrentStatOffset) .. "Player"):SetText(strTemp .. " " .. objTmpStats[k].name);
					getglobal("MStatsFrameBar" .. (intLineCounter - intCurrentStatOffset) .. "Value"):SetText(funcDataFunc(objTmpStats[k]));

					top = intLineCounter - intCurrentStatOffset;
				end

				intPercent = intPercent + funcDataFunc(objTmpStats[k]);
				intLineCounter = intLineCounter + 1;
				intCurrentStatMaximumOffset = intCurrentStatMaximumOffset + 1;
			end
		end
	else
		if (intCFIndex == nil) then
			Mstats_SetStatsLine(1, "준 피해량", 0);
			Mstats_SetStatsLine(2, "받은 피해량", 0);
			Mstats_SetStatsLine(3, "흡수된 피해량", 0);
			Mstats_SetStatsLine(4, "자신의 피해량", 0);
			Mstats_SetStatsLine(5, "치유량 (총합)", 0);
			Mstats_SetStatsLine(6, "치유량 (최종)", 0);
			Mstats_SetStatsLine(7, "초과 치유량", 0);
			Mstats_SetStatsLine(8, "받은 치유량", 0);

			Mstats_SetStatsLine(9, "준 물리 피해", 0);
			Mstats_SetStatsLine(10, "준 화염 피해", 0);
			Mstats_SetStatsLine(11, "준 냉기 피해", 0);
			Mstats_SetStatsLine(12, "준 자연 피해", 0);
			Mstats_SetStatsLine(13, "준 암흑 피해", 0);
			Mstats_SetStatsLine(14, "준 비전 피해", 0);
			Mstats_SetStatsLine(15, "준 신성 피해", 0);

			Mstats_SetStatsLine(16, "받은 물리 피해", 0);
			Mstats_SetStatsLine(17, "흡수된 물리 피해", 0);
			Mstats_SetStatsLine(18, "받은 화염 피해", 0);
			Mstats_SetStatsLine(19, "흡수된 화염 피해", 0);
			Mstats_SetStatsLine(20, "받은 냉기 피해", 0);
			Mstats_SetStatsLine(21, "흡수된 냉기 피해", 0);
			Mstats_SetStatsLine(22, "받은 자연 피해", 0);
			Mstats_SetStatsLine(23, "흡수된 자연 피해", 0);
			Mstats_SetStatsLine(24, "받은 암흑 피해", 0);
			Mstats_SetStatsLine(25, "흡수된 암흑 피해", 0);
			Mstats_SetStatsLine(26, "받은 비전 피해", 0);
			Mstats_SetStatsLine(27, "흡수된 비전 피해", 0);
			Mstats_SetStatsLine(28, "받은 신성 피해", 0);
			Mstats_SetStatsLine(29, "흡수된 신성 피해", 0);

			Mstats_SetStatsLine(30, "치명타율", 0);
			Mstats_SetStatsLine(31, "실패", 0);
			Mstats_SetStatsLine(32, "vs 실패", 0);

			Mstats_SetStatsLine(33, "죽음", 0);
			Mstats_SetStatsLine(34, "해제", 0);

			Mstats_SetStatsLine(35, "최대 명중", 0);
			Mstats_SetStatsLine(36, "평균 명중", 0);
			Mstats_SetStatsLine(37, "최대 치유", 0);
			Mstats_SetStatsLine(38, "평균 치유", 0);

			Mstats_SetStatsLine(39, "피해량/초", 0);
			Mstats_SetStatsLine(40, "명중/초", 0);
			Mstats_SetStatsLine(41, "치유량/초", 0);
		else
			Mstats_SetStatsLine(1, "준 피해량", objTmpStats[intCFIndex].dmgdone);
			Mstats_SetStatsLine(2, "받은 피해량", objTmpStats[intCFIndex].dmgtaken);
			Mstats_SetStatsLine(3, "흡수된 피해량", objTmpStats[intCFIndex].dmgabsorbed);
			Mstats_SetStatsLine(4, "자신의 피해량", objTmpStats[intCFIndex].dmgtoself);
			Mstats_SetStatsLine(5, "치유량 (총합)", objTmpStats[intCFIndex].healingdone);
			Mstats_SetStatsLine(6, "치유량 (최종)", objTmpStats[intCFIndex].healingdone - objTmpStats[intCFIndex].overhealingdone);
			Mstats_SetStatsLine(7, "초과 치유량", objTmpStats[intCFIndex].overhealingdone);
			Mstats_SetStatsLine(8, "받은 치유량", objTmpStats[intCFIndex].healingtaken);

			Mstats_SetStatsLine(9, "준 물리 피해", objTmpStats[intCFIndex].physicaldone);
			Mstats_SetStatsLine(10, "준 화염 피해", objTmpStats[intCFIndex].firedone);
			Mstats_SetStatsLine(11, "준 냉기 피해", objTmpStats[intCFIndex].frostdone);
			Mstats_SetStatsLine(12, "준 자연 피해", objTmpStats[intCFIndex].naturedone);
			Mstats_SetStatsLine(13, "준 암흑 피해", objTmpStats[intCFIndex].shadowdone);
			Mstats_SetStatsLine(14, "준 비전 피해", objTmpStats[intCFIndex].arcanedone);
			Mstats_SetStatsLine(15, "준 신성 피해", objTmpStats[intCFIndex].holydone);

			Mstats_SetStatsLine(16, "받은 물리 피해", objTmpStats[intCFIndex].physicaltaken);
			Mstats_SetStatsLine(17, "흡수된 물리 피해", objTmpStats[intCFIndex].physicalabsorbed);
			Mstats_SetStatsLine(18, "받은 화염 피해", objTmpStats[intCFIndex].firetaken);
			Mstats_SetStatsLine(19, "흡수된 화염 피해", objTmpStats[intCFIndex].fireabsorbed);
			Mstats_SetStatsLine(20, "받은 냉기 피해", objTmpStats[intCFIndex].frosttaken);
			Mstats_SetStatsLine(21, "흡수된 냉기 피해", objTmpStats[intCFIndex].frostabsorbed);
			Mstats_SetStatsLine(22, "받은 자연 피해", objTmpStats[intCFIndex].naturetaken);
			Mstats_SetStatsLine(23, "흡수된 자연 피해", objTmpStats[intCFIndex].natureabsorbed);
			Mstats_SetStatsLine(24, "받은 암흑 피해", objTmpStats[intCFIndex].shadowtaken);
			Mstats_SetStatsLine(25, "흡수된 암흑 피해", objTmpStats[intCFIndex].shadowabsorbed);
			Mstats_SetStatsLine(26, "받은 비전 피해", objTmpStats[intCFIndex].arcanetaken);
			Mstats_SetStatsLine(27, "흡수된 비전 피해", objTmpStats[intCFIndex].arcaneabsorbed);
			Mstats_SetStatsLine(28, "받은 신성 피해", objTmpStats[intCFIndex].holytaken);
			Mstats_SetStatsLine(29, "흡수된 신성 피해", objTmpStats[intCFIndex].holyabsorbed);

			Mstats_SetStatsLine(30, "치명타", objTmpStats[intCFIndex].crits);
			Mstats_SetStatsLine(31, "실패", objTmpStats[intCFIndex].misses);
			Mstats_SetStatsLine(32, "vs 실패", objTmpStats[intCFIndex].vsmisses);

			Mstats_SetStatsLine(33, "죽음", objTmpStats[intCFIndex].deaths);
			Mstats_SetStatsLine(34, "해제", objTmpStats[intCFIndex].decurses);

			Mstats_SetStatsLine(35, "최대 명중", objTmpStats[intCFIndex].highesthit);
			Mstats_SetStatsLine(36, "평균 명중", MStats_Getaveragehit(objTmpStats[intCFIndex]));
			Mstats_SetStatsLine(37, "최대 치유", objTmpStats[intCFIndex].highestheal);
			Mstats_SetStatsLine(38, "평균 치유", MStats_Getaverageheal(objTmpStats[intCFIndex]));

			Mstats_SetStatsLine(39, "피해량/초", MStats_Getdamageps(objTmpStats[intCFIndex]));
			Mstats_SetStatsLine(40, "명중/초", MStats_Gethitsps(objTmpStats[intCFIndex]));
			Mstats_SetStatsLine(41, "치유량/초", MStats_Gethealsps(objTmpStats[intCFIndex]));
		end

		top = 11;
	end

	intLineCounter = 1;
	high = 0;
	-- Show Percentage and colour bars
	if (strStatFighter == nil) then
		for k,v in ipairs(objTmpStats) do
			if (MStats_FilteredClass(objTmpStats[k].name, objTmpStats[k].class)) then
				if ((intLineCounter > intCurrentStatOffset) and (intLineCounter < (11 + intCurrentStatOffset))) then
					if (funcDataFunc(objTmpStats[k]) > 0) then
						local percentage = (floor((100 / intPercent) * (funcDataFunc(objTmpStats[k]) * 100)) / 100);

						getglobal("MStatsFrameBar" .. (intLineCounter - intCurrentStatOffset) .. "Percent"):SetText(percentage .. "%");
						getglobal("MStatsFrameBar" .. (intLineCounter - intCurrentStatOffset)):SetValue(percentage);
						if (percentage > high) then
							high = percentage;
						end
					else
						break;
					end

					MStats_SetBarColour(objTmpStats[k].class, getglobal("MStatsFrameBar" .. (intLineCounter - intCurrentStatOffset)));
				end
				intLineCounter = intLineCounter + 1;
			end
		end

		for i = 1, 10, 1 do
			getglobal("MStatsFrameBar" .. i):SetMinMaxValues(0, high);
	  end
	else
		if (intCFIndex == nil) then
			Mstats_SetStatsPercentage(1, 100, 100, 0, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(2, 100, 100, 0, 0.8, 0, 0);
			Mstats_SetStatsPercentage(3, 100, 100, 0, 0.8, 0.2, 0.2);
			Mstats_SetStatsPercentage(4, 100, 100, 0, 0.835, 0.203, 0.82);
			Mstats_SetStatsPercentage(5, 100, 100, 0, 1, 1, 1);
			Mstats_SetStatsPercentage(6, 100, 100, 0, 1, 1, 1);
			Mstats_SetStatsPercentage(7, 100, 100, 0, 1, 1, 1);
			Mstats_SetStatsPercentage(8, 100, 100, 0, 0.8, 0.8, 0.8);

			Mstats_SetStatsPercentage(9, 100, 100, 0, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(10, 100, 100, 0, 1, 0.211, 0.211);
			Mstats_SetStatsPercentage(11, 100, 100, 0, 0.345, 0.482, 0.894);
			Mstats_SetStatsPercentage(12, 100, 100, 0, 0.156, 0.752, 0.133);
			Mstats_SetStatsPercentage(13, 100, 100, 0, 0.651, 0.215, 0.874);
			Mstats_SetStatsPercentage(14, 100, 100, 0, 0.384, 0.290, 0.874);
			Mstats_SetStatsPercentage(15, 100, 100, 0, 0.874, 0.866, 0.639);

			Mstats_SetStatsPercentage(16, 100, 100, 0, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(17, 100, 100, 0, 1, 0.709, 0.431);
			Mstats_SetStatsPercentage(18, 100, 100, 0, 1, 0.211, 0.211);
			Mstats_SetStatsPercentage(19, 100, 100, 0, 1, 0.451, 0.502);
			Mstats_SetStatsPercentage(20, 100, 100, 0, 0.345, 0.482, 0.894);
			Mstats_SetStatsPercentage(21, 100, 100, 0, 0.547, 0.641, 1);
			Mstats_SetStatsPercentage(22, 100, 100, 0, 0.156, 0.752, 0.133);
			Mstats_SetStatsPercentage(23, 100, 100, 0, 0.411, 0.752, 0.376);
			Mstats_SetStatsPercentage(24, 100, 100, 0, 0.651, 0.215, 0.874);
			Mstats_SetStatsPercentage(25, 100, 100, 0, 0.821, 0.437, 0.874);
			Mstats_SetStatsPercentage(26, 100, 100, 0, 0.384, 0.290, 0.874);
			Mstats_SetStatsPercentage(27, 100, 100, 0, 0.605, 0.570, 0.874);
			Mstats_SetStatsPercentage(28, 100, 100, 0, 0.874, 0.866, 0.639);
			Mstats_SetStatsPercentage(29, 100, 100, 0, 0.874, 0.862, 0.780);

			Mstats_SetStatsPercentage(30, 100, 100, 0, 1, 0.211, 0.211);
			Mstats_SetStatsPercentage(31, 100, 100, 0, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(32, 100, 100, 0, 1, 0.709, 0.431);

			Mstats_SetStatsPercentage(33, 100, 100, 0, 0.8, 0, 0);
			Mstats_SetStatsPercentage(34, 100, 100, 0, 0.384, 0.290, 0.874);

			Mstats_SetStatsPercentage(35, 100, 100, 0, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(36, 100, 100, 0, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(37, 100, 100, 0, 1, 1, 1);
			Mstats_SetStatsPercentage(38, 100, 100, 0, 1, 1, 1);

			Mstats_SetStatsPercentage(39, 100, 100, 0, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(40, 100, 100, 0, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(41, 100, 100, 0, 1, 1, 1);
		else
			local intMaxDamage, intMaxDamageHealing, intMaxHealing, intMaxTaken, intMaxHitHeal;
			local intTotalDamage, intTotalDamageHealing, intTotalHealing, intTotalTaken, intTotalHitHeal;

			intMaxDamageHealing = math.max(objTmpStats[intCFIndex].dmgdone, objTmpStats[intCFIndex].dmgtaken, objTmpStats[intCFIndex].dmgabsorbed, objTmpStats[intCFIndex].dmgtoself);
			intTotalDamageHealing = objTmpStats[intCFIndex].dmgdone + objTmpStats[intCFIndex].dmgtaken + objTmpStats[intCFIndex].dmgabsorbed + objTmpStats[intCFIndex].dmgtoself;
			intMaxHealing = math.max(objTmpStats[intCFIndex].healingdone, objTmpStats[intCFIndex].healingtaken);
			intTotalHealing = objTmpStats[intCFIndex].healingdone + objTmpStats[intCFIndex].healingtaken;
			Mstats_SetStatsPercentage(1, intTotalDamageHealing, intMaxDamageHealing, objTmpStats[intCFIndex].dmgdone, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(2, intTotalDamageHealing, intMaxDamageHealing, objTmpStats[intCFIndex].dmgtaken, 0.8, 0, 0);
			Mstats_SetStatsPercentage(3, intTotalDamageHealing, intMaxDamageHealing, objTmpStats[intCFIndex].dmgabsorbed, 0.8, 0.2, 0.2);
			Mstats_SetStatsPercentage(4, intTotalDamageHealing, intMaxDamageHealing, objTmpStats[intCFIndex].dmgtoself, 0.835, 0.203, 0.82);
			Mstats_SetStatsPercentage(5, intTotalHealing, intMaxHealing, objTmpStats[intCFIndex].healingdone, 1, 1, 1);
			Mstats_SetStatsPercentage(6, intTotalHealing, intMaxHealing, objTmpStats[intCFIndex].healingdone - objTmpStats[intCFIndex].overhealingdone, 1, 1, 1);
			Mstats_SetStatsPercentage(7, intTotalHealing, intMaxHealing, objTmpStats[intCFIndex].overhealingdone, 1, 1, 1);
			Mstats_SetStatsPercentage(8, intTotalHealing, intMaxHealing, objTmpStats[intCFIndex].healingtaken, 0.8, 0.8, 0.8);

			intMaxDamage = math.max(objTmpStats[intCFIndex].physicaldone, objTmpStats[intCFIndex].firedone, objTmpStats[intCFIndex].frostdone, objTmpStats[intCFIndex].naturedone, objTmpStats[intCFIndex].shadowdone, objTmpStats[intCFIndex].arcanedone, objTmpStats[intCFIndex].holydone);
			intTotalDamage = objTmpStats[intCFIndex].physicaldone + objTmpStats[intCFIndex].firedone + objTmpStats[intCFIndex].frostdone + objTmpStats[intCFIndex].naturedone + objTmpStats[intCFIndex].shadowdone + objTmpStats[intCFIndex].arcanedone + objTmpStats[intCFIndex].holydone;
			Mstats_SetStatsPercentage(9, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].physicaldone, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(10, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].firedone, 1, 0.211, 0.211);
			Mstats_SetStatsPercentage(11, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].frostdone, 0.345, 0.482, 0.894);
			Mstats_SetStatsPercentage(12, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].naturedone, 0.156, 0.752, 0.133);
			Mstats_SetStatsPercentage(13, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].shadowdone, 0.651, 0.215, 0.874);
			Mstats_SetStatsPercentage(14, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].arcanedone, 0.384, 0.290, 0.874);
			Mstats_SetStatsPercentage(15, intTotalDamage, intMaxDamage, objTmpStats[intCFIndex].holydone, 0.874, 0.866, 0.639);

			intMaxTaken = math.max(objTmpStats[intCFIndex].physicaltaken, objTmpStats[intCFIndex].physicalabsorbed, objTmpStats[intCFIndex].firetaken, objTmpStats[intCFIndex].fireabsorbed, objTmpStats[intCFIndex].frosttaken, objTmpStats[intCFIndex].frostabsorbed, objTmpStats[intCFIndex].naturetaken, objTmpStats[intCFIndex].natureabsorbed, objTmpStats[intCFIndex].shadowtaken, objTmpStats[intCFIndex].shadowabsorbed, objTmpStats[intCFIndex].arcanetaken, objTmpStats[intCFIndex].arcaneabsorbed, objTmpStats[intCFIndex].holytaken, objTmpStats[intCFIndex].holyabsorbed);
			intTotalTaken = objTmpStats[intCFIndex].physicaltaken + objTmpStats[intCFIndex].physicalabsorbed + objTmpStats[intCFIndex].firetaken + objTmpStats[intCFIndex].fireabsorbed + objTmpStats[intCFIndex].frosttaken + objTmpStats[intCFIndex].frostabsorbed + objTmpStats[intCFIndex].naturetaken + objTmpStats[intCFIndex].natureabsorbed + objTmpStats[intCFIndex].shadowtaken + objTmpStats[intCFIndex].shadowabsorbed + objTmpStats[intCFIndex].arcanetaken + objTmpStats[intCFIndex].arcaneabsorbed + objTmpStats[intCFIndex].holytaken + objTmpStats[intCFIndex].holyabsorbed;
			Mstats_SetStatsPercentage(16, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].physicaltaken, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(17, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].physicalabsorbed, 1, 0.709, 0.431);
			Mstats_SetStatsPercentage(18, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].firetaken, 1, 0.211, 0.211);
			Mstats_SetStatsPercentage(19, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].fireabsorbed, 1, 0.451, 0.502);
			Mstats_SetStatsPercentage(20, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].frosttaken, 0.345, 0.482, 0.894);
			Mstats_SetStatsPercentage(21, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].frostabsorbed, 0.547, 0.641, 1);
			Mstats_SetStatsPercentage(22, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].naturetaken, 0.156, 0.752, 0.133);
			Mstats_SetStatsPercentage(23, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].natureabsorbed, 0.411, 0.752, 0.376);
			Mstats_SetStatsPercentage(24, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].shadowtaken, 0.651, 0.215, 0.874);
			Mstats_SetStatsPercentage(25, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].shadowabsorbed, 0.821, 0.437, 0.874);
			Mstats_SetStatsPercentage(26, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].arcanetaken, 0.384, 0.290, 0.874);
			Mstats_SetStatsPercentage(27, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].arcaneabsorbed, 0.605, 0.570, 0.874);
			Mstats_SetStatsPercentage(28, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].holytaken, 0.874, 0.866, 0.639);
			Mstats_SetStatsPercentage(29, intTotalTaken, intMaxTaken, objTmpStats[intCFIndex].holyabsorbed, 0.874, 0.862, 0.780);

			Mstats_SetStatsPercentage(30, (objTmpStats[intCFIndex].crits + objTmpStats[intCFIndex].noncrits), (objTmpStats[intCFIndex].crits + objTmpStats[intCFIndex].noncrits), objTmpStats[intCFIndex].crits, 1, 0.211, 0.211);
			Mstats_SetStatsPercentage(31, (objTmpStats[intCFIndex].hits + objTmpStats[intCFIndex].misses), (objTmpStats[intCFIndex].hits + objTmpStats[intCFIndex].misses), objTmpStats[intCFIndex].misses, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(32, (objTmpStats[intCFIndex].vshits + objTmpStats[intCFIndex].vsmisses), (objTmpStats[intCFIndex].vshits + objTmpStats[intCFIndex].vsmisses), objTmpStats[intCFIndex].vsmisses, 1, 0.709, 0.431);

			if (objTmpStats[intCFIndex].deaths > 0) then
				Mstats_SetStatsPercentage(33, objTmpStats[intCFIndex].deaths, objTmpStats[intCFIndex].deaths, objTmpStats[intCFIndex].deaths, 0.8, 0, 0);
			else
				Mstats_SetStatsPercentage(33, 100, 100, 0, 0.8, 0, 0);
			end
			if (objTmpStats[intCFIndex].decurses > 0 ) then
				Mstats_SetStatsPercentage(34, objTmpStats[intCFIndex].decurses, objTmpStats[intCFIndex].decurses, objTmpStats[intCFIndex].decurses, 0.384, 0.290, 0.874);
			else
				Mstats_SetStatsPercentage(34, 100, 100, 0, 0.384, 0.290, 0.874);
			end

			intMaxHitHeal = math.max(objTmpStats[intCFIndex].highesthit, objTmpStats[intCFIndex].highestheal);
			intTotalHitHeal = objTmpStats[intCFIndex].highesthit + objTmpStats[intCFIndex].highestheal;
			Mstats_SetStatsPercentage(35, intTotalHitHeal, intMaxHitHeal, objTmpStats[intCFIndex].highesthit, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(36, objTmpStats[intCFIndex].highesthit, objTmpStats[intCFIndex].highesthit, MStats_Getaveragehit(objTmpStats[intCFIndex]), 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(37, intTotalHitHeal, intMaxHitHeal, objTmpStats[intCFIndex].highestheal, 1, 1, 1);
			Mstats_SetStatsPercentage(38, objTmpStats[intCFIndex].highestheal, objTmpStats[intCFIndex].highestheal, MStats_Getaverageheal(objTmpStats[intCFIndex]), 1, 1, 1);

			Mstats_SetStatsPercentage(39, objTmpStats[intCFIndex].highesthit, MStats_Getdamageps(objTmpStats[intCFIndex]), 0, 0.854, 0.390, 0);
			Mstats_SetStatsPercentage(40, objTmpStats[intCFIndex].hits, MStats_Gethitsps(objTmpStats[intCFIndex]), 0, 1, 0.490, 0.192);
			Mstats_SetStatsPercentage(41, objTmpStats[intCFIndex].highestheal, MStats_Gethealsps(objTmpStats[intCFIndex]), 0, 1, 1, 1);
		end
	end

	--Wipe other rows
	top = top + 1;
	if (top < 11) then
		for i = top, 10, 1 do
			getglobal("MStatsFrameBar" .. i .. "Player"):SetText("");
			getglobal("MStatsFrameBar" .. i .. "Percent"):SetText("");
			getglobal("MStatsFrameBar" .. i .. "Value"):SetText("");
			getglobal("MStatsFrameBar" .. i):SetStatusBarColor(0, 0, 0, 0);
	  end
	end

	MStats_TimeSinceLastUpdate = 0;
end

function Mstats_SetStatsLine(intLineNumber, strText, intValue)
	if (((intLineNumber - intCurrentStatOffset) > 0) and ((intLineNumber - intCurrentStatOffset) < 11)) then
		getglobal("MStatsFrameBar" .. (intLineNumber - intCurrentStatOffset) .. "Player"):SetText(strText);
		getglobal("MStatsFrameBar" .. (intLineNumber - intCurrentStatOffset) .. "Value"):SetText(intValue);
	end
end

function Mstats_SetStatsPercentage(intLineNumber, intTotal, intHigh, intValue, r, g, b)
	if (((intLineNumber - intCurrentStatOffset) > 0) and ((intLineNumber - intCurrentStatOffset) < 11)) then
		local intPercentage, intBarSize;

		if (intValue > 0) then
			intPercentage = (floor((100 / intTotal) * (intValue * 100)) / 100);
			intBarSize = (floor((100 / intHigh) * (intValue * 100)) / 100);
		else
			intPercentage = 0;
			intBarSize = 0;
		end
		getglobal("MStatsFrameBar" .. (intLineNumber - intCurrentStatOffset) .. "Percent"):SetText(intPercentage .. "%");
		getglobal("MStatsFrameBar" .. (intLineNumber - intCurrentStatOffset)):SetMinMaxValues(0, 100);
		getglobal("MStatsFrameBar" .. (intLineNumber - intCurrentStatOffset)):SetValue(intBarSize);
		getglobal("MStatsFrameBar" .. (intLineNumber - intCurrentStatOffset)):SetStatusBarColor(r, g, b, 0.7);
	end
end

function MStats_FightsUpdate(elapsed)
	local intPercent = 0;
	local funcDataFunc, i, top, k, v;

	MStats_FightTimeSinceLastUpdate = MStats_FightTimeSinceLastUpdate + elapsed;

  if (MStats_FightTimeSinceLastUpdate < MStats_UpdateInterval) then
  	return;
  end

  if (intCurrentFight > 0) then
  	MStatsFightsFrameFightNumber:SetText("현재 결투: " .. intCurrentFight);
  else
  	if (objMStats.intLiveTotal == 1) then
  		MStatsFightsFrameFightNumber:SetText("현재 결투: 총합");
  	else
	  	MStatsFightsFrameFightNumber:SetText("현재 결투: 최종");
  	end
  end

  top = 0;
  for k,v in ipairs(objMStatsStatHistory) do
		if ((k > intCurrentFightOffset) and (k < (11 + intCurrentFightOffset))) then
			local hours, minutes;
			local strTemp;

			strTemp = k .. ".";
			if (string.len(strTemp) < 3) then
				strTemp = " " .. strTemp;
			end

			if (intCurrentFight == k) then
				getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset)):SetStatusBarColor(0.7, 0, 0.2, 1);
			else
				getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset)):SetStatusBarColor(0, 0, 0, 0);
			end

			if (objMStatsStatHistory[k].strFightName) then
				if (string.sub(objMStatsStatHistory[k].strFightName, 1, 6) == "보스: ") then
					getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset) .. "FightName"):SetText(strTemp .. " |c00FFF468" .. objMStatsStatHistory[k].strFightName);
				elseif (string.sub(objMStatsStatHistory[k].strFightName, 1, 5) == "TOTAL") then
					getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset) .. "FightName"):SetText(strTemp .. " |c00AAD372" .. objMStatsStatHistory[k].strFightName);
				elseif (string.sub(objMStatsStatHistory[k].strFightName, 1, 8) == "SUBTOTAL") then
					getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset) .. "FightName"):SetText(strTemp .. " |c00F48CBA" .. objMStatsStatHistory[k].strFightName);
				else
					getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset) .. "FightName"):SetText(strTemp .. " " .. objMStatsStatHistory[k].strFightName);
				end
			else
				getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset) .. "FightName"):SetText(strTemp .. " Unknown");
			end

			hours = floor(objMStatsStatHistory[k].intStartTime / 60);
			minutes = objMStatsStatHistory[k].intStartTime - (hours * 60);
			strTemp = MStats_MakeTimeString(hours,minutes);

			hours = floor(objMStatsStatHistory[k].intEndTime / 60);
			minutes = objMStatsStatHistory[k].intEndTime - (hours * 60);
			getglobal("MStatsFightsFrameBar" .. (k - intCurrentFightOffset) .. "Ended"):SetText(strTemp .. " - " .. MStats_MakeTimeString(hours,minutes));

			top = (k - intCurrentFightOffset);
		end
	end

	--Wipe other rows
	top = top + 1;
	if (top < 11) then
		for i = top, 10, 1 do
			getglobal("MStatsFightsFrameBar" .. i .. "FightName"):SetText("");
			getglobal("MStatsFightsFrameBar" .. i .. "Ended"):SetText("");
			getglobal("MStatsFightsFrameBar" .. i):SetStatusBarColor(0, 0, 0, 0);
		end
	end

	MStats_FightTimeSinceLastUpdate = 0;
end

function MStats_EditShow()
	if (intEditWindow == MSTATS_EW_FIGHTNAME) then
		if (intCurrentFight > 0) then
			MStatsEditFrameTitle:SetText("전투명 수정");
			MStatsEditFrameEditBox:SetText(objMStatsStatHistory[intCurrentFight].strFightName);
		else
			MStatsEditFrame:Hide();
		end
	elseif (intEditWindow == MSTATS_EW_CHANNELNAME) then
		MStatsEditFrameTitle:SetText("채널 대상 수정");
		MStatsEditFrameEditBox:SetText(objMStats.strAnnounceChannel);
	elseif (intEditWindow == MSTATS_EW_WHISPERNAME) then
		MStatsEditFrameTitle:SetText("귓말 대상 수정");
		MStatsEditFrameEditBox:SetText(objMStats.strAnnounceWhisper);
	end
end

function MStats_EditHide()
	if (intEditWindow == MSTATS_EW_FIGHTNAME) then
		if (intCurrentFight > 0) then
			objMStatsStatHistory[intCurrentFight].strFightName = MStatsEditFrameEditBox:GetText();
		end
	elseif (intEditWindow == MSTATS_EW_CHANNELNAME) then
		objMStats.strAnnounceChannel = MStatsEditFrameEditBox:GetText();
	elseif (intEditWindow == MSTATS_EW_WHISPERNAME) then
		objMStats.strAnnounceWhisper = MStatsEditFrameEditBox:GetText();
	end
end

function MStats_ExportShow()
	MStatsExportFrameTitle:SetText("전투기록 내보내기");
end

function DebugPrint(level, msg)
	if (objMStats.intDebugMode == 1) then
		if (level <= objMStats.intDebugLevel) then
			if getglobal("ChatFrame"..objMStats.intChatWindow) then
				getglobal("ChatFrame"..objMStats.intChatWindow):AddMessage("|c009382c9디버그 |c00FFFFFF" .. msg, 1,1,1);
			end
		end
	end
end

function MStats_MinimizeButton_OnClick(button)
	if (button == "LeftButton") then
		MStatsFrame:Hide();
		MStatsFightsFrame:Hide();
		objMStats.intGuiHidden = 1;
	elseif (button == "RightButton") then
		if (intMStatsProcessingDisabled == 0) then
			intMStatsProcessingDisabled = 1;
		else
			intMStatsProcessingDisabled = 0;
		end
	end
end

function MStats_MiniMapButtonPetsClick()
	if (objMStats.intTrackPets == 0) then
		objMStats.intTrackPets = 1;
		MStats_Print("이제 펫은 따로 기록합니다.");
	else
		objMStats.intTrackPets = 0;
		MStats_Print("이제 펫의 데이터를 플레이어 데이터와 병합해서 기록합니다.");
	end
end

function MStats_MiniMapButtonResetClick()
	objMStatsStatistics = {};
	objMStatsStatHistory = {};
	intMStatsProcessing = 0;
	strMStatsSyncOutLog = {};
	intMStatsSyncDirty = 0;
	MStats_UpdateTmpFightTotalObject();
	MStats_Print("초기화!");
end

function MStats_MiniMapButtonCheckClick()
	SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_SYNCCHECK .. UnitName("player"), "RAID");
	MStats_Print("동기 체크를 보냅니다.");
end

function MStats_MiniMapButtonTrackClick()
	if (objMStats.intTrackMobs == 0) then
		objMStats.intTrackMobs = 1;
		MStats_Print("이제 '플레이어 + 몬스터'를 기록합니다.");
	else
		objMStats.intTrackMobs = 0;
		MStats_Print("이제 플레이어만 기록합니다.");
	end
end

function MStats_MiniMapButtonOverHealClick()
	if (objMStats.intOverHeal == 0) then
		objMStats.intOverHeal = 1;
		MStats_Print("이제 초과치유량이 기록됩니다.");
	else
		objMStats.intOverHeal = 0;
		MStats_Print("이제 더이상 초과치유량이 기록되지 않습니다.");
	end
end

function MStats_MiniMapButtonGroupClick()
	if (objMStats.intPartyTrackOnly == 0) then
		objMStats.intPartyTrackOnly = 1;
		MStats_Print("이제 파티나 공격대에 참가했을 때만 기록합니다.");
	else
		objMStats.intPartyTrackOnly = 0;
		MStats_Print("이제 모든 전투에서 기록합니다.");
	end
end

function MStats_MiniMapButtonStatusClick()
	DEFAULT_CHAT_FRAME.editBox:SetText("/데미지 상태");
	ChatEdit_SendText(DEFAULT_CHAT_FRAME.editBox,0);
end

function MStats_MiniMapButtonBackClick()
	if (objMStats.strBackground == "background2") then
		objMStats.strBackground = "background";
	else
		objMStats.strBackground = "background2";
	end
	MStatsFrameMainBackdrop:SetTexture("Interface\\AddOns\\MinnaStats\\Textures\\" .. objMStats.strBackground);
end

function MStats_MiniMapButtonScaleClick()
	DEFAULT_CHAT_FRAME:AddMessage("|CFFFFFFFF그 상태로 채팅창에 |C00FF0000숫자 |CFFFFFFFF를 입력하세요. ")
	DEFAULT_CHAT_FRAME:AddMessage("|CFFFFFFFF숫자는 1~200 까지 가능합니다. (기본크기 100)")
	ChatFrame_OpenChat("/데미지 설정 크기 ");
end

function MStats_MiniMapButtonAlphaClick()
	DEFAULT_CHAT_FRAME:AddMessage("|CFFFFFFFF그 상태로 채팅창에 |C00FF0000숫자 |CFFFFFFFF를 입력하세요. ")
	DEFAULT_CHAT_FRAME:AddMessage("|CFFFFFFFF숫자는 1~100 까지 가능합니다. (기본 투명도 90)")
	ChatFrame_OpenChat("/데미지 설정 투명도 ");
end

function MStats_MiniMapButtonShowHideClick()
	if (MStatsFrame:IsShown()) then
		MStatsFrame:Hide();
		MStatsFightsFrame:Hide();
		objMStats.intGuiHidden = 1;
	else
		MStatsFrame:Show();
		-- Set frame width and height (to fix odd bug)
		MStatsFrame:SetWidth(230);
		MStatsFrame:SetHeight(200);
		objMStats.intGuiHidden = 0;
	end
end

function MStats_MiniMapButtonLockUnlockClick()
	if (objMStats.intGuiLocked == 0) then
		objMStats.intGuiLocked = 1;
		DEFAULT_CHAT_FRAME:AddMessage("데미지 미터기창이 |cffff0000잠겼습니다.")
	else
		objMStats.intGuiLocked = 0;
		DEFAULT_CHAT_FRAME:AddMessage("데미지 미터기창이 |cff00ff00풀렸습니다.")
	end
end

function MStats_FightBarClicked(button, bar)
	if (button == "LeftButton") then
		if (intCurrentFight == (bar + intCurrentFightOffset)) then
			intCurrentFight = 0;
		else
			intCurrentFight = bar + intCurrentFightOffset;
		end

		if (intCurrentFight > #objMStatsStatHistory) then
			intCurrentFight = 0;
		end
		if (strStatFighter == nil) then
			intCurrentStatOffset = 0;
		end
	elseif (button == "RightButton") then
		intCurrentFight = bar + intCurrentFightOffset;

		if (intCurrentFight > #objMStatsStatHistory) then
			intCurrentFight = 0;
		end
		if (strStatFighter == nil) then
			intCurrentStatOffset = 0;
		end

		if (intCurrentFight > 0) then
			MStats_ButtonFightEditMenu();
		end
	end
end

function MStats_StatBarClicked(button, bar)
	if (button == "LeftButton") then
		if (strStatFighter == nil) then
			-- No currently selected individual so find out who we clicked on
			local funcDataFunc, objTmpStats, intSelectedStatLine;

			funcDataFunc = MStats_GetDataValueFunction();
			intSelectedStatLine = bar + intCurrentStatOffset;

			if (intCurrentFight > 0) then
				objTmpStats = objMStatsStatHistory[intCurrentFight].objMStatsStatistics;
			else
				if (objMStats.intLiveTotal == 1) then
					objTmpStats = objMStatsTotalStatistics;
				else
					objTmpStats = objMStatsStatistics;
				end
			end

			intLineCounter = 1;
			for k,v in ipairs(objTmpStats) do
				if (MStats_FilteredClass(objTmpStats[k].name, objTmpStats[k].class)) then
					if ((intLineCounter > intCurrentStatOffset) and (intLineCounter < (11 + intCurrentStatOffset))) then
						local strTemp;

						if (intSelectedStatLine == intLineCounter) then
							if (funcDataFunc(objTmpStats[k]) > 0) then
								strStatFighter = objTmpStats[k].name;
								strStatFighterClass = objTmpStats[k].class;
								intCurrentStatOffset = 0;
							end
							break;
						end
					end

					intLineCounter = intLineCounter + 1;
				end
			end

		else
			-- As we are displaying data for a current individual, this click switches to a specific type of report
			local intSelectedStatLine;

			strStatFighter = nil;
			intSelectedStatLine = bar + intCurrentStatOffset;
			intCurrentStatOffset = 0;

			if (intSelectedStatLine == 1) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_ALL;
			elseif (intSelectedStatLine == 2) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_ALL;
			elseif (intSelectedStatLine == 3) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_ALL;
			elseif (intSelectedStatLine == 4) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETOSELF;
				objMStats.intDisplayMagicType = MSTATS_MT_ALL;
			elseif (intSelectedStatLine == 5) then
				objMStats.intDisplayType = MSTATS_DT_HEALINGDONE;
			elseif (intSelectedStatLine == 6) then
				objMStats.intDisplayType = MSTATS_DT_HEALINGNET;
			elseif (intSelectedStatLine == 7) then
				objMStats.intDisplayType = MSTATS_DT_OVERHEALING;
			elseif (intSelectedStatLine == 8) then
				objMStats.intDisplayType = MSTATS_DT_HEALINGTAKEN;
			elseif (intSelectedStatLine == 9) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_PHYSICAL;
			elseif (intSelectedStatLine == 10) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_FIRE;
			elseif (intSelectedStatLine == 11) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_FROST;
			elseif (intSelectedStatLine == 12) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_NATURE;
			elseif (intSelectedStatLine == 13) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_SHADOW;
			elseif (intSelectedStatLine == 14) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_ARCANE;
			elseif (intSelectedStatLine == 15) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
				objMStats.intDisplayMagicType = MSTATS_MT_HOLY;
			elseif (intSelectedStatLine == 16) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_PHYSICAL;
			elseif (intSelectedStatLine == 17) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_PHYSICAL;
			elseif (intSelectedStatLine == 18) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_FIRE;
			elseif (intSelectedStatLine == 19) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_FIRE;
			elseif (intSelectedStatLine == 20) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_FROST;
			elseif (intSelectedStatLine == 21) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_FROST;
			elseif (intSelectedStatLine == 22) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_NATURE;
			elseif (intSelectedStatLine == 23) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_NATURE;
			elseif (intSelectedStatLine == 24) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_SHADOW;
			elseif (intSelectedStatLine == 25) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_SHADOW;
			elseif (intSelectedStatLine == 26) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_ARCANE;
			elseif (intSelectedStatLine == 27) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_ARCANE;
			elseif (intSelectedStatLine == 28) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
				objMStats.intDisplayMagicType = MSTATS_MT_HOLY;
			elseif (intSelectedStatLine == 29) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
				objMStats.intDisplayMagicType = MSTATS_MT_HOLY;
			elseif (intSelectedStatLine == 30) then
				objMStats.intDisplayType = MSTATS_DT_CRITPERC;
			elseif (intSelectedStatLine == 31) then
				objMStats.intDisplayType = MSTATS_DT_MISSPERC;
			elseif (intSelectedStatLine == 32) then
				objMStats.intDisplayType = MSTATS_DT_VSMISSES;
			elseif (intSelectedStatLine == 33) then
				objMStats.intDisplayType = MSTATS_DT_DEATHS;
			elseif (intSelectedStatLine == 34) then
				objMStats.intDisplayType = MSTATS_DT_DECURSES;
			elseif (intSelectedStatLine == 35) then
				objMStats.intDisplayType = MSTATS_DT_HIGHESTHIT;
			elseif (intSelectedStatLine == 36) then
				objMStats.intDisplayType = MSTATS_DT_AVERAGEHIT;
			elseif (intSelectedStatLine == 37) then
				objMStats.intDisplayType = MSTATS_DT_HIGHESTHEAL;
			elseif (intSelectedStatLine == 38) then
				objMStats.intDisplayType = MSTATS_DT_AVERAGEHEAL;
			elseif (intSelectedStatLine == 39) then
				objMStats.intDisplayType = MSTATS_DT_DAMAGEPS;
			elseif (intSelectedStatLine == 40) then
				objMStats.intDisplayType = MSTATS_DT_HITSPS;
			elseif (intSelectedStatLine == 41) then
				objMStats.intDisplayType = MSTATS_DT_HEALINGPS;
			end
		end
	elseif (button == "RightButton") then
		-- Return report back to normal
		strStatFighter = nil;
		intCurrentStatOffset = 0;
	end
end

function MStats_StatsMouseWheel(direction)
	if (direction == -1) then
		intCurrentStatOffset = intCurrentStatOffset + 1;

		if (strStatFighter == nil) then
			if (intCurrentStatOffset > (intCurrentStatMaximumOffset - 10)) then
				intCurrentStatOffset = intCurrentStatMaximumOffset - 10;
			end
		else
			if (intCurrentStatOffset > 31) then
				intCurrentStatOffset = 31;
			end
		end
	elseif (direction == 1) then
		intCurrentStatOffset = intCurrentStatOffset - 1;
	end

	if (intCurrentStatOffset < 0) then
		intCurrentStatOffset = 0;
	end
end

function MStats_StatsUpButton()
	intCurrentStatOffset = 0;
end

function MStats_StatsDownButton()
	if (strStatFighter == nil) then
		intCurrentStatOffset = intCurrentStatMaximumOffset - 10;
	else
		intCurrentStatOffset = 31;
	end

	if (intCurrentStatOffset < 0) then
		intCurrentStatOffset = 0;
	end
end

function MStats_StatsLeftButton()
	MStats_StatsMouseWheel(1);
end

function MStats_StatsRightButton()
	MStats_StatsMouseWheel(-1);
end

function MStats_FightsMouseWheel(direction)
	if (direction == -1) then
		intCurrentFightOffset = intCurrentFightOffset + 1;

		if (intCurrentFightOffset > (#objMStatsStatHistory - 10)) then
			intCurrentFightOffset = #objMStatsStatHistory - 10;
		end
	elseif (direction == 1) then
		intCurrentFightOffset = intCurrentFightOffset - 1;
	end

	if (intCurrentFightOffset < 0) then
		intCurrentFightOffset = 0;
	end
end

function MStats_FightUpButton()
	intCurrentFightOffset = 0;
end

function MStats_FightDownButton()
	intCurrentFightOffset = #objMStatsStatHistory - 10;

	if (intCurrentFightOffset < 0) then
		intCurrentFightOffset = 0;
	end
end

function MStats_FightLeftButton()
	MStats_FightsMouseWheel(1);
end

function MStats_FightRightButton()
	MStats_FightsMouseWheel(-1)
end

function MStats_ButtonFight()
	if (MStatsFightsFrame:IsShown()) then
		MStatsFightsFrame:Hide();
		intCurrentFight = 0;
	else
		MStatsFightsFrame:Show();
	end
end

function MStats_SetBarColour(class, bar)
	local alpha = 0.7;
	if class == "사냥꾼" then
		bar:SetStatusBarColor(0.66, 0.82, 0.45, alpha);
	elseif class == "흑마법사" then
		bar:SetStatusBarColor(0.57, 0.51, 0.78, alpha);
	elseif class == "사제" then
		bar:SetStatusBarColor(1, 1, 1, alpha);
	elseif class == "성기사" then
		bar:SetStatusBarColor(0.95, 0.55, 0.73, alpha);
	elseif class == "마법사" then
		bar:SetStatusBarColor(0.41, 0.8, 0.93, alpha);
	elseif class == "도적" then
		bar:SetStatusBarColor(1, 0.95, 0.41, alpha);
	elseif class == "드루이드" then
		bar:SetStatusBarColor(1, 0.48, 0.04, alpha);
	elseif class == "주술사" then
		bar:SetStatusBarColor(0.14, 0.35, 1.0, alpha);
	elseif class == "전사" then
		bar:SetStatusBarColor(0.77, 0.61, 0.43, alpha);
	elseif class == "죽음의 기사" then
		bar:SetStatusBarColor(1, 0, 0, alpha);
	elseif class == "Pet" then
		bar:SetStatusBarColor(0.88, 0.76, 0.98, alpha);
	elseif class == "Mob" then
		bar:SetStatusBarColor(1, 0, 0, alpha);
	end
end
--[[-------------------------------------------
****************** STAT TABLE *****************
---------------------------------------------]]

function MStats_GetDataValueFunction()
	if (objMStats.intDisplayType == MSTATS_DT_DAMAGEDONE) then
		if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
			return MStats_Getdmgdone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
			return MStats_Getphysicaldone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
			return MStats_Getfiredone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
			return MStats_Getfrostdone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
			return MStats_Getnaturedone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
			return MStats_Getshadowdone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then
			return MStats_Getarcanedone;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
			return MStats_Getholydone;
		end
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGETAKEN) then
		if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
			return MStats_Getdmgtaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
			return MStats_Getphysicaltaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
			return MStats_Getfiretaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
			return MStats_Getfrosttaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
			return MStats_Getnaturetaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
			return MStats_Getshadowtaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then
			return MStats_Getarcanetaken;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
			return MStats_Getholytaken;
		end
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGEABSORBED) then
		if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
			return MStats_Getdmgabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
			return MStats_Getphysicalabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
			return MStats_Getfireabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
			return MStats_Getfrostabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
			return MStats_Getnatureabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
			return MStats_Getshadowabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then

			return MStats_Getarcaneabsorbed;
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
			return MStats_Getholyabsorbed;
		end
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGDONE) then
		return MStats_Gethealingdone;
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGTAKEN) then
		return MStats_Gethealingtaken;
	elseif (objMStats.intDisplayType == MSTATS_DT_OVERHEALING) then
		return MStats_Getoverhealingdone;
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGNET) then
		return MStats_Gethealingnet;
	elseif (objMStats.intDisplayType == MSTATS_DT_CRITS) then
		return MStats_Getcrits;
	elseif (objMStats.intDisplayType == MSTATS_DT_CRITPERC) then
		return MStats_Getcritperc;
	elseif (objMStats.intDisplayType == MSTATS_DT_DEATHS) then
		return MStats_Getdeaths;
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGETOSELF) then
		return MStats_Getdamagetoself;
	elseif (objMStats.intDisplayType == MSTATS_DT_MISSES) then
		return MStats_Getmisses;
	elseif (objMStats.intDisplayType == MSTATS_DT_MISSPERC) then
		return MStats_Getmissperc;
	elseif (objMStats.intDisplayType == MSTATS_DT_VSMISSES) then
		return MStats_Getvsmisses;
	elseif (objMStats.intDisplayType == MSTATS_DT_DECURSES) then
		return MStats_Getdecurses;
	elseif (objMStats.intDisplayType == MSTATS_DT_HIGHESTHIT) then
		return MStats_Gethighesthit;
	elseif (objMStats.intDisplayType == MSTATS_DT_AVERAGEHIT) then
		return MStats_Getaveragehit;
	elseif (objMStats.intDisplayType == MSTATS_DT_HIGHESTHEAL) then
		return MStats_Gethighestheal;
	elseif (objMStats.intDisplayType == MSTATS_DT_AVERAGEHEAL) then
		return MStats_Getaverageheal;
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGEPS) then
		return MStats_Getdamageps;
	elseif (objMStats.intDisplayType == MSTATS_DT_HITSPS) then
		return MStats_Gethitsps;
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGPS) then
		return MStats_Gethealsps;
	elseif (objMStats.intDisplayType == MSTATS_DT_EFFICIENCY) then
		return MStats_Getefficiency;
	elseif (objMStats.intDisplayType == MSTATS_DT_LIABILITY) then
		return MStats_Getliability;
	elseif (objMStats.intDisplayType == MSTATS_DT_RAIDWORTH) then
		return MStats_Getraidworth;
	end
end

function MStats_GetDataTypeString()
	if (objMStats.intDisplayType == MSTATS_DT_DAMAGEDONE) then
		if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
			return "준 피해량 (전체속성)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
			return "준 피해량 (물리)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
			return "준 피해량 (화염)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
			return "준 피해량 (냉기)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
			return "준 피해량 (자연)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
			return "준 피해량 (암흑)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then
			return "준 피해량 (비전)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
			return "준 피해량 (신성)";
		end
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGETAKEN) then
		if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
			return "받은 피해량 (전체속성)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
			return "받은 피해량 (물리)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
			return "받은 피해량 (화염)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
			return "받은 피해량 (냉기)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
			return "받은 피해량 (자연)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
			return "받은 피해량 (암흑)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then
			return "받은 피해량 (비전)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
			return "받은 피해량 (신성)";
		end
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGEABSORBED) then
		if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
			return "흡수된 피해량 (전체속성)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
			return "흡수된 피해량 (물리)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
			return "흡수된 피해량 (화염)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
			return "흡수된 피해량 (냉기)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
			return "흡수된 피해량 (자연)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
			return "흡수된 피해량 (암흑)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then
			return "흡수된 피해량 (비전)";
		elseif (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
			return "흡수된 피해량 (신성)";
		end
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGDONE) then
		return "치유량 (총합)";
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGTAKEN) then
		return "받은 치유량";
	elseif (objMStats.intDisplayType == MSTATS_DT_OVERHEALING) then
		return "초과 치유량";
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGNET) then
		return "치유량 (최종)";
	elseif (objMStats.intDisplayType == MSTATS_DT_CRITS) then
		return "치명타";
	elseif (objMStats.intDisplayType == MSTATS_DT_CRITPERC) then
		return "치명타율 %";
	elseif (objMStats.intDisplayType == MSTATS_DT_DEATHS) then
		return "죽음";
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGETOSELF) then
		return "자신의 피해량";
	elseif (objMStats.intDisplayType == MSTATS_DT_MISSES) then
		return "실패";
	elseif (objMStats.intDisplayType == MSTATS_DT_MISSPERC) then
		return "실패율 %";
	elseif (objMStats.intDisplayType == MSTATS_DT_VSMISSES) then
		return "Vs 실패";
	elseif (objMStats.intDisplayType == MSTATS_DT_DECURSES) then
		return "해제";
	elseif (objMStats.intDisplayType == MSTATS_DT_HIGHESTHIT) then
		return "최대 명중";
	elseif (objMStats.intDisplayType == MSTATS_DT_AVERAGEHIT) then
		return "평균 명중";
	elseif (objMStats.intDisplayType == MSTATS_DT_HIGHESTHEAL) then
		return "최대 치유";
	elseif (objMStats.intDisplayType == MSTATS_DT_AVERAGEHEAL) then
		return "평균 치유";
	elseif (objMStats.intDisplayType == MSTATS_DT_DAMAGEPS) then
		return "피해량/초 (" .. tostring(MStats_ReturnFightDuration() .. ")");
	elseif (objMStats.intDisplayType == MSTATS_DT_HITSPS) then
		return "명중/초 (" .. tostring(MStats_ReturnFightDuration() .. ")");
	elseif (objMStats.intDisplayType == MSTATS_DT_HEALINGPS) then
		return "치유량/초 (" .. tostring(MStats_ReturnFightDuration() .. ")");
	elseif (objMStats.intDisplayType == MSTATS_DT_EFFICIENCY) then
		return "효율";
	elseif (objMStats.intDisplayType == MSTATS_DT_LIABILITY) then
		return "부담";
	elseif (objMStats.intDisplayType == MSTATS_DT_RAIDWORTH) then
		return "공대 기여도";
	end
end

function MStats_Getdmgdone(k)
	return k.dmgdone;
end

function MStats_Getphysicaldone(k)
	return k.physicaldone;
end

function MStats_Getfiredone(k)
	return k.firedone;
end

function MStats_Getfrostdone(k)
	return k.frostdone;
end

function MStats_Getnaturedone(k)
	return k.naturedone;
end

function MStats_Getshadowdone(k)
	return k.shadowdone;
end

function MStats_Getarcanedone(k)
	return k.arcanedone;
end

function MStats_Getholydone(k)
	return k.holydone;
end

function MStats_Getdmgtaken(k)
	return k.dmgtaken;
end

function MStats_Getphysicaltaken(k)
	return k.physicaltaken;
end

function MStats_Getfiretaken(k)
	return k.firetaken;
end

function MStats_Getfrosttaken(k)
	return k.frosttaken;
end

function MStats_Getnaturetaken(k)
	return k.naturetaken;
end

function MStats_Getshadowtaken(k)
	return k.shadowtaken;
end

function MStats_Getarcanetaken(k)
	return k.arcanetaken;
end

function MStats_Getholytaken(k)
	return k.holytaken;
end

function MStats_Getdmgabsorbed(k)
	return k.dmgabsorbed;
end

function MStats_Getphysicalabsorbed(k)
	return k.physicalabsorbed;
end

function MStats_Getfireabsorbed(k)
	return k.fireabsorbed;
end

function MStats_Getfrostabsorbed(k)
	return k.frostabsorbed;
end

function MStats_Getnatureabsorbed(k)
	return k.natureabsorbed;
end

function MStats_Getshadowabsorbed(k)
	return k.shadowabsorbed;
end

function MStats_Getarcaneabsorbed(k)
	return k.arcaneabsorbed;
end

function MStats_Getholyabsorbed(k)
	return k.holyabsorbed;
end

function MStats_Gethealingdone(k)
	return k.healingdone;
end

function MStats_Gethealingtaken(k)
	return k.healingtaken;
end

function MStats_Getoverhealingdone(k)
	return k.overhealingdone;
end

function MStats_Gethealingnet(k)
	return k.healingdone - k.overhealingdone;
end

function MStats_Getcrits(k)
	return k.crits;
end

function MStats_Getcritperc(k)
	if ((k.noncrits + k.crits) == 0) then
		return 0;
	end

	return (floor((100 / (k.noncrits + k.crits)) * (k.crits) * 100) / 100);
end

function MStats_Getdeaths(k)
	return k.deaths;
end

function MStats_Getdamagetoself(k)
	return k.dmgtoself;
end

function MStats_Getmisses(k)
	return k.misses;
end

function MStats_Getmissperc(k)
	if ((k.hits + k.misses) == 0) then
		return 0;
	end

	return (floor((100 / (k.hits + k.misses)) * (k.misses) * 100) / 100);
end

function MStats_Getvsmisses(k)
	return k.vsmisses;
end

function MStats_Getdecurses(k)
	return k.decurses;
end

function MStats_Gethighesthit(k)
	return k.highesthit;
end

function MStats_Getaveragehit(k)
	if ((k.hits == 0) or (k.dmgdone == 0)) then
		return 0;
	end

	return floor(k.dmgdone / k.hits);
end

function MStats_Gethighestheal(k)
	return k.highestheal;
end

function MStats_Getaverageheal(k)
	if (((k.noncrits + k.crits) - k.hits) <= 0) then
		return 0;
	end

	if (k.healingdone == 0) then
		return 0;
	end

	return floor(k.healingdone / ((k.noncrits + k.crits) - k.hits));
end

function MStats_Getdamageps(k)
	local intDuration;

	intDuration = MStats_ReturnFightDuration();
	if (intDuration == 0) then
		return 0;
	end

	if (k.dmgdone == 0) then
		return 0;
	end

	return (floor(k.dmgdone / intDuration));
end

function MStats_Gethitsps(k)
	local intDuration;

	intDuration = MStats_ReturnFightDuration();
	if (intDuration == 0) then
		return 0;
	end

	if (k.hits == 0) then
		return 0;
	end

	return (floor((k.hits / intDuration) * 100)) / 100;
end

function MStats_Gethealsps(k)
	local intDuration;

	intDuration = MStats_ReturnFightDuration();
	if (intDuration == 0) then
		return 0;
	end

	if (k.healingdone == 0) then
		return 0;
	end

	return (floor(k.healingdone / intDuration));
end

function MStats_Getefficiency(k)
	if ((k.dmgdone + k.dmgtaken + k.healingdone) == 0) then
		return 0;
	end

	return (floor((100 / (k.dmgdone + k.dmgtaken + k.healingdone)) * (k.dmgdone + k.healingdone) * 100) / 100);
end

function MStats_Getliability(k)
	if ((k.dmgdone + k.dmgtaken + k.healingtaken + k.healingdone) == 0) then
		return 0;
	end

	return (floor((100 / (k.dmgdone + k.dmgtaken + k.healingtaken + k.healingdone)) * (k.dmgtaken + k.healingtaken) * 100) / 100);
end

function MStats_Getraidworth(k)
	return k.dmgdone + k.healingdone + (k.decurses * 500);
end

function MStats_ReturnFightDuration()
	if (intCurrentFight > 0) then
		return objMStatsStatHistory[intCurrentFight].intDuration;
	else
		if (objMStats.intLiveTotal == 1) then
			return (objMStatsTotalStatistics.intDuration + (objMStats.intFightLocalEndTime - objMStats.intFightLocalStartTime));
		else
			return (objMStats.intFightLocalEndTime - objMStats.intFightLocalStartTime);
		end
	end
end

function MStats_RecordDamage(source, srcflags, target, targflags, damage, type, crit, resisted, absorbed)
	local intSource, intTarget, intTotSource, intTotTarget, strClass;
	local srcFriendly, targFriendly;

	if not (resisted) then
		resisted = 0;
	end
	if not (absorbed) then
		absorbed = 0;
	end

	if (bit.band(srcflags, COMBATLOG_OBJECT_AFFILIATION_MASK) == COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) then
		srcFriendly = false;
	else
		srcFriendly = true;
	end

	if (bit.band(targflags, COMBATLOG_OBJECT_AFFILIATION_MASK) == COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) then
		targFriendly = false;
	else
		targFriendly = true;
	end

	-- If someone in the party or raid takes or gives damage to another target, reset incombat timer
	if ((source and target) and (srcFriendly or targFriendly) and not (source == target)) then
		if (intLogInCombat == 0) then
			MStats_AddFightToHistory();

			intLogInCombat = 1;
			intTimeOutOfCombat = 0;
			objMStats.intFightStartTime = (intLogLastHour * 60) + intLogLastMinute;
			objMStats.intFightLocalStartTime = intLogLastSeconds;
			SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_INCOMBAT .. intLogLastHour .. ":" .. intLogLastMinute, "RAID");
		else
			--Extend in combat time
			intTimeOutOfCombat = 0;
			objMStats.intFightLocalEndTime = intLogLastSeconds;
			objMStats.intFightEndTime = (intLogLastHour * 60) + intLogLastMinute;
		end
	end

	-- Get index to player data in stats
	intSource,intTarget,intTotSource,intTotTarget = MStats_FindOrAddPlayersAndOrMobs(source, srcFriendly, target, targFriendly);

	-- Only update stats for source if one exists and this isn't self damage
	if not (intSource == intTarget) and (intSource) then
		-- Update stats for this player

		-- Update damage done
		objMStatsStatistics[intSource].dmgdone = objMStatsStatistics[intSource].dmgdone + damage;
		objMStatsTotalStatistics[intTotSource].dmgdone = objMStatsTotalStatistics[intTotSource].dmgdone + damage;

		-- Update crits done
		if (crit == 1) then
			objMStatsStatistics[intSource].crits = objMStatsStatistics[intSource].crits + 1;
			objMStatsTotalStatistics[intTotSource].crits = objMStatsTotalStatistics[intTotSource].crits + 1;
		else
			objMStatsStatistics[intSource].noncrits = objMStatsStatistics[intSource].noncrits + 1;
			objMStatsTotalStatistics[intTotSource].noncrits = objMStatsTotalStatistics[intTotSource].noncrits + 1;
		end

		-- Update hits done
		objMStatsStatistics[intSource].hits = objMStatsStatistics[intSource].hits + 1;
		objMStatsTotalStatistics[intTotSource].hits = objMStatsTotalStatistics[intTotSource].hits + 1;

		-- Update highest damage
		objMStatsStatistics[intSource].highesthit = math.max(objMStatsStatistics[intSource].highesthit, damage);
		objMStatsTotalStatistics[intTotSource].highesthit = math.max(objMStatsTotalStatistics[intTotSource].highesthit, damage);

		-- Update damage type
		if (type == 4) then
			objMStatsStatistics[intSource].firedone = objMStatsStatistics[intSource].firedone + damage;
			objMStatsTotalStatistics[intTotSource].firedone = objMStatsTotalStatistics[intTotSource].firedone + damage;
		elseif (type == 16) then
			objMStatsStatistics[intSource].frostdone = objMStatsStatistics[intSource].frostdone + damage;
			objMStatsTotalStatistics[intTotSource].frostdone = objMStatsTotalStatistics[intTotSource].frostdone + damage;
		elseif (type == 8) then
			objMStatsStatistics[intSource].naturedone = objMStatsStatistics[intSource].naturedone + damage;
			objMStatsTotalStatistics[intTotSource].naturedone = objMStatsTotalStatistics[intTotSource].naturedone + damage;
		elseif (type == 32) then
			objMStatsStatistics[intSource].shadowdone = objMStatsStatistics[intSource].shadowdone + damage;
			objMStatsTotalStatistics[intTotSource].shadowdone = objMStatsTotalStatistics[intTotSource].shadowdone + damage;
		elseif (type ==2) then
			objMStatsStatistics[intSource].holydone = objMStatsStatistics[intSource].holydone + damage;
			objMStatsTotalStatistics[intTotSource].holydone = objMStatsTotalStatistics[intTotSource].holydone + damage;
		elseif (type == 64) then
			objMStatsStatistics[intSource].arcanedone = objMStatsStatistics[intSource].arcanedone + damage;
			objMStatsTotalStatistics[intTotSource].arcanedone = objMStatsTotalStatistics[intTotSource].arcanedone + damage;
		else
			objMStatsStatistics[intSource].physicaldone = objMStatsStatistics[intSource].physicaldone + damage;
			objMStatsTotalStatistics[intTotSource].physicaldone = objMStatsTotalStatistics[intTotSource].physicaldone + damage;
		end

		intMStatsSyncDirty = 1;
	end

	if (intTarget) then
		-- Update stats for this player

		-- Update damage taken
		objMStatsStatistics[intTarget].dmgtaken = objMStatsStatistics[intTarget].dmgtaken + damage;
		objMStatsTotalStatistics[intTotTarget].dmgtaken = objMStatsTotalStatistics[intTotTarget].dmgtaken + damage;

		-- Update self damage taken
		if (intSource == intTarget) then
			objMStatsStatistics[intTarget].dmgtoself = objMStatsStatistics[intTarget].dmgtoself + damage;
			objMStatsTotalStatistics[intTotTarget].dmgtoself = objMStatsTotalStatistics[intTotTarget].dmgtoself + damage;
		end

		-- Update hits done
		objMStatsStatistics[intTarget].vshits = objMStatsStatistics[intTarget].vshits + 1;
		objMStatsTotalStatistics[intTotTarget].vshits = objMStatsTotalStatistics[intTotTarget].vshits + 1;

		-- Update damage absorbed
		objMStatsStatistics[intTarget].dmgabsorbed = objMStatsStatistics[intTarget].dmgabsorbed + absorbed;
		objMStatsTotalStatistics[intTotTarget].dmgabsorbed = objMStatsTotalStatistics[intTotTarget].dmgabsorbed + absorbed;

		-- Update damage type
		if (type == 4) then
			objMStatsStatistics[intTarget].firetaken = objMStatsStatistics[intTarget].firetaken + damage;
			objMStatsTotalStatistics[intTotTarget].firetaken = objMStatsTotalStatistics[intTotTarget].firetaken + damage;
			objMStatsStatistics[intTarget].fireabsorbed = objMStatsStatistics[intTarget].fireabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].fireabsorbed = objMStatsTotalStatistics[intTotTarget].fireabsorbed + absorbed;
		elseif (type == 16) then
			objMStatsStatistics[intTarget].frosttaken = objMStatsStatistics[intTarget].frosttaken + damage;
			objMStatsTotalStatistics[intTotTarget].frosttaken = objMStatsTotalStatistics[intTotTarget].frosttaken + damage;
			objMStatsStatistics[intTarget].frostabsorbed = objMStatsStatistics[intTarget].frostabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].frostabsorbed = objMStatsTotalStatistics[intTotTarget].frostabsorbed + absorbed;
		elseif (type == 8) then
			objMStatsStatistics[intTarget].naturetaken = objMStatsStatistics[intTarget].naturetaken + damage;
			objMStatsTotalStatistics[intTotTarget].naturetaken = objMStatsTotalStatistics[intTotTarget].naturetaken + damage;
			objMStatsStatistics[intTarget].natureabsorbed = objMStatsStatistics[intTarget].natureabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].natureabsorbed = objMStatsTotalStatistics[intTotTarget].natureabsorbed + absorbed;
		elseif (type == 32) then
			objMStatsStatistics[intTarget].shadowtaken = objMStatsStatistics[intTarget].shadowtaken + damage;
			objMStatsTotalStatistics[intTotTarget].shadowtaken = objMStatsTotalStatistics[intTotTarget].shadowtaken + damage;
			objMStatsStatistics[intTarget].shadowabsorbed = objMStatsStatistics[intTarget].shadowabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].shadowabsorbed = objMStatsTotalStatistics[intTotTarget].shadowabsorbed + absorbed;
		elseif (type == 2) then
			objMStatsStatistics[intTarget].holytaken = objMStatsStatistics[intTarget].holytaken + damage;
			objMStatsTotalStatistics[intTotTarget].holytaken = objMStatsTotalStatistics[intTotTarget].holytaken + damage;
			objMStatsStatistics[intTarget].holyabsorbed = objMStatsStatistics[intTarget].holyabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].holyabsorbed = objMStatsTotalStatistics[intTotTarget].holyabsorbed + absorbed;
		elseif (type == 64) then
			objMStatsStatistics[intTarget].arcanetaken = objMStatsStatistics[intTarget].arcanetaken + damage;
			objMStatsTotalStatistics[intTotTarget].arcanetaken = objMStatsTotalStatistics[intTotTarget].arcanetaken + damage;
			objMStatsStatistics[intTarget].arcaneabsorbed = objMStatsStatistics[intTarget].arcaneabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].arcaneabsorbed = objMStatsTotalStatistics[intTotTarget].arcaneabsorbed + absorbed;
		else
			objMStatsStatistics[intTarget].physicaltaken = objMStatsStatistics[intTarget].physicaltaken + damage;
			objMStatsTotalStatistics[intTotTarget].physicaltaken = objMStatsTotalStatistics[intTotTarget].physicaltaken + damage;
			objMStatsStatistics[intTarget].physicalabsorbed = objMStatsStatistics[intTarget].physicalabsorbed + absorbed;
			objMStatsTotalStatistics[intTotTarget].physicalabsorbed = objMStatsTotalStatistics[intTotTarget].physicalabsorbed + absorbed;
		end

		intMStatsSyncDirty = 1;
	end
end

function MStats_RecordHealing(source, srcflags, target, targflags, healing, crit)
	local intSource, intTarget, intTotSource, intTotTarget, strClass;
	local srcFriendly, targFriendly;

	if not (crit) then
		crit = 0;
	end

	if (bit.band(srcflags, COMBATLOG_OBJECT_AFFILIATION_MASK) == COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) then
		srcFriendly = false;
	else
		srcFriendly = true;
	end

	if (bit.band(targflags, COMBATLOG_OBJECT_AFFILIATION_MASK) == COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) then
		targFriendly = false;
	else
		targFriendly = true;
	end

	-- Get index to player data in stats
	intSource,intTarget,intTotSource,intTotTarget = MStats_FindOrAddPlayersAndOrMobs(source, srcFriendly, target, targFriendly);

	if (intSource) then
		-- Update stats for this player

		-- Update healing done
		objMStatsStatistics[intSource].healingdone = objMStatsStatistics[intSource].healingdone + healing;
		objMStatsTotalStatistics[intTotSource].healingdone = objMStatsTotalStatistics[intTotSource].healingdone + healing;

		-- Update crits done
		if (crit == 1) then
			objMStatsStatistics[intSource].crits = objMStatsStatistics[intSource].crits + 1;
			objMStatsTotalStatistics[intTotSource].crits = objMStatsTotalStatistics[intTotSource].crits + 1;
		else
			objMStatsStatistics[intSource].noncrits = objMStatsStatistics[intSource].noncrits + 1;
			objMStatsTotalStatistics[intTotSource].noncrits = objMStatsTotalStatistics[intTotSource].noncrits + 1;
		end

		-- Update highest healing
		objMStatsStatistics[intSource].highestheal = math.max(objMStatsStatistics[intSource].highestheal, healing);
		objMStatsTotalStatistics[intTotSource].highestheal = math.max(objMStatsTotalStatistics[intTotSource].highestheal, healing);

		intMStatsSyncDirty = 1;
	end

	if (intTarget) then
		-- Update stats for this player

		-- Update overhealing done
		if (intSource) then
			if (objMStatsStatistics[intTarget].unitid) then
				local intUnitHealth, intUnitMaxHealth;

				intUnitHealth = UnitHealth(objMStatsStatistics[intTarget].unitid);
				intUnitMaxHealth = UnitHealthMax(objMStatsStatistics[intTarget].unitid);
				if ((intUnitHealth + healing) > intUnitMaxHealth) then
					objMStatsStatistics[intSource].overhealingdone = objMStatsStatistics[intSource].overhealingdone + ((intUnitHealth + healing) - intUnitMaxHealth);
					objMStatsTotalStatistics[intTotSource].overhealingdone = objMStatsTotalStatistics[intTotSource].overhealingdone + ((intUnitHealth + healing) - intUnitMaxHealth);
				end
			end
		end

		-- Update healing taken
		objMStatsStatistics[intTarget].healingtaken = objMStatsStatistics[intTarget].healingtaken + healing;
		objMStatsTotalStatistics[intTotTarget].healingtaken = objMStatsTotalStatistics[intTotTarget].healingtaken + healing;

		intMStatsSyncDirty = 1;
	end
end

function MStats_RecordDraining(source, srcFriendly, target, targFriendly, damage, type)
	-- Add to everything objMStatsTotalStatistics
	--if (objMStats.intTrackMobs == 1) then
	--intSource,intTarget,intTotSource,intTotTarget = MStats_FindOrAddPlayersAndOrMobs(source, srcFriendly, target, targFriendly);
end

function MStats_RecordDeath(target, targFriendly)
	local intTarget, intTotTarget;

	if (bit.band(targflags, COMBATLOG_OBJECT_AFFILIATION_MASK) == COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) then
		targFriendly = false;
	else
		targFriendly = true;
	end

	-- Add to everything objMStatsTotalStatistics
	--if (objMStats.intTrackMobs == 1) then

	-- Get index to player data in stats
	_,intTarget,_,intTotTarget = MStats_FindOrAddPlayersAndOrMobs(nil, nil, target, targFriendly);

	if (intTarget) then
		-- Update stats for this player but only if it isn't a pet (in which case it's name will differ from the name of the target)
		if (target == objMStatsStatistics[intTarget].name) then
			objMStatsStatistics[intTarget].deaths = objMStatsStatistics[intTarget].deaths + 1;
			objMStatsTotalStatistics[intTotTarget].deaths = objMStatsTotalStatistics[intTotTarget].deaths + 1;

			intMStatsSyncDirty = 1;
		end
	end
end

function MStats_RecordMisses(source, srcFriendly, target, targFriendly)
	local intSource, intTarget, intTotSource, intTotTarget;

	-- Add to everything objMStatsTotalStatistics
	--if (objMStats.intTrackMobs == 1) then

	-- If someone in the party or raid misses or is missed vs another target, reset incombat timer
	if ((source and target) and (MStats_InRaid(source) or MStats_InRaid(target))) then
		if (intLogInCombat == 0) then
			MStats_AddFightToHistory();

			intLogInCombat = 1;
			intTimeOutOfCombat = 0;
			objMStats.intFightStartTime = (intLogLastHour * 60) + intLogLastMinute;
			objMStats.intFightLocalStartTime = intLogLastSeconds;
			SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_INCOMBAT .. intLogLastHour .. ":" .. intLogLastMinute, "RAID");
		else
			--Extend in combat time
			intTimeOutOfCombat = 0;
			objMStats.intFightEndTime = (intLogLastHour * 60) + intLogLastMinute;
		end
	end

	-- Get index to player data in stats
	intSource,intTarget,intTotSource,intTotTarget = MStats_FindOrAddPlayersAndOrMobs(source, srcFriendly, target, targFriendly);

	if (intSource) then
		-- Update stats for this player
		objMStatsStatistics[intSource].misses = objMStatsStatistics[intSource].misses + 1;
		objMStatsTotalStatistics[intTotSource].misses = objMStatsTotalStatistics[intTotSource].misses + 1;

		intMStatsSyncDirty = 1;
	end

	if (intTarget) then
		-- Update stats for this player
		objMStatsStatistics[intTarget].vsmisses = objMStatsStatistics[intTarget].vsmisses + 1;
		objMStatsTotalStatistics[intTotTarget].vsmisses = objMStatsTotalStatistics[intTotTarget].vsmisses + 1;

		intMStatsSyncDirty = 1;
	end
end

function MStats_RecordDecurse(source, srcFriendly)
	local intSource, intTotSource;

	-- Add to everything objMStatsTotalStatistics
	--if (objMStats.intTrackMobs == 1) then

	-- Get index to player data in stats
	intSource,_,intTotSource = MStats_FindOrAddPlayersAndOrMobs(source, srcFriendly, nil, nil);

	if (intSource) then
		-- Update stats for this player
		objMStatsStatistics[intSource].decurses = objMStatsStatistics[intSource].decurses + 1;
		objMStatsTotalStatistics[intTotSource].decurses = objMStatsTotalStatistics[intTotSource].decurses + 1;

		intMStatsSyncDirty = 1;
	end
end

function MStats_FindOrAddPlayer(playername)
	local intPlayerIndex, strClass, intPosition;

	-- Get index to player data in stats
	intPlayerIndex = MStats_FindPlayerObj(playername);

	if not (intPlayerIndex) then
		-- Check if player is in current raid or party
		strClass, intPosition = MStats_InRaid(playername);

		-- Add player to stats if in raid or party (ie. has a valid class)
		if (strClass) then
			-- Check to see if pet owner already exists in table
			if ((objMStats.intTrackPets == 0) and (strClass == "Pet")) then
				playername = MStats_GetRaidName(intPosition);
				strClass = MStats_GetRaidClass(intPosition);
				intPlayerIndex = MStats_FindPlayerObj(playername);
			end

			-- Create new entry if still required
			if not (intPlayerIndex) then
				intPlayerIndex = #objMStatsStatistics + 1;
				table.insert(objMStatsStatistics, MStats_MakePlayerObj(playername, strClass));
			end
		end
	end

	return intPlayerIndex;
end

function MStats_FindOrAddTotPlayer(playername)
	local intPlayerIndex, strClass, intPosition;

	-- Get index to player data in stats
	intPlayerIndex = MStats_FindTotPlayerObj(playername);

	if not (intPlayerIndex) then
		-- Check if player is in current raid or party
		strClass, intPosition = MStats_InRaid(playername);

		-- If not in current raid or party then must be a mob
		if not (strClass) then
			strClass = "Mob";
		end

		-- Check to see if pet owner already exists in table
		if ((objMStats.intTrackPets == 0) and (strClass == "Pet")) then
			playername = MStats_GetRaidName(intPosition);
			strClass = MStats_GetRaidClass(intPosition);
			intPlayerIndex = MStats_FindTotPlayerObj(playername);
		end

		-- Create new entry if still required
		if not (intPlayerIndex) then
			intPlayerIndex = #objMStatsTotalStatistics + 1;
			table.insert(objMStatsTotalStatistics, MStats_MakePlayerObj(playername, strClass));
		end
	end

	return intPlayerIndex;
end

function MStats_FindOrAddMob(mobname)
	local intPlayerIndex;

	-- Get index to player data in stats
	intPlayerIndex = MStats_FindPlayerObj(mobname);

	if not (intPlayerIndex) then
		intPlayerIndex = #objMStatsStatistics + 1;
		table.insert(objMStatsStatistics, MStats_MakePlayerObj(mobname, "Mob"));
	end

	return intPlayerIndex;
end

function MStats_FindOrAddPlayerOrMob(playername)
	local intIndex;

	intIndex = MStats_FindOrAddPlayer(playername);
	if not (intIndex) then
		intIndex = MStats_FindOrAddMob(playername);
	end

	return intIndex;
end

function MStats_FindOrAddPlayersAndOrMobs(source, srcFriendly, target, targFriendly)
	local intSource, intTarget, intTotSource, intTotTarget;

	if (source and srcFriendly) then
		intSource = MStats_FindOrAddPlayer(source);
	else
		intSource = nil;
	end

	if (target and targFriendly) then
		intTarget = MStats_FindOrAddPlayer(target);
	else
		intTarget = nil;
	end

	if (intSource and target and (intTarget == nil)) then
		-- Damage given to non tabled entity

		if (objMStats.intFightNamed == MSTATS_NAMED_NOT) then
			-- Name fight using mob
			objMStats.intFightNamed = MSTATS_NAMED_TRASH;
			objMStats.strFightName = target;
		elseif (objMStats.intFightNamed == MSTATS_NAMED_TRASH) then
			-- Check if this mob is a boss
			if (MStats_IsBoss(target) == true) then
				objMStats.intFightNamed = MSTATS_NAMED_BOSS;
				objMStats.strFightName = "보스: " .. target;
			end
		end

		-- If track mobs is on, we need to add this entity to the stats table
		if (objMStats.intTrackMobs == 1) then
			intTarget = MStats_FindOrAddMob(target);
		end
	elseif (intTarget and source and (intSource == nil)) then
		-- Damage taken from non tabled entity

		if (objMStats.intFightNamed == MSTATS_NAMED_NOT) then
			-- Name fight using mob
			objMStats.intFightNamed = MSTATS_NAMED_TRASH;
			objMStats.strFightName = source;
		elseif (objMStats.intFightNamed == MSTATS_NAMED_TRASH) then
			-- Check if this mob is a boss
			if (MStats_IsBoss(source) == true) then
				objMStats.intFightNamed = MSTATS_NAMED_BOSS;
				objMStats.strFightName = "보스: " .. source;
			end
		end

		-- If track mobs is on, we need to add this entity to the stats table
		if (objMStats.intTrackMobs == 1) then
			intSource = MStats_FindOrAddMob(source);
		end
	end

	if (intSource) then
		intTotSource = MStats_FindOrAddTotPlayer(source);
	else
		intTotSource = nil;
	end

	if (intTarget) then
		intTotTarget = MStats_FindOrAddTotPlayer(target);
	else
		intTotTarget = nil;
	end

	return intSource, intTarget, intTotSource, intTotTarget;
end

function MStats_MakePlayerObj(name, class)
	return { name = name, unitid = MStats_GetPlayerUnitId(name), class = class, dmgdone = 0, dmgtaken = 0, dmgabsorbed = 0, dmgtoself = 0, physicaldone = 0, physicaltaken = 0, physicalabsorbed = 0; firedone = 0, firetaken = 0, fireabsorbed = 0, frostdone = 0, frosttaken = 0, frostabsorbed = 0, naturedone = 0, naturetaken = 0, natureabsorbed = 0, shadowdone = 0, shadowtaken = 0, shadowabsorbed = 0, arcanedone = 0, arcanetaken = 0, arcaneabsorbed = 0, holydone = 0, holytaken = 0, holyabsorbed = 0, noncrits = 0, crits = 0, hits = 0, misses = 0, vshits = 0, vsmisses = 0, healingdone = 0, overhealingdone = 0, healingtaken = 0, deaths = 0, decurses = 0, highesthit = 0, highestheal = 0};
end

function MStats_FindPlayerObj(name)
	local k, v;

	for k,v in ipairs(objMStatsStatistics) do
		if objMStatsStatistics[k].name == name then
			return k;
		end
	end
end

function MStats_FindTotPlayerObj(name)
	local k, v;

	for k,v in ipairs(objMStatsTotalStatistics) do
		if objMStatsTotalStatistics[k].name == name then
			return k;
		end
	end
end

function MStats_InRaid(name)
	local i;

	if (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers(), 1 do
			local strThisName, _, _, _, strThisClass, _, _, _, _ = GetRaidRosterInfo(i);
			-- 클래스
			--strThisClass = classList[strThisClass]

			if (strThisName == name) then
				return strThisClass, i;
			end
		end

		for i = 1, GetNumRaidMembers(), 1 do
			if (UnitExists("raidpet" .. i)) then
				local strThisName = UnitName("raidpet" .. i);

				if (strThisName == name) then
					return "Pet", i;
				end
			end
		end

		return nil, nil;
	end

	-- If we aren't in a raid, then we should check to see if this is us or our pet
	if (name == UnitName("player")) then
		return UnitClass("player"), 1;
	elseif (name == UnitName("pet")) then
		return "Pet", 0;
	end

	-- We aren't in a raid and this isn't us, so we need to check if we are in a party
	if (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers(), 1 do
			if (UnitName("party" .. i) == name) then
				return UnitClass("party" .. i), i;
			end
		end

		for i = 1, GetNumPartyMembers(), 1 do
			if (UnitExists("partypet" .. i)) then
				local strThisName = UnitName("partypet" .. i);

				if (strThisName == name) then
					return "Pet", i;
				end
			end
		end

		return nil, nil;
	end

	return nil, nil;
end

function MStats_GetPlayerUnitId(name)
	local i;

	if (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers(), 1 do
			if (GetRaidRosterInfo(i) == name) then
				return "raid" .. i;
			end
		end

		for i = 1, GetNumRaidMembers(), 1 do
			if (UnitExists("raidpet" .. i)) then
				if (UnitName("raidpet" .. i) == name) then
					return "raidpet" .. i;
				end
			end
		end

		return nil;
	end

	-- If we aren't in a raid, then we should check to see if this is us or our pet
	if (name == UnitName("player")) then
		return "player";
	elseif (name == UnitName("pet")) then
		return "pet";
	end

	-- We aren't in a raid and this isn't us, so we need to check if we are in a party
	if (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers(), 1 do
			if (UnitName("party" .. i) == name) then
				return "party" .. i;
			end
		end

		for i = 1, GetNumPartyMembers(), 1 do
			if (UnitExists("partypet" .. i)) then
				if (UnitName("partypet" .. i) == name) then
					return "pet" .. i;
				end
			end
		end

		return nil;
	end

	return nil;
end

function MStats_GetRaidName(intPosition)
	if (GetNumRaidMembers() > 0) then
		return UnitName("raid" .. intPosition);
	elseif (GetNumPartyMembers() > 0) then
		if (intPosition == 0) then
			return UnitName("player");
		else
			return UnitName("party" .. intPosition);
		end
	else
		if (intPosition == 0) then
			return UnitName("player");
		end
	end
end

function MStats_GetRaidClass(intPosition)
	if (GetNumRaidMembers() > 0) then
		return UnitClass("raid" .. intPosition);
	elseif (GetNumPartyMembers() > 0) then
		if (intPosition == 0) then
			return UnitName("player");
		else
			return UnitClass("party" .. intPosition);
		end
	else
		if (intPosition == 0) then
			return UnitClass("player");
		end
	end
end

function MStats_AddFightToHistory()
	local intTopIndex, intScrollFightHistory, intEndFightOffset;

	if (objMStats.intFightEndTime == 0) then
		return;
	end

	intEndFightOffset = #objMStatsStatHistory - 10;
	if (intEndFightOffset < 0) then
		intEndFightOffset = 0;
	end
	if (intCurrentFightOffset == intEndFightOffset) then
		intScrollFightHistory = 1;
	else
		intScrollFightHistory = 0;
	end

	intTopIndex = #objMStatsStatHistory + 1;
	if (objMStats.strFightName == "") then
		objMStats.strFightName = "Unknown";
	end
	table.insert(objMStatsStatHistory, { intStartTime = objMStats.intFightStartTime, intEndTime = objMStats.intFightEndTime, intDuration = (objMStats.intFightLocalEndTime - objMStats.intFightLocalStartTime), strFightName = objMStats.strFightName, objMStatsStatistics = objMStatsStatistics});

	MStats_WipeCurrentFight();

	if (intScrollFightHistory == 1) then
		intCurrentFightOffset = #objMStatsStatHistory - 10;
		if (intCurrentFightOffset < 0) then
			intCurrentFightOffset = 0;
		end
	end

	MStats_UpdateTmpFightTotalObject();
end

function MStats_WipeCurrentFight()
	objMStatsStatistics = {};
	objMStats.intFightNamed = MSTATS_NAMED_NOT;
	objMStats.strFightName = "";
	objMStats.intFightStartTime = 0;
	objMStats.intFightEndTime = 0;
end

function MStats_AddFightTotalToHistory(subtotal)
	local intTopIndex, intScrollFightHistory, intEndFightOffset;
	local objTmpStats, intTmpFightStartTime, intTmpFightEndTime, intTmpFightDuration;
	local intCurrentIndex;

	if (#objMStatsStatHistory < 1) then
		return;
	end

	intEndFightOffset = #objMStatsStatHistory - 10;
	if (intEndFightOffset < 0) then
		intEndFightOffset = 0;
	end
	if (intCurrentFightOffset == intEndFightOffset) then
		intScrollFightHistory = 1;
	else
		intScrollFightHistory = 0;
	end

	if (intCurrentFight == 0) then
		intTopIndex = #objMStatsStatHistory;
	else
		intTopIndex = intCurrentFight;
	end

	objTmpStats, intTmpFightStartTime, intTmpFightEndTime, intTmpFightDuration = MStats_MakeFightTotalObject(intTopIndex, subtotal);

	if (#objTmpStats > 0) then
		if (subtotal == 1) then
			table.insert(objMStatsStatHistory, intTopIndex + 1, { intStartTime = intTmpFightStartTime, intEndTime = intTmpFightEndTime, intDuration = intTmpFightDuration, strFightName = "SUBTOTAL", objMStatsStatistics = objTmpStats } );
		else
			table.insert(objMStatsStatHistory, intTopIndex + 1, { intStartTime = intTmpFightStartTime, intEndTime = intTmpFightEndTime, intDuration = intTmpFightDuration, strFightName = "TOTAL", objMStatsStatistics = objTmpStats } );
		end
	else
		message("총합이 없습니다!");
	end

	if (intScrollFightHistory == 1) then
		intCurrentFightOffset = #objMStatsStatHistory - 10;
		if (intCurrentFightOffset < 0) then
			intCurrentFightOffset = 0;
		end
	end
end

function MStats_UpdateTmpFightTotalObject()
	if (#objMStatsStatHistory < 1) then
		objMStatsTotalStatistics = {};
		objMStatsTotalStatistics.intDuration = 0;
	else
		local intTempDuration;
		objMStatsTotalStatistics,_,_,intTempDuration = MStats_MakeFightTotalObject(#objMStatsStatHistory, 0);
		objMStatsTotalStatistics.intDuration = intTempDuration;
	end
end

function MStats_MakeFightTotalObject(intTopIndex, subtotal)
	local objTmpStats, i, intTmpFightStartTime, intTmpFightEndTime, intTempDuration, k, v, l, w;

	objTmpStats = {};
	intTmpFightEndTime = objMStatsStatHistory[intTopIndex].intEndTime;
	intTempDuration = 0;

	for i = intTopIndex, 1, -1 do
		if (objMStatsStatHistory[i].strFightName) then
			if (string.sub(objMStatsStatHistory[i].strFightName, 1, 5) == "TOTAL") then
				break;
			end

			if (subtotal == 1) then
				if (string.sub(objMStatsStatHistory[i].strFightName, 1, 8) == "SUBTOTAL") then
					break;
				end
			end
		end

		if not (objMStatsStatHistory[i].strFightName == "SUBTOTAL") then
			intTmpFightStartTime = objMStatsStatHistory[i].intStartTime;
			intTempDuration = intTempDuration + objMStatsStatHistory[i].intDuration;

			for k,v in ipairs(objMStatsStatHistory[i].objMStatsStatistics) do
				local intIndex, index2, value2;

				intIndex = nil;
				for l,w in ipairs(objTmpStats) do
					if objTmpStats[l].name == objMStatsStatHistory[i].objMStatsStatistics[k].name then
						intIndex = l;
						break;
					end
				end

				if (intIndex == nil) then
					intIndex = #objTmpStats + 1;
					table.insert(objTmpStats, MStats_MakePlayerObj(objMStatsStatHistory[i].objMStatsStatistics[k].name, objMStatsStatHistory[i].objMStatsStatistics[k].class));
				end

				objTmpStats[intIndex].dmgdone = objTmpStats[intIndex].dmgdone + objMStatsStatHistory[i].objMStatsStatistics[k].dmgdone;
				objTmpStats[intIndex].dmgtaken = objTmpStats[intIndex].dmgtaken + objMStatsStatHistory[i].objMStatsStatistics[k].dmgtaken;
				objTmpStats[intIndex].dmgabsorbed = objTmpStats[intIndex].dmgabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].dmgabsorbed;
				objTmpStats[intIndex].dmgtoself = objTmpStats[intIndex].dmgtoself + objMStatsStatHistory[i].objMStatsStatistics[k].dmgtoself;

				objTmpStats[intIndex].physicaldone = objTmpStats[intIndex].physicaldone + objMStatsStatHistory[i].objMStatsStatistics[k].physicaldone;
				objTmpStats[intIndex].physicaltaken = objTmpStats[intIndex].physicaltaken + objMStatsStatHistory[i].objMStatsStatistics[k].physicaltaken;
				objTmpStats[intIndex].physicalabsorbed = objTmpStats[intIndex].physicalabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].physicalabsorbed;

				objTmpStats[intIndex].firedone = objTmpStats[intIndex].firedone + objMStatsStatHistory[i].objMStatsStatistics[k].firedone;
				objTmpStats[intIndex].firetaken = objTmpStats[intIndex].firetaken + objMStatsStatHistory[i].objMStatsStatistics[k].firetaken;
				objTmpStats[intIndex].fireabsorbed = objTmpStats[intIndex].fireabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].fireabsorbed;

				objTmpStats[intIndex].frostdone = objTmpStats[intIndex].frostdone + objMStatsStatHistory[i].objMStatsStatistics[k].frostdone;
				objTmpStats[intIndex].frosttaken = objTmpStats[intIndex].frosttaken + objMStatsStatHistory[i].objMStatsStatistics[k].frosttaken;
				objTmpStats[intIndex].frostabsorbed = objTmpStats[intIndex].frostabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].frostabsorbed;

				objTmpStats[intIndex].naturedone = objTmpStats[intIndex].naturedone + objMStatsStatHistory[i].objMStatsStatistics[k].naturedone;
				objTmpStats[intIndex].naturetaken = objTmpStats[intIndex].naturetaken + objMStatsStatHistory[i].objMStatsStatistics[k].naturetaken;
				objTmpStats[intIndex].natureabsorbed = objTmpStats[intIndex].natureabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].natureabsorbed;

				objTmpStats[intIndex].shadowdone = objTmpStats[intIndex].shadowdone + objMStatsStatHistory[i].objMStatsStatistics[k].shadowdone;
				objTmpStats[intIndex].shadowtaken = objTmpStats[intIndex].shadowtaken + objMStatsStatHistory[i].objMStatsStatistics[k].shadowtaken;
				objTmpStats[intIndex].shadowabsorbed = objTmpStats[intIndex].shadowabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].shadowabsorbed;

				objTmpStats[intIndex].arcanedone = objTmpStats[intIndex].arcanedone + objMStatsStatHistory[i].objMStatsStatistics[k].arcanedone;
				objTmpStats[intIndex].arcanetaken = objTmpStats[intIndex].arcanetaken + objMStatsStatHistory[i].objMStatsStatistics[k].arcanetaken;
				objTmpStats[intIndex].arcaneabsorbed = objTmpStats[intIndex].arcaneabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].arcaneabsorbed;

				objTmpStats[intIndex].holydone = objTmpStats[intIndex].holydone + objMStatsStatHistory[i].objMStatsStatistics[k].holydone;
				objTmpStats[intIndex].holytaken = objTmpStats[intIndex].holytaken + objMStatsStatHistory[i].objMStatsStatistics[k].holytaken;
				objTmpStats[intIndex].holyabsorbed = objTmpStats[intIndex].holyabsorbed + objMStatsStatHistory[i].objMStatsStatistics[k].holyabsorbed;

				objTmpStats[intIndex].noncrits = objTmpStats[intIndex].noncrits + objMStatsStatHistory[i].objMStatsStatistics[k].noncrits;
				objTmpStats[intIndex].crits = objTmpStats[intIndex].crits + objMStatsStatHistory[i].objMStatsStatistics[k].crits;
				objTmpStats[intIndex].hits = objTmpStats[intIndex].hits + objMStatsStatHistory[i].objMStatsStatistics[k].hits;
				objTmpStats[intIndex].misses = objTmpStats[intIndex].misses + objMStatsStatHistory[i].objMStatsStatistics[k].misses;
				objTmpStats[intIndex].vshits = objTmpStats[intIndex].vshits + objMStatsStatHistory[i].objMStatsStatistics[k].vshits;
				objTmpStats[intIndex].vsmisses = objTmpStats[intIndex].vsmisses + objMStatsStatHistory[i].objMStatsStatistics[k].vsmisses;

				objTmpStats[intIndex].healingdone = objTmpStats[intIndex].healingdone + objMStatsStatHistory[i].objMStatsStatistics[k].healingdone;
				objTmpStats[intIndex].overhealingdone = objTmpStats[intIndex].overhealingdone + objMStatsStatHistory[i].objMStatsStatistics[k].overhealingdone;
				objTmpStats[intIndex].healingtaken = objTmpStats[intIndex].healingtaken + objMStatsStatHistory[i].objMStatsStatistics[k].healingtaken;
				objTmpStats[intIndex].deaths = objTmpStats[intIndex].deaths + objMStatsStatHistory[i].objMStatsStatistics[k].deaths;
				objTmpStats[intIndex].decurses = objTmpStats[intIndex].decurses + objMStatsStatHistory[i].objMStatsStatistics[k].decurses;

				objTmpStats[intIndex].highesthit = math.max(objTmpStats[intIndex].highesthit, objMStatsStatHistory[i].objMStatsStatistics[k].highesthit);
				objTmpStats[intIndex].highestheal = math.max(objTmpStats[intIndex].highestheal, objMStatsStatHistory[i].objMStatsStatistics[k].highestheal);
			end
		end
	end

	return objTmpStats, intTmpFightStartTime, intTmpFightEndTime, intTempDuration;
end

--[[-------------------------------------------
******************* FILTERING *****************
---------------------------------------------]]

function MStats_ProcessSyncEvents(combatstring)
	local starttime,name,intPlayer;

	if (strfind(combatstring, MSTATS_CL_SYNCDATA_TYPEA) ~= nil) then
		local dmgdone,dmgtaken,dmgabsorbed,dmgtoself,noncrits,crits,hits,misses,vshits;
		local vsmisses,healingdone,overhealingdone,healingtaken,deaths,decurses,highesthit,highestheal;

		-- Just ignore sync data if we are tracking pets separately
		if (objMStats.intTrackPets == 1) then
			return;
		end

		_,_,starttime,name,dmgdone,dmgtaken,dmgabsorbed,dmgtoself,noncrits,crits,hits,misses,vshits,vsmisses,healingdone,overhealingdone,healingtaken,deaths,decurses,highesthit,highestheal = string.find(combatstring, MSTATS_CL_SYNCDATA_TYPEA .. "(%d+)||(.+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)");

		if (objMStats.intTrackMobs == 1) then
			intPlayer = MStats_FindOrAddPlayerOrMob(name);
		else
			intPlayer = MStats_FindOrAddPlayer(name);
		end

		if (intPlayer) then
			if (tonumber(starttime) == objMStats.intFightStartTime) then
				objMStatsStatistics[intPlayer].dmgdone = math.max(objMStatsStatistics[intPlayer].dmgdone, dmgdone);
				objMStatsStatistics[intPlayer].dmgtaken = math.max(objMStatsStatistics[intPlayer].dmgtaken, dmgtaken);
				objMStatsStatistics[intPlayer].dmgabsorbed = math.max(objMStatsStatistics[intPlayer].dmgabsorbed, dmgabsorbed);
				objMStatsStatistics[intPlayer].dmgtoself = math.max(objMStatsStatistics[intPlayer].dmgtoself, dmgtoself);
				objMStatsStatistics[intPlayer].noncrits = math.max(objMStatsStatistics[intPlayer].noncrits, noncrits);
				objMStatsStatistics[intPlayer].crits = math.max(objMStatsStatistics[intPlayer].crits, crits);
				objMStatsStatistics[intPlayer].hits = math.max(objMStatsStatistics[intPlayer].hits, hits);
				objMStatsStatistics[intPlayer].misses = math.max(objMStatsStatistics[intPlayer].misses, misses);
				objMStatsStatistics[intPlayer].vshits = math.max(objMStatsStatistics[intPlayer].vshits, vshits);
				objMStatsStatistics[intPlayer].vsmisses = math.max(objMStatsStatistics[intPlayer].vsmisses, vsmisses);
				objMStatsStatistics[intPlayer].healingdone = math.max(objMStatsStatistics[intPlayer].healingdone, healingdone);
				objMStatsStatistics[intPlayer].overhealingdone = math.max(objMStatsStatistics[intPlayer].overhealingdone, overhealingdone);
				objMStatsStatistics[intPlayer].healingtaken = math.max(objMStatsStatistics[intPlayer].healingtaken, healingtaken);
				objMStatsStatistics[intPlayer].deaths = math.max(objMStatsStatistics[intPlayer].deaths, deaths);
				objMStatsStatistics[intPlayer].decurses = math.max(objMStatsStatistics[intPlayer].decurses, decurses);
				objMStatsStatistics[intPlayer].highesthit = math.max(objMStatsStatistics[intPlayer].highesthit, highesthit);
				objMStatsStatistics[intPlayer].highestheal = math.max(objMStatsStatistics[intPlayer].highestheal, highestheal);
			end
		end

	elseif (strfind(combatstring, MSTATS_CL_SYNCDATA_TYPEB) ~= nil) then
		local firedone,firetaken,fireabsorbed,frostdone,frosttaken,frostabsorbed,naturedone;
		local naturetaken,natureabsorbed,shadowdone,shadowtaken,shadowabsorbed,arcanedone;
		local arcanetaken,arcaneabsorbed,holydone,holytaken,holyabsorbed;

		-- Just ignore sync data if we are tracking pets separately
		if (objMStats.intTrackPets == 1) then
			return;
		end

		_,_,starttime,name,firedone,firetaken,fireabsorbed,frostdone,frosttaken,frostabsorbed,naturedone,naturetaken,natureabsorbed,shadowdone,shadowtaken,shadowabsorbed,arcanedone,arcanetaken,arcaneabsorbed,holydone,holytaken,holyabsorbed = string.find(combatstring, MSTATS_CL_SYNCDATA_TYPEB .. "(%d+)||(.+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)||(%d+)");

		if (objMStats.intTrackMobs == 1) then
			intPlayer = MStats_FindOrAddPlayerOrMob(name);
		else
			intPlayer = MStats_FindOrAddPlayer(name);
		end

		if (intPlayer) then
			if (tonumber(starttime) == objMStats.intFightStartTime) then
				objMStatsStatistics[intPlayer].firedone = math.max(objMStatsStatistics[intPlayer].firedone, firedone);
				objMStatsStatistics[intPlayer].firetaken = math.max(objMStatsStatistics[intPlayer].firetaken, firetaken);
				objMStatsStatistics[intPlayer].fireabsorbed = math.max(objMStatsStatistics[intPlayer].fireabsorbed, fireabsorbed);
				objMStatsStatistics[intPlayer].frostdone = math.max(objMStatsStatistics[intPlayer].frostdone, frostdone);
				objMStatsStatistics[intPlayer].frosttaken = math.max(objMStatsStatistics[intPlayer].frosttaken, frosttaken);
				objMStatsStatistics[intPlayer].frostabsorbed = math.max(objMStatsStatistics[intPlayer].frostabsorbed, frostabsorbed);
				objMStatsStatistics[intPlayer].naturedone = math.max(objMStatsStatistics[intPlayer].naturedone, naturedone);
				objMStatsStatistics[intPlayer].naturetaken = math.max(objMStatsStatistics[intPlayer].naturetaken, naturetaken);
				objMStatsStatistics[intPlayer].natureabsorbed = math.max(objMStatsStatistics[intPlayer].natureabsorbed, natureabsorbed);
				objMStatsStatistics[intPlayer].shadowdone = math.max(objMStatsStatistics[intPlayer].shadowdone, shadowdone);
				objMStatsStatistics[intPlayer].shadowtaken = math.max(objMStatsStatistics[intPlayer].shadowtaken, shadowtaken);
				objMStatsStatistics[intPlayer].shadowabsorbed = math.max(objMStatsStatistics[intPlayer].shadowabsorbed, shadowabsorbed);
				objMStatsStatistics[intPlayer].arcanedone = math.max(objMStatsStatistics[intPlayer].arcanedone, arcanedone);
				objMStatsStatistics[intPlayer].arcanetaken = math.max(objMStatsStatistics[intPlayer].arcanetaken, arcanetaken);
				objMStatsStatistics[intPlayer].arcaneabsorbed = math.max(objMStatsStatistics[intPlayer].arcaneabsorbed, arcaneabsorbed);
				objMStatsStatistics[intPlayer].holydone = math.max(objMStatsStatistics[intPlayer].holydone, holydone);
				objMStatsStatistics[intPlayer].holytaken = math.max(objMStatsStatistics[intPlayer].holytaken, holytaken);
				objMStatsStatistics[intPlayer].holyabsorbed = math.max(objMStatsStatistics[intPlayer].holyabsorbed, holyabsorbed);

				objMStatsStatistics[intPlayer].physicaldone = objMStatsStatistics[intPlayer].dmgdone - objMStatsStatistics[intPlayer].firedone - objMStatsStatistics[intPlayer].frostdone - objMStatsStatistics[intPlayer].naturedone - objMStatsStatistics[intPlayer].shadowdone - objMStatsStatistics[intPlayer].arcanedone - objMStatsStatistics[intPlayer].holydone;
				objMStatsStatistics[intPlayer].physicaltaken = objMStatsStatistics[intPlayer].dmgtaken - objMStatsStatistics[intPlayer].firetaken - objMStatsStatistics[intPlayer].frosttaken - objMStatsStatistics[intPlayer].naturetaken - objMStatsStatistics[intPlayer].shadowtaken - objMStatsStatistics[intPlayer].arcanetaken - objMStatsStatistics[intPlayer].holytaken;
				objMStatsStatistics[intPlayer].physicalabsorbed = objMStatsStatistics[intPlayer].dmgabsorbed - objMStatsStatistics[intPlayer].fireabsorbed - objMStatsStatistics[intPlayer].frostabsorbed - objMStatsStatistics[intPlayer].natureabsorbed - objMStatsStatistics[intPlayer].shadowabsorbed - objMStatsStatistics[intPlayer].arcaneabsorbed - objMStatsStatistics[intPlayer].holyabsorbed;
			end
		end
	elseif (strfind(combatstring, MSTATS_CL_TIMETICKER) ~= nil) then
		_,_,intLogLastHour,intLogLastMinute = string.find(combatstring, MSTATS_CL_TIMETICKER .. "(%d+):(%d+)");
	elseif (strfind(combatstring, MSTATS_CL_LOCALTIMETICKER) ~= nil) then
		_,_,intLogLastSeconds = string.find(combatstring, MSTATS_CL_LOCALTIMETICKER .. "(%d+)");
	elseif (strfind(combatstring, MSTATS_CL_INCOMBAT) ~= nil) then
		_,_,intLogLastHour,intLogLastMinute = string.find(combatstring, MSTATS_CL_INCOMBAT .. "(%d+):(%d+)");

		if (intLogInCombat == 0) then
			MStats_AddFightToHistory();

			intLogInCombat = 1;
			intTimeOutOfCombat = 0;
			objMStats.intFightStartTime = (intLogLastHour * 60) + intLogLastMinute;
		else
			-- Check incombat message is 1 minute before our current incombat, if so we should make sure ours is the same to avoid sync rejects due to lag
			if (((intLogLastHour * 60) + intLogLastMinute) == (objMStats.intFightStartTime - 1)) then
				objMStats.intFightStartTime = (intLogLastHour * 60) + intLogLastMinute;
			end
		end
	elseif (strfind(combatstring, MSTATS_CL_STILLINCOMBAT) ~= nil) then
		local strSender;

		_,_,strSender = strfind(combatstring, MSTATS_CL_STILLINCOMBAT .. "(%a+)");
		intTimeOutOfCombat = 0;
		objMStats.intFightEndTime = (intLogLastHour * 60) + intLogLastMinute;
	elseif (strfind(combatstring, MSTATS_CL_SYNCCHECK) ~= nil) then
		local strSender;

		_,_,strSender = strfind(combatstring, MSTATS_CL_SYNCCHECK .. "(%a+)");
		SendAddonMessage(MSTATS_CL_PREFIX, MSTATS_CL_SYNCREPLY .. UnitName("player") .. " " .. strSender .. " " .. MSTATS_VERSION, "RAID");
		MStats_Print(strSender .. "에 의한 동조 체크 방송");
	elseif (strfind(combatstring, MSTATS_CL_SYNCREPLY) ~= nil) then
		local strSender, strOriginator, strVersion;

		first,_,strSender,strOriginator,strVersion = strfind(combatstring, MSTATS_CL_SYNCREPLY .. "(%a+) (%a+) (%d+%.%d+)");
		if (first ~= nil) then
			-- Version 1.40 or later, so show reply and version if we sent the original request
			if (strOriginator == UnitName("player")) then
				MStats_Print(strSender .. " (v" .. strVersion .. ")" .. "에 의해 동조 체크가 응답되었습니다.");

				if (tonumber(strVersion) < tonumber(MSTATS_VERSION)) then
					SendChatMessage("MinnaStats v" .. MSTATS_VERSION .. "의 이용이 불가능합니다. 업데이트 해주세요!", "whisper", nil, strSender);
				end
			end
		else
			-- Pre 1.40 version, so just show
			_,_,strSender = strfind(combatstring, MSTATS_CL_SYNCREPLY .. "(%a+)");
			MStats_Print(strSender .. " (pre v1.40)" .. "에 의해 동조 체크가 응답되었습니다.");
		end
	end
end

function MStats_ProcessUnprocessedEvents()
	local strString;

	strString = "";
	if (arg1) then
		strString = strString .. arg1 .. ",";
	end
	if (arg2) then
		strString = strString .. arg2 .. ",";
	end
	if (arg3) then
		strString = strString .. arg3 .. ",";
	end
	if (arg4) then
		strString = strString .. arg4 .. ",";
	end
	if (arg5) then
		strString = strString .. arg5 .. ",";
	end
	if (arg6) then
		strString = strString .. arg6 .. ",";
	end
	if (arg7) then
		strString = strString .. arg7 .. ",";
	end
	if (arg8) then
		strString = strString .. arg8 .. ",";
	end
	if (arg9) then
		strString = strString .. arg9 .. ",";
	end
	if (arg10) then
		strString = strString .. arg10 .. ",";
	end
	if (arg11) then
		strString = strString .. arg11 .. ",";
	end
	if (arg12) then
		strString = strString .. arg12 .. ",";
	end
	if (arg13) then
		strString = strString .. arg13 .. ",";
	end
	if (arg14) then
		strString = strString .. arg14 .. ",";
	end
	if (arg15) then
		strString = strString .. arg15 .. ",";
	end
	if (arg16) then
		strString = strString .. arg16 .. ",";
	end
	if (arg17) then
		strString = strString .. arg17 .. ",";
	end
	if (arg18) then
		strString = strString .. arg18 .. ",";
	end
	if (arg19) then
		strString = strString .. arg19 .. ",";
	end

	DEFAULT_CHAT_FRAME:AddMessage("|c00FF0000MSTATS: |c0000FFFF" .. strString, 1,1,1);
end

--[[-------------------------------------------
******************** MENUS ********************
---------------------------------------------]]

function MStats_ButtonAnnounceMenu()
	ToggleDropDownMenu(1, nil, MStatsFrameAnnounceMenu);
end

function MStats_InitialiseAnnounceMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildAnnounceMenu, "MENU");
end

function MStats_BuildAnnounceMenu()
	local info = {};

	info.text = "채널 설정";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};

	info.func = MStats_AnnounceMenuOption;

	if (objMStats.intAnnounceType == MSTATS_AN_PARTY) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "파티";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intAnnounceType == MSTATS_AN_RAID) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "공격대";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intAnnounceType == MSTATS_AN_GUILD) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "길드";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intAnnounceType == MSTATS_AN_OFFICER) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "길드 관리자";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intAnnounceType == MSTATS_AN_CHANNEL) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "채널";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intAnnounceType == MSTATS_AN_WHISPER) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "귓속말";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "알림";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};

	info.func = MStats_AnnounceMenuOption;

	if (strStatFighter == nil) then
		info.text = "톱 5";
		UIDropDownMenu_AddButton(info);

		info.text = "톱 10";
		UIDropDownMenu_AddButton(info);

		info.text = "톱 15";
		UIDropDownMenu_AddButton(info);

		info.text = "톱 20";
		UIDropDownMenu_AddButton(info);

		info.text = "톱 30";
		UIDropDownMenu_AddButton(info);

		info.text = "전체";
		UIDropDownMenu_AddButton(info);
	else
		info.text = "일반";
		UIDropDownMenu_AddButton(info);

		info.text = "준 피해량";
		UIDropDownMenu_AddButton(info);

		info.text = "받은 피해량";
		UIDropDownMenu_AddButton(info);

		info.text = "기타사항";
		UIDropDownMenu_AddButton(info);
	end
end

function MStats_AnnounceMenuOption()
	local idSelected = this:GetID();

	if ( idSelected == 2 ) then
		objMStats.intAnnounceType = MSTATS_AN_PARTY;
	elseif ( idSelected == 3 ) then
		objMStats.intAnnounceType = MSTATS_AN_RAID;
	elseif ( idSelected == 4 ) then
		objMStats.intAnnounceType = MSTATS_AN_GUILD;
	elseif ( idSelected == 5 ) then
		objMStats.intAnnounceType = MSTATS_AN_OFFICER;
	elseif ( idSelected == 6 ) then
		objMStats.intAnnounceType = MSTATS_AN_CHANNEL;
		intEditWindow = MSTATS_EW_CHANNELNAME;
		MStatsEditFrame:Show();
	elseif ( idSelected == 7 ) then
		objMStats.intAnnounceType = MSTATS_AN_WHISPER;
		intEditWindow = MSTATS_EW_WHISPERNAME;
		MStatsEditFrame:Show();
	elseif ( idSelected == 9 ) then
		if (strStatFighter == nil) then
			MStats_AnnounceStats(5);
		else
			MStats_AnnounceIndividualStats(1);
		end
	elseif ( idSelected == 10 ) then
		if (strStatFighter == nil) then
			MStats_AnnounceStats(10);
		else
			MStats_AnnounceIndividualStats(2);
		end
	elseif ( idSelected == 11 ) then
		if (strStatFighter == nil) then
			MStats_AnnounceStats(15);
		else
			MStats_AnnounceIndividualStats(3);
		end
	elseif ( idSelected == 12 ) then
		if (strStatFighter == nil) then
			MStats_AnnounceStats(20);
		else
			MStats_AnnounceIndividualStats(4);
		end
	elseif ( idSelected == 13 ) then
		MStats_AnnounceStats(30);
	elseif ( idSelected == 14 ) then
		MStats_AnnounceStats(100);
	end
end

function MStats_ButtonDisplayMenu()
	ToggleDropDownMenu(1, nil, MStatsFrameDisplayMenu);
end

function MStats_InitialiseDisplayMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildDisplayMenu, "MENU");
end

function MStats_BuildDisplayMenu(self, level)
	local info;

	level = level or 1;

	if (level == 1) then
		info = {};

		info.text = "보고서";
		info.isTitle = 1;
		info.justifyH = "LEFT";
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};

		info.text = "일반";
		info.hasArrow = true;
		info.notCheckable = 1;
		info.value = "reports_general";
		UIDropDownMenu_AddButton(info);

		info = {};

		info.text = "기타";
		info.hasArrow = true;
		info.justifyH = "LEFT";
		info.notCheckable = 1;
		info.value = "reports_misc";
		UIDropDownMenu_AddButton(info);

		info = {};

		info.text = "변동사항";
		info.hasArrow = true;
		info.justifyH = "LEFT";
		info.notCheckable = 1;
		info.value = "reports_variances";
		UIDropDownMenu_AddButton(info);

		info = {};

		info.text = "장난!!";
		info.hasArrow = true;
		info.justifyH = "LEFT";
		info.notCheckable = 1;
		info.value = "reports_fun";
		UIDropDownMenu_AddButton(info);
	elseif (level == 2) then
		if (UIDROPDOWNMENU_MENU_VALUE == "reports_general") then
			info = {};
			info.func = MStats_DisplayMenuOptionGeneral;

			if (objMStats.intDisplayType == MSTATS_DT_DAMAGEDONE) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "준 피해량";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_DAMAGETAKEN) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "받은 피해량";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_DAMAGEABSORBED) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "흡수된 피해량";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_HEALINGDONE) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "치유량 (총합)";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_HEALINGNET) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "치유량 (최종)";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_OVERHEALING) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "초과 치유량";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_HEALINGTAKEN) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "받은 치유량";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		elseif (UIDROPDOWNMENU_MENU_VALUE == "reports_misc") then
			info = {};
			info.func = MStats_DisplayMenuOptionMisc;

			if (objMStats.intDisplayType == MSTATS_DT_CRITS) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "치명타";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_CRITPERC) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "치명타율 %";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_DEATHS) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "죽음";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_DAMAGETOSELF) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "자신의 피해량";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_MISSES) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "실패";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_MISSPERC) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "실패율 %";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_VSMISSES) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "Vs 실패";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_DECURSES) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "해제";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		elseif (UIDROPDOWNMENU_MENU_VALUE == "reports_variances") then
			info = {};
			info.func = MStats_DisplayMenuOptionVariances;

			if (objMStats.intDisplayType == MSTATS_DT_HIGHESTHIT) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "최대 명중";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_AVERAGEHIT) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "평균 명중";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_HIGHESTHEAL) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "최대 치유";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_AVERAGEHEAL) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "평균 치유";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_DAMAGEPS) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "피해량/초";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_HITSPS) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "명중/초";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_HEALINGPS) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "치유량/초";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		elseif (UIDROPDOWNMENU_MENU_VALUE == "reports_fun") then
			info = {};
			info.func = MStats_DisplayMenuOptionFun;

			if (objMStats.intDisplayType == MSTATS_DT_EFFICIENCY) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "효율";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_LIABILITY) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "부담";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			if (objMStats.intDisplayType == MSTATS_DT_RAIDWORTH) then
				info.checked = 1;
			else
				info.checked = nil;
			end
			info.text = "공대 기여도";
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		end
	end
end

function MStats_DisplayMenuOptionGeneral()
	local idSelected = this:GetID();

	if ( idSelected == 1 ) then
		objMStats.intDisplayType = MSTATS_DT_DAMAGEDONE;
	elseif ( idSelected == 2 ) then
		objMStats.intDisplayType = MSTATS_DT_DAMAGETAKEN;
	elseif ( idSelected == 3 ) then
		objMStats.intDisplayType = MSTATS_DT_DAMAGEABSORBED;
	elseif ( idSelected == 4 ) then
		objMStats.intDisplayType = MSTATS_DT_HEALINGDONE;
	elseif ( idSelected == 5 ) then
		objMStats.intDisplayType = MSTATS_DT_HEALINGNET;
	elseif ( idSelected == 6 ) then
		objMStats.intDisplayType = MSTATS_DT_OVERHEALING;
	elseif ( idSelected == 7 ) then
		objMStats.intDisplayType = MSTATS_DT_HEALINGTAKEN;
	end

	intCurrentStatOffset = 0;
	strStatFighter = nil;
end

function MStats_DisplayMenuOptionMisc()
	local idSelected = this:GetID();

	if ( idSelected == 1 ) then
		objMStats.intDisplayType = MSTATS_DT_CRITS;
	elseif ( idSelected == 2 ) then
		objMStats.intDisplayType = MSTATS_DT_CRITPERC;
	elseif ( idSelected == 3 ) then
		objMStats.intDisplayType = MSTATS_DT_DEATHS;
	elseif ( idSelected == 4 ) then
		objMStats.intDisplayType = MSTATS_DT_DAMAGETOSELF;
	elseif ( idSelected == 5 ) then
		objMStats.intDisplayType = MSTATS_DT_MISSES;
	elseif ( idSelected == 6 ) then
		objMStats.intDisplayType = MSTATS_DT_MISSPERC;
	elseif ( idSelected == 7 ) then
		objMStats.intDisplayType = MSTATS_DT_VSMISSES;
	elseif ( idSelected == 8 ) then
		objMStats.intDisplayType = MSTATS_DT_DECURSES;
	end

	intCurrentStatOffset = 0;
	strStatFighter = nil;
end

function MStats_DisplayMenuOptionVariances()
	local idSelected = this:GetID();

	if ( idSelected == 1 ) then
		objMStats.intDisplayType = MSTATS_DT_HIGHESTHIT;
	elseif ( idSelected == 2 ) then
		objMStats.intDisplayType = MSTATS_DT_AVERAGEHIT;
	elseif ( idSelected == 3 ) then
		objMStats.intDisplayType = MSTATS_DT_HIGHESTHEAL;
	elseif ( idSelected == 4 ) then
		objMStats.intDisplayType = MSTATS_DT_AVERAGEHEAL;
	elseif ( idSelected == 5 ) then
		objMStats.intDisplayType = MSTATS_DT_DAMAGEPS;
	elseif ( idSelected == 6 ) then
		objMStats.intDisplayType = MSTATS_DT_HITSPS;
	elseif ( idSelected == 7 ) then
		objMStats.intDisplayType = MSTATS_DT_HEALINGPS;
	end

	intCurrentStatOffset = 0;
	strStatFighter = nil;
end

function MStats_DisplayMenuOptionFun()
	local idSelected = this:GetID();

	if ( idSelected == 1 ) then
		objMStats.intDisplayType = MSTATS_DT_EFFICIENCY;
	elseif ( idSelected == 2 ) then
		objMStats.intDisplayType = MSTATS_DT_LIABILITY;
	elseif ( idSelected == 3 ) then
		objMStats.intDisplayType = MSTATS_DT_RAIDWORTH;
	end

	intCurrentStatOffset = 0;
	strStatFighter = nil;
end

function MStats_ButtonCombatRecord(arg1)
	if (arg1 == "LeftButton") then
		MStats_ButtonFight();
	elseif (arg1 == "RightButton") then
		MStats_ButtonQuickFightMenu();
	end
end

function MStats_ButtonQuickFightMenu()
	ToggleDropDownMenu(1, nil, MStatsFrameQuickFightMenu);
end

function MStats_InitialiseQuickFightMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildQuickFightMenu, "MENU");
end

function MStats_BuildQuickFightMenu()
	local info = {};

	info.text = "빠른 삭제";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_QuickFightMenuOption;

	info.text = "진행중인 전투";
	UIDropDownMenu_AddButton(info);

	info.text = "지난 전투";
	UIDropDownMenu_AddButton(info);

	info.text = "모든 전투";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "빠른 설정";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_QuickFightMenuOption;

	if ((objMStats.intLiveTotal == 1) and (objMStats.intTrackMobs == 0) and (objMStats.intTrackPets == 0)) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "던전";
	UIDropDownMenu_AddButton(info);

	if ((objMStats.intLiveTotal == 0) and (objMStats.intTrackMobs == 1) and (objMStats.intTrackPets == 1)) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "레벨업";
	UIDropDownMenu_AddButton(info);

	if ((objMStats.intLiveTotal == 0) and (objMStats.intTrackMobs == 1) and (objMStats.intTrackPets == 0)) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "PVP";
	UIDropDownMenu_AddButton(info);
end

function MStats_QuickFightMenuOption()
	local idSelected = this:GetID();

	if ( idSelected == 2 ) then
		--Wipe current fight
		MStats_WipeCurrentFight();
		MStats_UpdateTmpFightTotalObject();
	elseif ( idSelected == 3 ) then
		--Wipe fight history
		if (#objMStatsStatHistory > 0) then
			MStats_WipeFightsDown(1);
		end
	elseif ( idSelected == 4 ) then
		--Wipe all fights
		MStats_WipeCurrentFight();
		if (#objMStatsStatHistory > 0) then
			MStats_WipeFightsDown(1);
		end
	elseif ( idSelected == 6 ) then
		--Quick settings for Instancing
		objMStats.intLiveTotal = 1;
		objMStats.intTrackMobs = 0;
		objMStats.intTrackPets = 0;
		objMStats.intFilterType = MSTATS_FT_EXCLUDEMOBS;
		MStats_Print("다음과 같이 동작합니다. : 총합 ON, 몬스터 기록 OFF, 펫 개별 기록 OFF");
	elseif ( idSelected == 7 ) then
		--Quick settings for Leveling
		objMStats.intLiveTotal = 0;
		objMStats.intTrackMobs = 1;
		objMStats.intTrackPets = 1;
		objMStats.intFilterType = MSTATS_FT_ALL;
		MStats_Print("다음과 같이 동작합니다. : 총합 OFF, 몬스터 기록 ON, 펫 개별 기록 ON");
	elseif ( idSelected == 8 ) then
		--Quick settings for PVP
		objMStats.intLiveTotal = 0;
		objMStats.intTrackMobs = 1;
		objMStats.intTrackPets = 0;
		objMStats.intFilterType = MSTATS_FT_ALL;
		MStats_Print("다음과 같이 동작합니다. : 총합 OFF, 몬스터 기록 ON, 펫 개별 기록 OFF");
	end
end

function MStats_ButtonTypeMenu()
	ToggleDropDownMenu(1, nil, MStatsFrameTypeMenu);
end

function MStats_InitialiseTypeMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildTypeMenu, "MENU");
end

function MStats_BuildTypeMenu()
	local info = {};

	info.text = "피해량 종류 변경";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_TypeMenuOption;

	if (objMStats.intDisplayMagicType == MSTATS_MT_ALL) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "전체";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_PHYSICAL) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "물리";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_FIRE) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "화염";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_FROST) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "냉기";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_NATURE) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "자연";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_SHADOW) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "암흑";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_ARCANE) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "비전";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intDisplayMagicType == MSTATS_MT_HOLY) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "신성";
	UIDropDownMenu_AddButton(info);
end

function MStats_TypeMenuOption()
	local idSelected = this:GetID();

	if ( idSelected == 2 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_ALL;
	elseif ( idSelected == 3 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_PHYSICAL;
	elseif ( idSelected == 4 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_FIRE;
	elseif ( idSelected == 5 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_FROST;
	elseif ( idSelected == 6 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_NATURE;
	elseif ( idSelected == 7 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_SHADOW;
	elseif ( idSelected == 8 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_ARCANE;
	elseif ( idSelected == 9 ) then
		objMStats.intDisplayMagicType = MSTATS_MT_HOLY;
	end

	intCurrentStatOffset = 0;
	strStatFighter = nil;
end

function MStats_ButtonFightEditMenu()
	ToggleDropDownMenu(1, nil, MStatsFightsFrameEditMenu);
end

function MStats_InitialiseFightEditMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildFightEditMenu, "MENU");
end

function MStats_BuildFightEditMenu()
	local info = {};

	info.text = "전투편집";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};

	info.func = MStats_FightEditMenuOption;

	info.text = "전투명";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "전투병합";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FightEditMenuOption;

	if (intCurrentFight == 0) then
		info.notClickable = 1;
	end
	info.text = "이전과통합";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "선택삭제";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FightEditMenuOption;

	if (intCurrentFight == 0) then
		info.notClickable = 1;
	end
	info.text = "전투만";
	UIDropDownMenu_AddButton(info);

	info.text = "전투 이상";
	UIDropDownMenu_AddButton(info);

	info.text = "전투 이하";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "다른것삭제";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FightEditMenuOption;

	info.text = "진행중인 전투";
	UIDropDownMenu_AddButton(info);

	info.text = "이전 전투";
	UIDropDownMenu_AddButton(info);

	info.text = "모든 전투";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "설정";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FightEditMenuOption;

	if (objMStats.intDeleteProtectTotals == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "보호 총합";
	UIDropDownMenu_AddButton(info);
end

function MStats_FightEditMenuOption()
	local idSelected = this:GetID();

	if ( idSelected == 2 ) then
		--Edit Fight Name
		intEditWindow = MSTATS_EW_FIGHTNAME;
		MStatsEditFrame:Show();
	elseif ( idSelected == 4 ) then
		--Combine with Previous Fight
		message("죄송합니다! 아직 이행할 수 없습니다!");
	elseif ( idSelected == 6 ) then
		--Delete current selected fight
		table.remove(objMStatsStatHistory, intCurrentFight);
		intCurrentFight = 0;
		MStats_UpdateTmpFightTotalObject();
	elseif ( idSelected == 7 ) then
		--Delete current selected fight upwards
		local i, offset;

		offset = 1;
		for i = 1, intCurrentFight, 1 do
			if (objMStats.intDeleteProtectTotals == 1) then
				if (string.sub(objMStatsStatHistory[offset].strFightName, 1, 5) == "TOTAL") then
					offset = offset + 1;
				elseif (string.sub(objMStatsStatHistory[offset].strFightName, 1, 8) == "SUBTOTAL") then
					offset = offset + 1;
				else
					table.remove(objMStatsStatHistory, offset);
				end
			else
				table.remove(objMStatsStatHistory, offset);
			end
		end
		intCurrentFight = 0;
		MStats_StatsUpButton();
		MStats_FightUpButton();
		MStats_UpdateTmpFightTotalObject();
	elseif ( idSelected == 8 ) then
		--Delete current selected downwards
		MStats_WipeFightsDown(intCurrentFight);
	elseif ( idSelected == 10 ) then
		--Wipe current fight
		MStats_WipeCurrentFight();
		MStats_UpdateTmpFightTotalObject();
	elseif ( idSelected == 11 ) then
		--Wipe fight history
		if (#objMStatsStatHistory > 0) then
			MStats_WipeFightsDown(1);
		end
	elseif ( idSelected == 12 ) then
		--Wipe all fights
		MStats_WipeCurrentFight();
		if (#objMStatsStatHistory > 0) then
			MStats_WipeFightsDown(1);
		end
	elseif ( idSelected == 14 ) then
		if (objMStats.intDeleteProtectTotals == 1) then
			objMStats.intDeleteProtectTotals = 0;
		else
			objMStats.intDeleteProtectTotals = 1;
		end
	end
end

function MStats_WipeFightsDown(intStartingPos)
	repeat
		if (objMStats.intDeleteProtectTotals == 1) then
			if (string.sub(objMStatsStatHistory[intStartingPos].strFightName, 1, 5) == "TOTAL") then
				intStartingPos = intStartingPos + 1;
			elseif (string.sub(objMStatsStatHistory[intStartingPos].strFightName, 1, 8) == "SUBTOTAL") then
				intStartingPos = intStartingPos + 1;
			else
				table.remove(objMStatsStatHistory, intStartingPos);
			end
		else
			table.remove(objMStatsStatHistory);
		end
	until (#objMStatsStatHistory < intStartingPos);
	intCurrentFight = 0;
	MStats_StatsUpButton();
	MStats_FightDownButton();
	MStats_UpdateTmpFightTotalObject();
end

function MStats_ButtonFightAddMenu()
	ToggleDropDownMenu(1, nil, MStatsFightsFrameAddMenu);
end

function MStats_InitialiseFightAddMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildFightAddMenu, "MENU");
end

function MStats_BuildFightAddMenu()
	local info = {};

	info.text = "전투 개체 추가";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};

	info.func = MStats_FightAddMenuOption;

	info.text = "현재 전투";
	UIDropDownMenu_AddButton(info);

	info.text = "소계";
	UIDropDownMenu_AddButton(info);

	info.text = "총계";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "개체 내보내기";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};

	info.func = MStats_FightAddMenuOption;

	if (intCurrentFight == 0) then
		info.notClickable = 1;
	end

	--info.text = "Current Fight";
	--UIDropDownMenu_AddButton(info);

	info.text = "현재 전투 이상";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "누적 개체 내보내기";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};

	info.func = MStats_FightAddMenuOption;

	if (intCurrentFight == 0) then
		info.notClickable = 1;
	end

	info.text = "현재 전투 이상";
	UIDropDownMenu_AddButton(info);

	--info.text = "Current Fight Downwards";
	--UIDropDownMenu_AddButton(info);
end

function MStats_FightAddMenuOption()
	local idSelected = this:GetID();

	if ( idSelected == 2 ) then
		MStats_AddFightToHistory();
		objMStats.intFightStartTime = (intLogLastHour * 60) + intLogLastMinute;
	elseif ( idSelected == 3 ) then
		MStats_AddFightTotalToHistory(1);
	elseif ( idSelected == 4 ) then
		MStats_AddFightTotalToHistory(0);
	elseif ( idSelected == 6 ) then
		--Export all fights
		MStats_ExportCurrentFightUpwards(0);
	elseif ( idSelected == 8 ) then
		--Export all fights
		MStats_ExportCurrentFightUpwards(1);
	end

	MStats_UpdateTmpFightTotalObject();
end

function MStats_ButtonFilterMenu(button)
	if (button == "LeftButton") then
		ToggleDropDownMenu(1, nil, MStatsFrameFilterMenu);
	elseif (button == "RightButton") then
		if (objMStats.intLiveTotal == 1) then
			objMStats.intLiveTotal = 0;
		else
			objMStats.intLiveTotal = 1;
		end
	end
end

function MStats_InitialiseFilterMenu()
	UIDropDownMenu_Initialize(this, MStats_BuildFilterMenu, "MENU");
end

function MStats_BuildFilterMenu()
	local info = {};

	info.text = "직업에 의한 필터링";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FilterMenuOption;

	if (objMStats.intFilterType == MSTATS_FT_ALL) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "없음";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_WARRIOR) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "전사";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_ROGUE) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "도적";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_PALADIN) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "성기사";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_PRIEST) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "사제";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_SHAMAN) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "주술사";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_DRUID) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "드루이드";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_WARLOCK) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "흑마법사";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_MAGE) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "마법사";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_HUNTER) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "사냥꾼";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_PET) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "Pet";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "다른 필터링";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FilterMenuOption;

	if (objMStats.intFilterType == MSTATS_FT_YOURSELF) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "자신만";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_MOBS) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "몬스터";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intFilterType == MSTATS_FT_EXCLUDEMOBS) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "몬스터 제외";
	UIDropDownMenu_AddButton(info);

	info = {};

	info.text = "기록에 의한 필터링";
	info.isTitle = 1;
	info.justifyH = "LEFT";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.func = MStats_FilterMenuOption;

	if (objMStats.intLiveTotal == nil) then
		objMStats.intLiveTotal = 0;
	end

	if (objMStats.intLiveTotal == 0) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "전투에서 벗어난 전투";
	UIDropDownMenu_AddButton(info);

	if (objMStats.intLiveTotal == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "현행 총합";
	UIDropDownMenu_AddButton(info);
end

function MStats_FilterMenuOption()
	local idSelected = this:GetID();

	if ( idSelected == 2 ) then
		objMStats.intFilterType = MSTATS_FT_ALL;
	elseif ( idSelected == 3 ) then
		objMStats.intFilterType = MSTATS_FT_WARRIOR;
	elseif ( idSelected == 4 ) then
		objMStats.intFilterType = MSTATS_FT_ROGUE;
	elseif ( idSelected == 5 ) then
		objMStats.intFilterType = MSTATS_FT_PALADIN;
	elseif ( idSelected == 6 ) then
		objMStats.intFilterType = MSTATS_FT_PRIEST;
	elseif ( idSelected == 7 ) then
		objMStats.intFilterType = MSTATS_FT_SHAMAN;
	elseif ( idSelected == 8 ) then
		objMStats.intFilterType = MSTATS_FT_DRUID;
	elseif ( idSelected == 9 ) then
		objMStats.intFilterType = MSTATS_FT_WARLOCK;
	elseif ( idSelected == 10 ) then
		objMStats.intFilterType = MSTATS_FT_MAGE;
	elseif ( idSelected == 11 ) then
		objMStats.intFilterType = MSTATS_FT_HUNTER;
	elseif ( idSelected == 12 ) then
		objMStats.intFilterType = MSTATS_FT_PET;
	elseif ( idSelected == 14 ) then
		objMStats.intFilterType = MSTATS_FT_YOURSELF;
	elseif ( idSelected == 15 ) then
		objMStats.intFilterType = MSTATS_FT_MOBS;
	elseif ( idSelected == 16 ) then
		objMStats.intFilterType = MSTATS_FT_EXCLUDEMOBS;
	elseif ( idSelected == 18 ) then
		objMStats.intLiveTotal = 0;
	elseif ( idSelected == 19 ) then
		objMStats.intLiveTotal = 1;
	end

	intCurrentStatOffset = 0;
	strStatFighter = nil;
end


--[[-------------------------------------------
****************** MISC FUNCS *****************
---------------------------------------------]]

function MStats_SortData()
	local i, j, intIndex;

	funcSortDataFunc = MStats_GetDataValueFunction();

	if (intCurrentFight > 0) then
		table.sort(objMStatsStatHistory[intCurrentFight].objMStatsStatistics, MStats_SortDataCheck);
	elseif (objMStats.intLiveTotal == 1) then
		table.sort(objMStatsTotalStatistics, MStats_SortDataCheck);
	else
		table.sort(objMStatsStatistics, MStats_SortDataCheck);
	end
end

function MStats_SortDataCheck(a, b)
	if (funcSortDataFunc(a) > funcSortDataFunc(b)) then
		return true;
	elseif (funcSortDataFunc(a) == funcSortDataFunc(b)) then
		if (a.name < b.name) then
			return true;
		end
	end

	return false;
end

function MStats_AnnounceStats(linecount)
	local funcDataFunc, strDataTypeString, strDashString;
	local objTmpStats, strTmpFightName, strTmpFightLen, intLineCounter;
	local chattype, channel, k, v, hours, minutes, strTemp;
	local intPercent = 0;

	if (objMStats.intAnnounceType == MSTATS_AN_PARTY) then
		chattype = "party";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_RAID) then
		chattype = "raid";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_GUILD) then
		chattype = "guild";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_OFFICER) then
		chattype = "officer";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_CHANNEL) then
		chattype = "channel";
		channel = GetChannelName(objMStats.strAnnounceChannel);
	elseif (objMStats.intAnnounceType == MSTATS_AN_WHISPER) then
		chattype = "whisper";
		channel = objMStats.strAnnounceWhisper;
	end

	if (intCurrentFight > 0) then
		objTmpStats = objMStatsStatHistory[intCurrentFight].objMStatsStatistics;
		strTmpFightName = objMStatsStatHistory[intCurrentFight].strFightName;

		hours = floor(objMStatsStatHistory[intCurrentFight].intStartTime / 60);
		minutes = objMStatsStatHistory[intCurrentFight].intStartTime - (hours * 60);
		strTemp = MStats_MakeTimeString(hours,minutes);

		hours = floor(objMStatsStatHistory[intCurrentFight].intEndTime / 60);
		minutes = objMStatsStatHistory[intCurrentFight].intEndTime - (hours * 60);
		strTmpFightLen = strTemp .. " to " .. MStats_MakeTimeString(hours,minutes);
	elseif (objMStats.intLiveTotal == 1) then
		objTmpStats = objMStatsTotalStatistics;
		strTmpFightName = "현행 총합";
		strTmpFightLen = "";
	else
		objTmpStats = objMStatsStatistics;
		strTmpFightName = objMStats.strFightName;
		strTmpFightLen = "";
	end

	if not (strTmpFightName) then
		strTmpFightName = "알수없음";
	end

	strDataTypeString = MStats_GetDataTypeString();
	funcDataFunc = MStats_GetDataValueFunction();

	for k,v in ipairs(objTmpStats) do
		if (MStats_FilteredClass(objTmpStats[k].name, objTmpStats[k].class)) then
			intPercent = intPercent + funcDataFunc(objTmpStats[k]);
		end
	end

	SendChatMessage(MSTATS_MINNAEARSANDTAIL, chattype, nil, channel);
	strDashString = string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strDataTypeString)) / 2) .. strDataTypeString .. string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strDataTypeString)) / 2);
	if (string.len(strDashString) < 33) then
		strDashString = strDashString .. "-";
	end
	SendChatMessage(strDashString, chattype, nil, channel);

	intLineCounter = 1;
	for k,v in ipairs(objTmpStats) do
		if (MStats_FilteredClass(objTmpStats[k].name, objTmpStats[k].class)) then
			if (intLineCounter <= linecount) then
				local strPositionString;
				local strPercentString;
				local strCharString;

				if (funcDataFunc(objTmpStats[k]) == 0) then
					break;
				end

				strPositionString = tostring(intLineCounter);
				if (string.len(strPositionString) < 2) then
					strPositionString = "0" .. strPositionString;
				end

				strPercentString = tostring(floor((100 / intPercent) * (funcDataFunc(objTmpStats[k]) * 100)) / 100);
				if (string.len(strPercentString) < 5) then
					strPercentString = string.sub(MSTATS_SPACES, 1, (5 - string.len(strPercentString))) .. strPercentString;
				end

				strCharString = objTmpStats[k].name;
				if (string.len(strCharString) < string.len(MSTATS_SPACES)) then
					strCharString = strCharString .. string.sub(MSTATS_SPACES, 1, (string.len(MSTATS_SPACES) - string.len(strCharString)));
				end

				--'01 - 15.22% 123456789012 xxxxxxxx'
				SendChatMessage((strPositionString .. " - " .. strPercentString .. "% " .. strCharString .. funcDataFunc(objTmpStats[k])), chattype, nil, channel);
			end
			intLineCounter = intLineCounter + 1;
		end
	end

	if not (strTmpFightLen == "") then
		strDashString = string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strTmpFightLen)) / 2) .. strTmpFightLen .. string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strTmpFightLen)) / 2);
		if (string.len(strDashString) < 33) then
			strDashString = strDashString .. "-";
		end
		SendChatMessage(strDashString, chattype, nil, channel);
	end

	strDashString = string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strTmpFightName)) / 2) .. strTmpFightName .. string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strTmpFightName)) / 2);
	if (string.len(strDashString) < 33) then
		strDashString = strDashString .. "-";
	end
	SendChatMessage(strDashString, chattype, nil, channel);
end

function MStats_AnnounceIndividualStats(section)
	local objTmpStats, strTmpFightName, intCFIndex;
	local strPositionString, strCharString;
	local strDataTypeString, strDashString;
	local chattype, channel, k, v;
	local intPercent = 0;

	if (objMStats.intAnnounceType == MSTATS_AN_PARTY) then
		chattype = "party";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_RAID) then
		chattype = "raid";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_GUILD) then
		chattype = "guild";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_OFFICER) then
		chattype = "officer";
		channel = nil;
	elseif (objMStats.intAnnounceType == MSTATS_AN_CHANNEL) then
		chattype = "channel";
		channel = GetChannelName(objMStats.strAnnounceChannel);
	elseif (objMStats.intAnnounceType == MSTATS_AN_WHISPER) then
		chattype = "whisper";
		channel = objMStats.strAnnounceWhisper;
	end

	if (intCurrentFight > 0) then
		objTmpStats = objMStatsStatHistory[intCurrentFight].objMStatsStatistics;
		strTmpFightName = objMStatsStatHistory[intCurrentFight].strFightName;
	elseif (objMStats.intLiveTotal == 1) then
		objTmpStats = objMStatsTotalStatistics;
		strTmpFightName = "현행 총합";
	else
		objTmpStats = objMStatsStatistics;
		strTmpFightName = objMStats.strFightName;
	end

	if not (strTmpFightName) then
		strTmpFightName = "알수없음";
	end

	intCFIndex = 0;
	-- Find index of this player
	for k,v in ipairs(objTmpStats) do
		if (objTmpStats[k].name == strStatFighter) then
			intCFIndex = k;
			break;
		end
	end

	if (intCFIndex == 0) then
		return
	end

	if (section == 1) then
		strDataTypeString = strStatFighter .. " - 일반";
	elseif (section == 2) then
		strDataTypeString = strStatFighter .. " - 준 피해량";
	elseif (section == 3) then
		strDataTypeString = strStatFighter .. " - 받은 피해량";
	elseif (section == 4) then
		strDataTypeString = strStatFighter .. " - 기타";
	end

	SendChatMessage(MSTATS_MINNAEARSANDTAIL, chattype, nil, channel);
	strDashString = string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strDataTypeString)) / 2) .. strDataTypeString .. string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strDataTypeString)) / 2);
	if (string.len(strDashString) < 33) then
		strDashString = strDashString .. "-";
	end
	SendChatMessage(strDashString, chattype, nil, channel);

	if (section == 1) then
		local intTotalDamageHealing, intTotalHealing;

		-- General section
		intTotalDamageHealing = objTmpStats[intCFIndex].dmgdone + objTmpStats[intCFIndex].dmgtaken + objTmpStats[intCFIndex].dmgabsorbed + objTmpStats[intCFIndex].dmgtoself;
		intTotalHealing = objTmpStats[intCFIndex].healingdone + objTmpStats[intCFIndex].healingtaken;

		SendChatMessage(MStats_AnnounceMakeChatString("준 피해량", intTotalDamageHealing, objTmpStats[intCFIndex].dmgdone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 피해량", intTotalDamageHealing, objTmpStats[intCFIndex].dmgtaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 피해량", intTotalDamageHealing, objTmpStats[intCFIndex].dmgabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("자신의 피해량", intTotalDamageHealing, objTmpStats[intCFIndex].dmgtoself), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("치유량 (총합)", intTotalHealing, objTmpStats[intCFIndex].healingdone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("치유량 (최종)", intTotalHealing, objTmpStats[intCFIndex].healingdone - objTmpStats[intCFIndex].overhealingdone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("초과 치유량", intTotalHealing, objTmpStats[intCFIndex].overhealingdone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 치유량", intTotalHealing, objTmpStats[intCFIndex].healingtaken), chattype, nil, channel);
	elseif (section == 2) then
		local intTotalDamage;

		-- Damage done section
		intTotalDamage = objTmpStats[intCFIndex].physicaldone + objTmpStats[intCFIndex].firedone + objTmpStats[intCFIndex].frostdone + objTmpStats[intCFIndex].naturedone + objTmpStats[intCFIndex].shadowdone + objTmpStats[intCFIndex].arcanedone + objTmpStats[intCFIndex].holydone;

		SendChatMessage(MStats_AnnounceMakeChatString("준 물리피해", intTotalDamage, objTmpStats[intCFIndex].physicaldone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("준 화염피해", intTotalDamage, objTmpStats[intCFIndex].firedone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("준 냉기피해", intTotalDamage, objTmpStats[intCFIndex].frostdone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("준 자연피해", intTotalDamage, objTmpStats[intCFIndex].naturedone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("준 암흑피해", intTotalDamage, objTmpStats[intCFIndex].shadowdone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("준 비전피해", intTotalDamage, objTmpStats[intCFIndex].arcanedone), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("준 신성피해", intTotalDamage, objTmpStats[intCFIndex].holydone), chattype, nil, channel);
	elseif (section == 3) then
		local intTotalTaken;

		-- Damage taken section
		intTotalTaken = objTmpStats[intCFIndex].physicaltaken + objTmpStats[intCFIndex].physicalabsorbed + objTmpStats[intCFIndex].firetaken + objTmpStats[intCFIndex].fireabsorbed + objTmpStats[intCFIndex].frosttaken + objTmpStats[intCFIndex].frostabsorbed + objTmpStats[intCFIndex].naturetaken + objTmpStats[intCFIndex].natureabsorbed + objTmpStats[intCFIndex].shadowtaken + objTmpStats[intCFIndex].shadowabsorbed + objTmpStats[intCFIndex].arcanetaken + objTmpStats[intCFIndex].arcaneabsorbed + objTmpStats[intCFIndex].holytaken + objTmpStats[intCFIndex].holyabsorbed;

		SendChatMessage(MStats_AnnounceMakeChatString("받은 물리피해", intTotalTaken, objTmpStats[intCFIndex].physicaltaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 물리피해", intTotalTaken, objTmpStats[intCFIndex].physicalabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 화염피해", intTotalTaken, objTmpStats[intCFIndex].firetaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 화염피해", intTotalTaken, objTmpStats[intCFIndex].fireabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 냉기피해", intTotalTaken, objTmpStats[intCFIndex].frosttaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 냉기피해", intTotalTaken, objTmpStats[intCFIndex].frostabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 자연피해", intTotalTaken, objTmpStats[intCFIndex].naturetaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 자연피해", intTotalTaken, objTmpStats[intCFIndex].natureabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 암흑피해", intTotalTaken, objTmpStats[intCFIndex].shadowtaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 암흑피해", intTotalTaken, objTmpStats[intCFIndex].shadowabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 비전피해", intTotalTaken, objTmpStats[intCFIndex].arcanetaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 비전피해", intTotalTaken, objTmpStats[intCFIndex].arcaneabsorbed), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("받은 신성피해", intTotalTaken, objTmpStats[intCFIndex].holytaken), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("흡수된 신성피해", intTotalTaken, objTmpStats[intCFIndex].holyabsorbed), chattype, nil, channel);
	elseif (section == 4) then
		local intTotalHitHeal;

		-- Misc section
		intTotalHitHeal = objTmpStats[intCFIndex].highesthit + objTmpStats[intCFIndex].highestheal;

		SendChatMessage(MStats_AnnounceMakeChatString("치명타", (objTmpStats[intCFIndex].crits + objTmpStats[intCFIndex].noncrits), objTmpStats[intCFIndex].crits), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("실패", (objTmpStats[intCFIndex].hits + objTmpStats[intCFIndex].misses), objTmpStats[intCFIndex].misses), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("vs 실패", (objTmpStats[intCFIndex].vshits + objTmpStats[intCFIndex].vsmisses), objTmpStats[intCFIndex].vsmisses), chattype, nil, channel);

		if (objTmpStats[intCFIndex].deaths > 0) then
			SendChatMessage(MStats_AnnounceMakeChatString("죽음", objTmpStats[intCFIndex].deaths, objTmpStats[intCFIndex].deaths), chattype, nil, channel);
		else
			SendChatMessage(MStats_AnnounceMakeChatString("죽음", 100, 0), chattype, nil, channel);
		end

		if (objTmpStats[intCFIndex].decurses > 0 ) then
			SendChatMessage(MStats_AnnounceMakeChatString("해제", objTmpStats[intCFIndex].decurses, objTmpStats[intCFIndex].decurses), chattype, nil, channel);
		else
			SendChatMessage(MStats_AnnounceMakeChatString("해제", 100, 0), chattype, nil, channel);
		end

		SendChatMessage(MStats_AnnounceMakeChatString("최대 명중", intTotalHitHeal, objTmpStats[intCFIndex].highesthit), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("평균 명중", objTmpStats[intCFIndex].highesthit, MStats_Getaveragehit(objTmpStats[intCFIndex])), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("최대 치유", intTotalHitHeal, objTmpStats[intCFIndex].highestheal), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("평균 치유", objTmpStats[intCFIndex].highestheal, MStats_Getaverageheal(objTmpStats[intCFIndex])), chattype, nil, channel);

		SendChatMessage(MStats_AnnounceMakeChatString("피해량/초", objTmpStats[intCFIndex].highesthit, MStats_Getdamageps(objTmpStats[intCFIndex])), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("명중/초", objTmpStats[intCFIndex].hits, MStats_Gethitsps(objTmpStats[intCFIndex])), chattype, nil, channel);
		SendChatMessage(MStats_AnnounceMakeChatString("치유량/초", objTmpStats[intCFIndex].highestheal, MStats_Gethealsps(objTmpStats[intCFIndex])), chattype, nil, channel);
	end

	strDashString = string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strTmpFightName)) / 2) .. strTmpFightName .. string.sub(MSTATS_DASHES, 1, (string.len(MSTATS_DASHES) - string.len(strTmpFightName)) / 2);
	if (string.len(strDashString) < 33) then
		strDashString = strDashString .. "-";
	end
	SendChatMessage(strDashString, chattype, nil, channel);
end

function MStats_AnnounceMakeChatString(strCharString, intTotal, intValue)
	local strPercentString, intPercentage, intBarSize;

	if ((intValue > 0) and (intTotal > 0)) then
		strPercentString = tostring(floor((100 / intTotal) * (intValue * 100)) / 100);
	else
		strPercentString = "0";
	end

	if (string.len(strPercentString) < 5) then
		strPercentString = string.sub(MSTATS_SPACES, 1, (5 - string.len(strPercentString))) .. strPercentString;
	end

	if (string.len(strCharString) < string.len(MSTATS_SPACES)) then
		strCharString = strCharString .. string.sub(MSTATS_SPACES, 1, (string.len(MSTATS_SPACES) - string.len(strCharString)));
	end

	return (strPercentString .. "% " .. strCharString .. intValue);
end

function MStats_ExportCurrentFightUpwards(cumulative)
	local objTmpTotalStatistics, strExportString, funcDataFunc;
	local intTopFight, intBottomFight, intCellCount, intRowCount, i, j, k, v;

	-- Various checks to make sure there are data entries to actually export
	if (intCurrentFight == 0) then
		message("내보낼 것이 없습니다!");
		return;
	end

	intTopFight = intCurrentFight;
	repeat
		if (string.sub(objMStatsStatHistory[intTopFight].strFightName, 1, 5) ~= "TOTAL") then
			if (string.sub(objMStatsStatHistory[intTopFight].strFightName, 1, 8) ~= "SUBTOTAL") then
				-- Found first entry that isn't a total
				break;
			end
		end

		intTopFight = intTopFight - 1;
	until (intTopFight == 0);

	if (intTopFight == 0) then
		message("내보낼 것이 없습니다!");
		return;
	end

	intBottomFight = intTopFight;
	repeat
		if (string.sub(objMStatsStatHistory[intBottomFight].strFightName, 1, 5) == "TOTAL") then
			if (string.sub(objMStatsStatHistory[intBottomFight].strFightName, 1, 8) == "SUBTOTAL") then
				-- Found last entry that is a total
				intBottomFight = intBottomFight + 1;
				break;
			end
		end

		intBottomFight = intBottomFight - 1;
	until (intBottomFight == 0);

	if (intBottomFight == 0) then
		intBottomFight = 1;
	end

	funcDataFunc = MStats_GetDataValueFunction();

	-- First grab a master list of all participants for export
	objTmpTotalStatistics = MStats_MakeFightTotalObject(intTopFight, 0);

	strExportString = "<?xml version=\"1.0\"?><Report>";
	intRowCount = 1;

	-- Add first row (fightnames)
	intCellCount = 2;
	strExportString = strExportString .. "<Row>";
	strExportString = strExportString .. "<Heading>" .. MStats_GetDataTypeString() .. " vs</Heading>";
	for i = intBottomFight, intTopFight, 1 do
		strExportString = strExportString .. "<Fight" .. intCellCount .. ">" .. objMStatsStatHistory[i].strFightName .. "</Fight" .. intCellCount .. ">";
		intCellCount = intCellCount + 1;
	end
	strExportString = strExportString .. "</Row>";

--[[------------------------------------------
	-- Add second row (fight lengths in seconds)
	intCellCount = 2;
	strExportString = strExportString .. "<Row>";
	strExportString = strExportString .. "<Heading>Fight Length</Heading>";
	for i = intBottomFight, intTopFight, 1 do
		local intLength;

		if (objMStatsStatHistory[i].intEndTime < objMStatsStatHistory[i].intStartTime) then
			intLength = (objMStatsStatHistory[i].intEndTime + (24 * 60)) - objMStatsStatHistory[i].intStartTime;
		else
			intLength = objMStatsStatHistory[i].intEndTime - objMStatsStatHistory[i].intStartTime;
		end

		strExportString = strExportString .. "<Fight" .. intCellCount .. ">" .. (intLength * 60) .. "</Fight" .. intCellCount .. ">";
		intCellCount = intCellCount + 1;
	end
	strExportString = strExportString .. "</Row>";
--------------------------------------------]]

	-- Add a row for each entity
	for j = 1, #objTmpTotalStatistics, 1 do
		local value;

		value = 0;
		if (MStats_FilteredClass(objTmpTotalStatistics[j].name, objTmpTotalStatistics[j].class)) then
			intCellCount = 2;
			strExportString = strExportString .. "<Row>";
			strExportString = strExportString .. "<Heading>" .. objTmpTotalStatistics[j].name .. " (" .. objTmpTotalStatistics[j].class .. ")</Heading>";
			for i = intBottomFight, intTopFight, 1 do
				local index;

				index = nil;
				for k,v in ipairs(objMStatsStatHistory[i].objMStatsStatistics) do
					if (objMStatsStatHistory[i].objMStatsStatistics[k].name == objTmpTotalStatistics[j].name) then
						index = k;
						break;
					end
				end

				if (cumulative == 1) then
					if (index) then
						value = value + funcDataFunc(objMStatsStatHistory[i].objMStatsStatistics[index]);
					end
				else
					if (index) then
						value = funcDataFunc(objMStatsStatHistory[i].objMStatsStatistics[index]);
					else
						value = 0;
					end
				end

				strExportString = strExportString .. "<Fight" .. intCellCount .. ">" .. value .. "</Fight" .. intCellCount .. ">";
				intCellCount = intCellCount + 1;
			end
			strExportString = strExportString .. "</Row>";
		end
	end

	-- Close XML
	strExportString = strExportString .. "</Report>";

	-- Put XML into export frame and highlight
	MStatsExportFrameExportBox:SetText(strExportString);
	MStatsExportFrame:Show();
	MStatsExportFrameExportBox:HighlightText();
end

function MStats_IsBoss(mobname)
	local k, v;

	if (mobname) then
		for k,v in ipairs(strBossNames) do
			--if (strfind(mobname, strBossNames[k]) ~= nil) then
			if (mobname == strBossNames[k]) then
				return true;
			end
		end
	end

	return false;
end

function MStats_FilteredClass(player, class)
	if (objMStats.intFilterType == MSTATS_FT_ALL) then
		return true;
	elseif (objMStats.intFilterType == MSTATS_FT_WARRIOR) then
		return (class == "Warrior");
	elseif (objMStats.intFilterType == MSTATS_FT_ROGUE) then
		return (class == "Rogue");
	elseif (objMStats.intFilterType == MSTATS_FT_PALADIN) then
		return (class == "Paladin");
	elseif (objMStats.intFilterType == MSTATS_FT_PRIEST) then
		return (class == "Priest");
	elseif (objMStats.intFilterType == MSTATS_FT_SHAMAN) then
		return (class == "Shaman");
	elseif (objMStats.intFilterType == MSTATS_FT_DRUID) then
		return (class == "Druid");
	elseif (objMStats.intFilterType == MSTATS_FT_WARLOCK) then
		return (class == "Warlock");
	elseif (objMStats.intFilterType == MSTATS_FT_MAGE) then
		return (class == "Mage");
	elseif (objMStats.intFilterType == MSTATS_FT_HUNTER) then
		return (class == "Hunter");
	elseif (objMStats.intFilterType == MSTATS_FT_PET) then
		return (class == "Pet");
	elseif (objMStats.intFilterType == MSTATS_FT_YOURSELF) then
		return (player == UnitName("player"));
	elseif (objMStats.intFilterType == MSTATS_FT_MOBS) then
		return (class == "Mob");
	elseif (objMStats.intFilterType == MSTATS_FT_EXCLUDEMOBS) then
		return (class ~= "Mob");
	else
		return nil;
	end
end

function MStats_MakeTimeString(hours, minutes)
	local tmpStringHours, tmpStringMinutes;

	hours = hours + objMStats.intHourOffset;
	if (hours < 0) then
		hours = hours + 24;
	elseif (hours > 23) then
		hours = hours - 24;
	end

	if (hours < 10) then
		tmpStringHours = "0" .. hours .. ":";
	else
		tmpStringHours = hours .. ":";
	end

	if (minutes < 10) then
		tmpStringMinutes = tmpStringHours .. "0" .. minutes;
	else
		tmpStringMinutes = tmpStringHours .. minutes;
	end

	return tmpStringMinutes;
end

function MStats_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(MSTATS_NAMEANDVER .. ": " .. msg, 1,1,1);
end
